@@ -279,6 +279,7 @@ _cogl_atlas_create_texture (CoglAtlas *atlas,
                             int height)
 {
   CoglTexture2D *tex;
+  CoglError *ignore_error = NULL;
 
   _COGL_GET_CONTEXT (ctx, NULL);
 
@@ -299,7 +300,10 @@ _cogl_atlas_create_texture (CoglAtlas *atlas,
 
       tex = cogl_texture_2d_new_from_bitmap (clear_bmp,
                                              atlas->texture_format,
-                                             NULL);
+                                             &ignore_error);
+      if (!tex)
+        cogl_error_free (ignore_error);
+
       cogl_object_unref (clear_bmp);
 
       g_free (clear_data);
@@ -308,8 +312,13 @@ _cogl_atlas_create_texture (CoglAtlas *atlas,
     {
       tex = cogl_texture_2d_new_with_size (ctx,
                                            width, height,
-                                           atlas->texture_format,
-                                           NULL);
+                                           atlas->texture_format);
+      if (!cogl_texture_allocate (COGL_TEXTURE (tex), &ignore_error))
+        {
+          cogl_error_free (ignore_error);
+          cogl_object_unref (tex);
+          tex = NULL;
+        }
     }
 
   return tex;
@@ -533,19 +542,28 @@ _cogl_atlas_remove (CoglAtlas *atlas,
 };
 
 CoglTexture *
-_cogl_atlas_copy_rectangle (CoglAtlas        *atlas,
-                            unsigned int      x,
-                            unsigned int      y,
-                            unsigned int      width,
-                            unsigned int      height,
-                            CoglTextureFlags  flags,
-                            CoglPixelFormat   format)
+_cogl_atlas_copy_rectangle (CoglAtlas *atlas,
+                            int x,
+                            int y,
+                            int width,
+                            int height,
+                            CoglTextureFlags flags,
+                            CoglPixelFormat format)
 {
   CoglTexture *tex;
   CoglBlitData blit_data;
+  CoglError *ignore_error = NULL;
+
+  _COGL_GET_CONTEXT (ctx, NULL);
 
   /* Create a new texture at the right size */
   tex = cogl_texture_new_with_size (width, height, flags, format);
+  if (!cogl_texture_allocate (tex, &ignore_error))
+    {
+      cogl_error_free (ignore_error);
+      cogl_object_unref (tex);
+      return NULL;
+    }
 
   /* Blit the data out of the atlas to the new texture. If FBOs
      aren't available this will end up having to copy the entire
