@@ -32,10 +32,10 @@
 #ifndef __COGL_BUFFER_H__
 #define __COGL_BUFFER_H__
 
-#include <glib.h>
 #include <cogl/cogl-types.h>
+#include <cogl/cogl-error.h>
 
-G_BEGIN_DECLS
+COGL_BEGIN_DECLS
 
 /**
  * SECTION:cogl-buffer
@@ -63,6 +63,25 @@ G_BEGIN_DECLS
 
 typedef struct _CoglBuffer CoglBuffer;
 
+#define COGL_BUFFER_ERROR (_cogl_buffer_error_domain ())
+
+/**
+ * CoglBufferError:
+ * @COGL_BUFFER_ERROR_MAP: A buffer could not be mapped either
+ *    because the feature isn't supported or because a system
+ *    limitation was hit.
+ *
+ * Error enumeration for #CoglBuffer
+ *
+ * Stability: unstable
+ */
+typedef enum { /*< prefix=COGL_BUFFER_ERROR >*/
+  COGL_BUFFER_ERROR_MAP,
+} CoglBufferError;
+
+uint32_t
+_cogl_buffer_error_domain (void);
+
 /**
  * cogl_is_buffer:
  * @object: a buffer object
@@ -161,7 +180,13 @@ typedef enum { /*< prefix=COGL_BUFFER_ACCESS >*/
 /**
  * CoglBufferMapHint:
  * @COGL_BUFFER_MAP_HINT_DISCARD: Tells Cogl that you plan to replace
- *    all the buffer's contents.
+ *    all the buffer's contents. When this flag is used to map a
+ *    buffer, the entire contents of the buffer become undefined, even
+ *    if only a subregion of the buffer is mapped.
+ * @COGL_BUFFER_MAP_HINT_DISCARD_RANGE: Tells Cogl that you plan to
+ *    replace all the contents of the mapped region. The contents of
+ *    the region specified are undefined after this flag is used to
+ *    map a buffer.
  *
  * Hints to Cogl about how you are planning to modify the data once it
  * is mapped.
@@ -170,7 +195,8 @@ typedef enum { /*< prefix=COGL_BUFFER_ACCESS >*/
  * Stability: unstable
  */
 typedef enum { /*< prefix=COGL_BUFFER_MAP_HINT >*/
-  COGL_BUFFER_MAP_HINT_DISCARD = 1 << 0
+  COGL_BUFFER_MAP_HINT_DISCARD = 1 << 0,
+  COGL_BUFFER_MAP_HINT_DISCARD_RANGE = 1 << 1
 } CoglBufferMapHint;
 
 /**
@@ -180,7 +206,9 @@ typedef enum { /*< prefix=COGL_BUFFER_MAP_HINT >*/
  * @hints: A mask of #CoglBufferMapHint<!-- -->s that tell Cogl how
  *   the data will be modified once mapped.
  *
- * Maps the buffer into the application address space for direct access.
+ * Maps the buffer into the application address space for direct
+ * access. This is equivalent to calling cogl_buffer_map_range() with
+ * zero as the offset and the size of the entire buffer as the size.
  *
  * It is strongly recommended that you pass
  * %COGL_BUFFER_MAP_HINT_DISCARD as a hint if you are going to replace
@@ -199,9 +227,48 @@ typedef enum { /*< prefix=COGL_BUFFER_MAP_HINT >*/
  * Stability: unstable
  */
 void *
-cogl_buffer_map (CoglBuffer        *buffer,
-                 CoglBufferAccess   access,
-                 CoglBufferMapHint  hints);
+cogl_buffer_map (CoglBuffer *buffer,
+                 CoglBufferAccess access,
+                 CoglBufferMapHint hints);
+
+/**
+ * cogl_buffer_map_range:
+ * @buffer: a buffer object
+ * @offset: Offset within the buffer to start the mapping
+ * @size: The size of data to map
+ * @access: how the mapped buffer will be used by the application
+ * @hints: A mask of #CoglBufferMapHint<!-- -->s that tell Cogl how
+ *   the data will be modified once mapped.
+ * @error: A #CoglError for catching exceptional errors
+ *
+ * Maps a sub-region of the buffer into the application's address space
+ * for direct access.
+ *
+ * It is strongly recommended that you pass
+ * %COGL_BUFFER_MAP_HINT_DISCARD as a hint if you are going to replace
+ * all the buffer's data. This way if the buffer is currently being
+ * used by the GPU then the driver won't have to stall the CPU and
+ * wait for the hardware to finish because it can instead allocate a
+ * new buffer to map. You can pass
+ * %COGL_BUFFER_MAP_HINT_DISCARD_RANGE instead if you want the
+ * regions outside of the mapping to be retained.
+ *
+ * The behaviour is undefined if you access the buffer in a way
+ * conflicting with the @access mask you pass. It is also an error to
+ * release your last reference while the buffer is mapped.
+ *
+ * Return value: A pointer to the mapped memory or %NULL is the call fails
+ *
+ * Since: 2.0
+ * Stability: unstable
+ */
+void *
+cogl_buffer_map_range (CoglBuffer *buffer,
+                       size_t offset,
+                       size_t size,
+                       CoglBufferAccess access,
+                       CoglBufferMapHint hints,
+                       CoglError **error);
 
 /**
  * cogl_buffer_unmap:
@@ -237,6 +304,6 @@ cogl_buffer_set_data (CoglBuffer *buffer,
                       const void *data,
                       size_t size);
 
-G_END_DECLS
+COGL_END_DECLS
 
 #endif /* __COGL_BUFFER_H__ */
