@@ -24,7 +24,6 @@
 #ifndef __COGL_CONTEXT_PRIVATE_H
 #define __COGL_CONTEXT_PRIVATE_H
 
-#include "cogl-internal.h"
 #include "cogl-context.h"
 #include "cogl-winsys-private.h"
 #include "cogl-flags.h"
@@ -75,11 +74,17 @@ struct _CoglContext
   const CoglDriverVtable *driver_vtable;
   const CoglTextureDriver *texture_driver;
 
+  int glsl_major;
+  int glsl_minor;
+
   /* Features cache */
   unsigned long features[COGL_FLAGS_N_LONGS_FOR_SIZE (_COGL_N_FEATURE_IDS)];
   CoglFeatureFlags feature_flags; /* legacy/deprecated feature flags */
   CoglPrivateFeatureFlags private_feature_flags;
 
+  CoglBool needs_viewport_scissor_workaround;
+  CoglFramebuffer *viewport_scissor_workaround_framebuffer;
+
   CoglPipeline *default_pipeline;
   CoglPipelineLayer *default_layer_0;
   CoglPipelineLayer *default_layer_n;
@@ -134,6 +139,7 @@ struct _CoglContext
   CoglPipeline     *texture_pipeline; /* used for set_source_texture */
   GString          *codegen_header_buffer;
   GString          *codegen_source_buffer;
+  GString          *codegen_boilerplate_buffer;
   GList            *source_stack;
 
   int               legacy_state_set;
@@ -266,6 +272,7 @@ struct _CoglContext
      data */
   GByteArray       *buffer_map_fallback_array;
   CoglBool          buffer_map_fallback_in_use;
+  size_t            buffer_map_fallback_offset;
 
   CoglWinsysRectangleState rectangle_state;
 
@@ -336,7 +343,7 @@ _cogl_context_get_winsys (CoglContext *context);
  * return FALSE and set @error */
 CoglBool
 _cogl_context_update_features (CoglContext *context,
-                               GError **error);
+                               CoglError **error);
 
 /* Obtains the context and returns retval if NULL */
 #define _COGL_GET_CONTEXT(ctxvar, retval) \
@@ -353,7 +360,15 @@ void
 _cogl_context_set_current_modelview_entry (CoglContext *context,
                                            CoglMatrixEntry *entry);
 
-const char *
+/*
+ * _cogl_context_get_gl_extensions:
+ * @context: A CoglContext
+ *
+ * Return value: a NULL-terminated array of strings representing the
+ *   supported extensions by the current driver. This array is owned
+ *   by the caller and should be freed with g_strfreev().
+ */
+char **
 _cogl_context_get_gl_extensions (CoglContext *context);
 
 const char *
