@@ -26,13 +26,13 @@
 #endif
 
 #include "cogl-object.h"
-#include "cogl-internal.h"
 #include "cogl-private.h"
 #include "cogl-winsys-private.h"
 #include "winsys/cogl-winsys-stub-private.h"
 #include "cogl-profile.h"
 #include "cogl-util.h"
 #include "cogl-context-private.h"
+#include "cogl-util-gl-private.h"
 #include "cogl-display-private.h"
 #include "cogl-renderer-private.h"
 #include "cogl-journal-private.h"
@@ -49,8 +49,10 @@
 #include "cogl1-context.h"
 #include "cogl-gpu-info-private.h"
 #include "cogl-config-private.h"
+#include "cogl-error-private.h"
 
 #include <string.h>
+#include <stdlib.h>
 
 #ifdef HAVE_COGL_GL
 #include "cogl-pipeline-fragend-arbfp-private.h"
@@ -61,15 +63,6 @@
 #define GL_POINT_SPRITE 0x8861
 #endif
 
-#ifdef HAVE_COGL_GL
-extern const CoglTextureDriver _cogl_texture_driver_gl;
-extern const CoglDriverVtable _cogl_driver_gl;
-#endif
-#if defined (HAVE_COGL_GLES) || defined (HAVE_COGL_GLES2)
-extern const CoglTextureDriver _cogl_texture_driver_gles;
-extern const CoglDriverVtable _cogl_driver_gles;
-#endif
-
 static void _cogl_context_free (CoglContext *context);
 
 COGL_OBJECT_DEFINE (Context, context);
@@ -133,13 +126,14 @@ _cogl_context_get_winsys (CoglContext *context)
  */
 CoglContext *
 cogl_context_new (CoglDisplay *display,
-                  GError **error)
+                  CoglError **error)
 {
   CoglContext *context;
   GLubyte default_texture_data[] = { 0xff, 0xff, 0xff, 0x0 };
   CoglBitmap *default_texture_bitmap;
   const CoglWinsysVtable *winsys;
   int i;
+  CoglError *internal_error = NULL;
 
   _cogl_init ();
 
@@ -216,26 +210,10 @@ cogl_context_new (CoglDisplay *display,
      lot throughout Cogl */
   context->driver = display->renderer->driver;
 
-  switch (context->driver)
-    {
-#ifdef HAVE_COGL_GL
-    case COGL_DRIVER_GL:
-      context->driver_vtable = &_cogl_driver_gl;
-      context->texture_driver = &_cogl_texture_driver_gl;
-      break;
-#endif
-
-#if defined (HAVE_COGL_GLES) || defined (HAVE_COGL_GLES2)
-    case COGL_DRIVER_GLES1:
-    case COGL_DRIVER_GLES2:
-      context->driver_vtable = &_cogl_driver_gles;
-      context->texture_driver = &_cogl_texture_driver_gles;
-      break;
-#endif
-
-    default:
-      g_assert_not_reached ();
-    }
+  /* Again this is duplicated data, but it convenient to be able
+   * access these from the context. */
+  context->driver_vtable = display->renderer->driver_vtable;
+  context->texture_driver = display->renderer->texture_driver;
 
   winsys = _cogl_context_get_winsys (context);
   if (!winsys->context_init (context, error))
@@ -263,6 +241,22 @@ cogl_context_new (CoglDisplay *display,
   /* Initialise the driver specific state */
   _cogl_init_feature_overrides (context);
 
+  /* XXX: ONGOING BUG: Intel viewport scissor
+   *
+   * Intel gen6 drivers don't currently correctly handle offset
+   * viewports, since primitives aren't clipped within the bounds of
+   * the viewport.  To workaround this we push our own clip for the
+   * viewport that will use scissoring to ensure we clip as expected.
+   *
+   * TODO: file a bug upstream!
+   */
+  if (context->gpu.driver_package == COGL_GPU_INFO_DRIVER_PACKAGE_MESA &&
+      context->gpu.architecture == COGL_GPU_INFO_ARCHITECTURE_SANDYBRIDGE &&
+      !getenv ("COGL_DISABLE_INTEL_VIEWPORT_SCISSORT_WORKAROUND"))
+    context->needs_viewport_scissor_workaround = TRUE;
+  else
+    context->needs_viewport_scissor_workaround = FALSE;
+
   context->sampler_cache = _cogl_sampler_cache_new (context);
 
   _cogl_pipeline_init_default_pipeline ();
@@ -284,10 +278,13 @@ cogl_context_new (CoglDisplay *display,
   context->texture_units =
     g_array_new (FALSE, FALSE, sizeof (CoglTextureUnit));
 
-  /* See cogl-pipeline.c for more details about why we leave texture unit 1
-   * active by default... */
-  context->active_texture_unit = 1;
-  GE (context, glActiveTexture (GL_TEXTURE1));
+  if ((context->private_feature_flags & COGL_PRIVATE_FEATURE_ANY_GL))
+    {
+      /* See cogl-pipeline.c for more details about why we leave texture unit 1
+       * active by default... */
+      context->active_texture_unit = 1;
+      GE (context, glActiveTexture (GL_TEXTURE1));
+    }
 
   context->legacy_fog_state.enabled = FALSE;
 
@@ -296,6 +293,7 @@ cogl_context_new (CoglDisplay *display,
   context->texture_pipeline = cogl_pipeline_new (context);
   context->codegen_header_buffer = g_string_new ("");
   context->codegen_source_buffer = g_string_new ("");
+  context->codegen_boilerplate_buffer = g_string_new ("");
   context->source_stack = NULL;
 
   context->legacy_state_set = 0;
@@ -350,7 +348,7 @@ cogl_context_new (CoglDisplay *display,
 
   context->legacy_depth_test_enabled = FALSE;
 
-  context->pipeline_cache = cogl_pipeline_cache_new ();
+  context->pipeline_cache = _cogl_pipeline_cache_new ();
 
   for (i = 0; i < COGL_BUFFER_BIND_TARGET_COUNT; i++)
     context->current_buffer[i] = NULL;
@@ -386,7 +384,7 @@ cogl_context_new (CoglDisplay *display,
   context->blit_texture_pipeline = NULL;
 
 #if defined (HAVE_COGL_GL) || defined (HAVE_COGL_GLES)
-  if (context->driver != COGL_DRIVER_GLES2)
+  if ((context->private_feature_flags & COGL_PRIVATE_FEATURE_ALPHA_TEST))
     /* The default for GL_ALPHA_TEST is to always pass which is equivalent to
      * the test being disabled therefore we assume that for all drivers there
      * will be no performance impact if we always leave the test enabled which
@@ -396,6 +394,22 @@ cogl_context_new (CoglDisplay *display,
     GE (context, glEnable (GL_ALPHA_TEST));
 #endif
 
+#if defined (HAVE_COGL_GL)
+  if ((context->driver == COGL_DRIVER_GL3))
+    {
+      GLuint vertex_array;
+
+      /* In a forward compatible context, GL 3 doesn't support rendering
+       * using the default vertex array object. Cogl doesn't use vertex
+       * array objects yet so for now we just create a dummy array
+       * object that we will use as our own default object. Eventually
+       * it could be good to attach the vertex array objects to
+       * CoglPrimitives */
+      context->glGenVertexArrays (1, &vertex_array);
+      context->glBindVertexArray (vertex_array);
+    }
+#endif
+
   context->current_modelview_entry = NULL;
   context->current_projection_entry = NULL;
   _cogl_matrix_entry_identity_init (&context->identity_entry);
@@ -415,18 +429,26 @@ cogl_context_new (CoglDisplay *display,
                                      /* internal format */
                                      COGL_PIXEL_FORMAT_RGBA_8888_PRE,
                                      NULL);
-  /* If 3D or rectangle textures aren't supported then these should
-     just silently return NULL */
+
+  /* If 3D or rectangle textures aren't supported then these will
+   * return errors that we can simply ignore. */
+  internal_error = NULL;
   context->default_gl_texture_3d_tex =
     cogl_texture_3d_new_from_bitmap (default_texture_bitmap,
                                      1, /* height */
                                      1, /* depth */
                                      COGL_PIXEL_FORMAT_RGBA_8888_PRE,
-                                     NULL);
+                                     &internal_error);
+  if (internal_error)
+    cogl_error_free (internal_error);
+
+  internal_error = NULL;
   context->default_gl_texture_rect_tex =
     cogl_texture_rectangle_new_from_bitmap (default_texture_bitmap,
                                             COGL_PIXEL_FORMAT_RGBA_8888_PRE,
-                                            NULL);
+                                            &internal_error);
+  if (internal_error)
+    cogl_error_free (internal_error);
 
   cogl_object_unref (default_texture_bitmap);
 
@@ -439,13 +461,13 @@ cogl_context_new (CoglDisplay *display,
   context->buffer_map_fallback_in_use = FALSE;
 
   /* As far as I can tell, GL_POINT_SPRITE doesn't have any effect
-     unless GL_COORD_REPLACE is enabled for an individual
-     layer. Therefore it seems like it should be ok to just leave it
-     enabled all the time instead of having to have a set property on
-     each pipeline to track whether any layers have point sprite
-     coords enabled. We don't need to do this for GLES2 because point
+     unless GL_COORD_REPLACE is enabled for an individual layer.
+     Therefore it seems like it should be ok to just leave it enabled
+     all the time instead of having to have a set property on each
+     pipeline to track whether any layers have point sprite coords
+     enabled. We don't need to do this for GL3 or GLES2 because point
      sprites are handled using a builtin varying in the shader. */
-  if (context->driver != COGL_DRIVER_GLES2 &&
+  if ((context->private_feature_flags & COGL_PRIVATE_FEATURE_FIXED_FUNCTION) &&
       cogl_has_feature (context, COGL_FEATURE_ID_POINT_SPRITE))
     GE (context, glEnable (GL_POINT_SPRITE));
 
@@ -529,13 +551,13 @@ _cogl_context_free (CoglContext *context)
   g_slist_free (context->buffer_types);
 
   if (context->current_modelview_entry)
-    _cogl_matrix_entry_unref (context->current_modelview_entry);
+    cogl_matrix_entry_unref (context->current_modelview_entry);
   if (context->current_projection_entry)
-    _cogl_matrix_entry_unref (context->current_projection_entry);
+    cogl_matrix_entry_unref (context->current_projection_entry);
   _cogl_matrix_entry_cache_destroy (&context->builtin_flushed_projection);
   _cogl_matrix_entry_cache_destroy (&context->builtin_flushed_modelview);
 
-  cogl_pipeline_cache_free (context->pipeline_cache);
+  _cogl_pipeline_cache_free (context->pipeline_cache);
 
   _cogl_sampler_cache_free (context->sampler_cache);
 
@@ -557,7 +579,7 @@ _cogl_context_free (CoglContext *context)
 CoglContext *
 _cogl_context_get_default (void)
 {
-  GError *error = NULL;
+  CoglError *error = NULL;
   /* Create if doesn't exist yet */
   if (_cogl_context == NULL)
     {
@@ -566,7 +588,7 @@ _cogl_context_get_default (void)
         {
           g_warning ("Failed to create default context: %s",
                      error->message);
-          g_error_free (error);
+          cogl_error_free (error);
         }
     }
 
@@ -594,7 +616,7 @@ cogl_egl_context_get_egl_display (CoglContext *context)
 
 CoglBool
 _cogl_context_update_features (CoglContext *context,
-                               GError **error)
+                               CoglError **error)
 {
   return context->driver_vtable->update_features (context, error);
 }
@@ -603,9 +625,9 @@ void
 _cogl_context_set_current_projection_entry (CoglContext *context,
                                             CoglMatrixEntry *entry)
 {
-  _cogl_matrix_entry_ref (entry);
+  cogl_matrix_entry_ref (entry);
   if (context->current_projection_entry)
-    _cogl_matrix_entry_unref (context->current_projection_entry);
+    cogl_matrix_entry_unref (context->current_projection_entry);
   context->current_projection_entry = entry;
 }
 
@@ -613,44 +635,51 @@ void
 _cogl_context_set_current_modelview_entry (CoglContext *context,
                                            CoglMatrixEntry *entry)
 {
-  _cogl_matrix_entry_ref (entry);
+  cogl_matrix_entry_ref (entry);
   if (context->current_modelview_entry)
-    _cogl_matrix_entry_unref (context->current_modelview_entry);
+    cogl_matrix_entry_unref (context->current_modelview_entry);
   context->current_modelview_entry = entry;
 }
 
-const char *
+char **
 _cogl_context_get_gl_extensions (CoglContext *context)
 {
   const char *env_disabled_extensions;
+  char **ret;
+
+  /* In GL 3, querying GL_EXTENSIONS is deprecated so we have to build
+   * the array using glGetStringi instead */
+  if (context->driver == COGL_DRIVER_GL3)
+    {
+      int num_extensions, i;
+
+      context->glGetIntegerv (GL_NUM_EXTENSIONS, &num_extensions);
+
+      ret = g_malloc (sizeof (char *) * (num_extensions + 1));
+
+      for (i = 0; i < num_extensions; i++)
+        {
+          const char *ext =
+            (const char *) context->glGetStringi (GL_EXTENSIONS, i);
+          ret[i] = g_strdup (ext);
+        }
+
+      ret[num_extensions] = NULL;
+    }
+  else
+    {
+      const char *all_extensions =
+        (const char *) context->glGetString (GL_EXTENSIONS);
+
+      ret = g_strsplit (all_extensions, " ", 0 /* max tokens */);
+    }
 
   if ((env_disabled_extensions = g_getenv ("COGL_DISABLE_GL_EXTENSIONS"))
       || _cogl_config_disable_gl_extensions)
     {
-      static CoglUserDataKey extensions_key;
-      const char *enabled_extensions;
-      char **split_enabled_extensions;
       char **split_env_disabled_extensions;
       char **split_conf_disabled_extensions;
-      char **e;
-      GString *result;
-
-      /* We need to return a const string so we'll attach the results
-       * to the CoglContext to avoid leaking the generated string.
-       * This string is only used rarely so we are using
-       * cogl_object_set_user_data instead of adding an explicit
-       * member to CoglContext to avoid making the struct bigger */
-
-      enabled_extensions =
-        cogl_object_get_user_data (COGL_OBJECT (context), &extensions_key);
-      if (enabled_extensions)
-        return enabled_extensions;
-
-      enabled_extensions = (const char *) context->glGetString (GL_EXTENSIONS);
-
-      split_enabled_extensions = g_strsplit (enabled_extensions,
-                                             " ",
-                                             0 /* no max tokens */);
+      char **src, **dst;
 
       if (env_disabled_extensions)
         split_env_disabled_extensions =
@@ -668,46 +697,38 @@ _cogl_context_get_gl_extensions (CoglContext *context)
       else
         split_conf_disabled_extensions = NULL;
 
-      result = g_string_new (NULL);
-
-      for (e = split_enabled_extensions; *e; e++)
+      for (dst = ret, src = ret;
+           *src;
+           src++)
         {
           char **d;
 
           if (split_env_disabled_extensions)
             for (d = split_env_disabled_extensions; *d; d++)
-              if (!strcmp (*e, *d))
+              if (!strcmp (*src, *d))
                 goto disabled;
           if (split_conf_disabled_extensions)
             for (d = split_conf_disabled_extensions; *d; d++)
-              if (!strcmp (*e, *d))
+              if (!strcmp (*src, *d))
                 goto disabled;
 
-          if (result->len > 0)
-            g_string_append_c (result, ' ');
-          g_string_append (result, *e);
+          *(dst++) = *src;
+          continue;
 
         disabled:
+          g_free (*src);
           continue;
         }
 
-      enabled_extensions = g_string_free (result, FALSE);
+      *dst = NULL;
 
-      g_strfreev (split_enabled_extensions);
       if (split_env_disabled_extensions)
         g_strfreev (split_env_disabled_extensions);
       if (split_conf_disabled_extensions)
         g_strfreev (split_conf_disabled_extensions);
-
-      cogl_object_set_user_data (COGL_OBJECT (context),
-                                 &extensions_key,
-                                 (void *) enabled_extensions,
-                                 (CoglUserDataDestroyCallback) g_free);
-
-      return enabled_extensions;
     }
-  else
-    return (const char *) context->glGetString (GL_EXTENSIONS);
+
+  return ret;
 }
 
 const char *
