@@ -31,9 +31,9 @@ typedef struct _CoglGLXCachedConfig
 {
   /* This will be -1 if there is no cached config in this slot */
   int depth;
-  gboolean found;
+  CoglBool found;
   GLXFBConfig fb_config;
-  gboolean can_mipmap;
+  CoglBool can_mipmap;
 } CoglGLXCachedConfig;
 
 #define COGL_GLX_N_CACHED_CONFIGS 3
@@ -42,15 +42,16 @@ typedef struct _CoglGLXDisplay
 {
   CoglGLXCachedConfig glx_cached_configs[COGL_GLX_N_CACHED_CONFIGS];
 
-  gboolean found_fbconfig;
-  gboolean fbconfig_has_rgba_visual;
+  CoglBool found_fbconfig;
+  CoglBool fbconfig_has_rgba_visual;
   GLXFBConfig fbconfig;
 
   /* Single context for all wins */
   GLXContext glx_context;
   GLXWindow dummy_glxwin;
   Window dummy_xwin;
-  gboolean pending_swap_notify;
+  CoglBool pending_swap_notify;
+  CoglBool pending_resize_notify;
 } CoglGLXDisplay;
 
 #endif /* __COGL_DISPLAY_GLX_PRIVATE_H */
