@@ -26,7 +26,6 @@
 #endif
 
 #include "cogl-debug.h"
-#include "cogl-internal.h"
 #include "cogl-context-private.h"
 #include "cogl-journal-private.h"
 #include "cogl-texture-private.h"
@@ -94,6 +93,8 @@
 
 typedef struct _CoglJournalFlushState
 {
+  CoglContext *ctx;
+
   CoglJournal *journal;
 
   CoglAttributeBuffer *attribute_buffer;
@@ -162,8 +163,6 @@ _cogl_journal_dump_logged_quad (uint8_t *data, int n_layers)
   size_t stride = GET_JOURNAL_ARRAY_STRIDE_FOR_N_LAYERS (n_layers);
   int i;
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
   g_print ("n_layers = %d; rgba=0x%02X%02X%02X%02X\n",
            n_layers, data[0], data[1], data[2], data[3]);
 
@@ -191,8 +190,6 @@ _cogl_journal_dump_quad_vertices (uint8_t *data, int n_layers)
   size_t stride = GET_JOURNAL_VB_STRIDE_FOR_N_LAYERS (n_layers);
   int i;
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
   g_print ("n_layers = %d; stride = %d; pos stride = %d; color stride = %d; "
            "tex stride = %d; stride in bytes = %d\n",
            n_layers, (int)stride, POS_STRIDE, COLOR_STRIDE,
@@ -273,6 +270,7 @@ _cogl_journal_flush_modelview_and_entries (CoglJournalEntry *batch_start,
                                            void             *data)
 {
   CoglJournalFlushState *state = data;
+  CoglContext *ctx = state->ctx;
   CoglFramebuffer *framebuffer = state->journal->framebuffer;
   CoglAttribute **attributes;
   CoglDrawFlags draw_flags = (COGL_DRAW_SKIP_JOURNAL_FLUSH |
@@ -286,8 +284,6 @@ _cogl_journal_flush_modelview_and_entries (CoglJournalEntry *batch_start,
                      "The time spent flushing modelview + entries",
                      0 /* no application private data */);
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
   COGL_TIMER_START (_cogl_uprof_context, time_flush_modelview_and_entries);
 
   if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_BATCHING)))
@@ -303,7 +299,7 @@ _cogl_journal_flush_modelview_and_entries (CoglJournalEntry *batch_start,
     draw_flags |= COGL_DRAW_COLOR_ATTRIBUTE_IS_OPAQUE;
 
 #ifdef HAVE_COGL_GL
-  if (ctx->driver == COGL_DRIVER_GL)
+  if ((ctx->private_feature_flags & COGL_PRIVATE_FEATURE_QUADS))
     {
       /* XXX: it's rather evil that we sneak in the GL_QUADS enum here... */
       _cogl_framebuffer_draw_attributes (framebuffer,
@@ -356,10 +352,8 @@ _cogl_journal_flush_modelview_and_entries (CoglJournalEntry *batch_start,
       int i;
       CoglAttribute *loop_attributes[1];
 
-      _COGL_GET_CONTEXT (ctxt, NO_RETVAL);
-
       if (outline == NULL)
-        outline = cogl_pipeline_new (ctxt);
+        outline = cogl_pipeline_new (ctx);
 
       /* The least significant three bits represent the three
          components so that the order of colours goes red, green,
@@ -368,13 +362,13 @@ _cogl_journal_flush_modelview_and_entries (CoglJournalEntry *batch_start,
          in the order 0xff, 0xcc, 0x99, and 0x66. This gives a total
          of 24 colours. If there are more than 24 batches on the stage
          then it will wrap around */
-      color_intensity = 0xff - 0x33 * (ctxt->journal_rectangles_color >> 3);
+      color_intensity = 0xff - 0x33 * (ctx->journal_rectangles_color >> 3);
       cogl_pipeline_set_color4ub (outline,
-                                  (ctxt->journal_rectangles_color & 1) ?
+                                  (ctx->journal_rectangles_color & 1) ?
                                   color_intensity : 0,
-                                  (ctxt->journal_rectangles_color & 2) ?
+                                  (ctx->journal_rectangles_color & 2) ?
                                   color_intensity : 0,
-                                  (ctxt->journal_rectangles_color & 4) ?
+                                  (ctx->journal_rectangles_color & 4) ?
                                   color_intensity : 0,
                                   0xff);
 
@@ -390,11 +384,11 @@ _cogl_journal_flush_modelview_and_entries (CoglJournalEntry *batch_start,
 
       /* Go to the next color */
       do
-        ctxt->journal_rectangles_color = ((ctxt->journal_rectangles_color + 1) &
-                                          ((1 << 5) - 1));
+        ctx->journal_rectangles_color = ((ctx->journal_rectangles_color + 1) &
+                                         ((1 << 5) - 1));
       /* We don't want to use black or white */
-      while ((ctxt->journal_rectangles_color & 0x07) == 0
-             || (ctxt->journal_rectangles_color & 0x07) == 0x07);
+      while ((ctx->journal_rectangles_color & 0x07) == 0
+             || (ctx->journal_rectangles_color & 0x07) == 0x07);
     }
 
   state->current_vertex += (4 * batch_len);
@@ -424,8 +418,6 @@ _cogl_journal_flush_pipeline_and_entries (CoglJournalEntry *batch_start,
                      "The time spent flushing pipeline + entries",
                      0 /* no application private data */);
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
   COGL_TIMER_START (_cogl_uprof_context, time_flush_pipeline_entries);
 
   if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_BATCHING)))
@@ -465,6 +457,67 @@ compare_entry_pipelines (CoglJournalEntry *entry0, CoglJournalEntry *entry1)
     return FALSE;
 }
 
+typedef struct _CreateAttributeState
+{
+  int current;
+  CoglJournalFlushState *flush_state;
+} CreateAttributeState;
+
+static CoglBool
+create_attribute_cb (CoglPipeline *pipeline,
+                     int layer_number,
+                     void *user_data)
+{
+  CreateAttributeState *state = user_data;
+  CoglJournalFlushState *flush_state = state->flush_state;
+  CoglAttribute **attribute_entry =
+    &g_array_index (flush_state->attributes,
+                    CoglAttribute *,
+                    state->current + 2);
+  const char *names[] = {
+      "cogl_tex_coord0_in",
+      "cogl_tex_coord1_in",
+      "cogl_tex_coord2_in",
+      "cogl_tex_coord3_in",
+      "cogl_tex_coord4_in",
+      "cogl_tex_coord5_in",
+      "cogl_tex_coord6_in",
+      "cogl_tex_coord7_in"
+  };
+  char *name;
+
+  /* XXX NB:
+   * Our journal's vertex data is arranged as follows:
+   * 4 vertices per quad:
+   *    2 or 3 floats per position (3 when doing software transforms)
+   *    4 RGBA bytes,
+   *    2 floats per tex coord * n_layers
+   * (though n_layers may be padded; see definition of
+   *  GET_JOURNAL_VB_STRIDE_FOR_N_LAYERS for details)
+   */
+  name = layer_number < 8 ? (char *)names[layer_number] :
+    g_strdup_printf ("cogl_tex_coord%d_in", layer_number);
+
+  /* XXX: it may be worth having some form of static initializer for
+   * attributes... */
+  *attribute_entry =
+    cogl_attribute_new (flush_state->attribute_buffer,
+                        name,
+                        flush_state->stride,
+                        flush_state->array_offset +
+                        (POS_STRIDE + COLOR_STRIDE) * 4 +
+                        TEX_STRIDE * 4 * state->current,
+                        2,
+                        COGL_ATTRIBUTE_TYPE_FLOAT);
+
+  if (layer_number >= 8)
+    g_free (name);
+
+  state->current++;
+
+  return TRUE;
+}
+
 /* Since the stride may not reflect the number of texture layers in use
  * (due to padding) we deal with texture coordinate offsets separately
  * from vertex and color offsets... */
@@ -475,7 +528,8 @@ _cogl_journal_flush_texcoord_vbo_offsets_and_entries (
                                           void             *data)
 {
   CoglJournalFlushState *state = data;
-  int                    i;
+  CreateAttributeState create_attrib_state;
+  int i;
   COGL_STATIC_TIMER (time_flush_texcoord_pipeline_entries,
                      "flush: vbo+texcoords+pipeline+entries", /* parent */
                      "flush: texcoords+pipeline+entries",
@@ -483,8 +537,6 @@ _cogl_journal_flush_texcoord_vbo_offsets_and_entries (
                      "+ entries",
                      0 /* no application private data */);
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
   COGL_TIMER_START (_cogl_uprof_context, time_flush_texcoord_pipeline_entries);
 
   /* NB: attributes 0 and 1 are position and color */
@@ -494,49 +546,12 @@ _cogl_journal_flush_texcoord_vbo_offsets_and_entries (
 
   g_array_set_size (state->attributes, batch_start->n_layers + 2);
 
-  for (i = 0; i < batch_start->n_layers; i++)
-    {
-      CoglAttribute **attribute_entry =
-        &g_array_index (state->attributes, CoglAttribute *, i + 2);
-      const char *names[] = {
-          "cogl_tex_coord0_in",
-          "cogl_tex_coord1_in",
-          "cogl_tex_coord2_in",
-          "cogl_tex_coord3_in",
-          "cogl_tex_coord4_in",
-          "cogl_tex_coord5_in",
-          "cogl_tex_coord6_in",
-          "cogl_tex_coord7_in"
-      };
-      char *name;
+  create_attrib_state.current = 0;
+  create_attrib_state.flush_state = state;
 
-      /* XXX NB:
-       * Our journal's vertex data is arranged as follows:
-       * 4 vertices per quad:
-       *    2 or 3 floats per position (3 when doing software transforms)
-       *    4 RGBA bytes,
-       *    2 floats per tex coord * n_layers
-       * (though n_layers may be padded; see definition of
-       *  GET_JOURNAL_VB_STRIDE_FOR_N_LAYERS for details)
-       */
-      name = i < 8 ? (char *)names[i] :
-        g_strdup_printf ("cogl_tex_coord%d_in", i);
-
-      /* XXX: it may be worth having some form of static initializer for
-       * attributes... */
-      *attribute_entry =
-        cogl_attribute_new (state->attribute_buffer,
-                            name,
-                            state->stride,
-                            state->array_offset +
-                            (POS_STRIDE + COLOR_STRIDE) * 4 +
-                            TEX_STRIDE * 4 * i,
-                            2,
-                            COGL_ATTRIBUTE_TYPE_FLOAT);
-
-      if (i >= 8)
-        g_free (name);
-    }
+  cogl_pipeline_foreach_layer (batch_start->pipeline,
+                               create_attribute_cb,
+                               &create_attrib_state);
 
   batch_and_call (batch_start,
                   batch_len,
@@ -547,9 +562,9 @@ _cogl_journal_flush_texcoord_vbo_offsets_and_entries (
 }
 
 static CoglBool
-compare_entry_n_layers (CoglJournalEntry *entry0, CoglJournalEntry *entry1)
+compare_entry_layer_numbers (CoglJournalEntry *entry0, CoglJournalEntry *entry1)
 {
-  if (entry0->n_layers == entry1->n_layers)
+  if (_cogl_pipeline_layer_numbers_equal (entry0->pipeline, entry1->pipeline))
     return TRUE;
   else
     return FALSE;
@@ -615,7 +630,7 @@ _cogl_journal_flush_vbo_offsets_and_entries (CoglJournalEntry *batch_start,
                         4,
                         COGL_ATTRIBUTE_TYPE_UNSIGNED_BYTE);
 
-  if (ctx->driver != COGL_DRIVER_GL)
+  if (!(ctx->private_feature_flags & COGL_PRIVATE_FEATURE_QUADS))
     state->indices = cogl_get_rectangle_indices (ctx, batch_len);
 
   /* We only create new Attributes when the stride within the
@@ -633,8 +648,9 @@ _cogl_journal_flush_vbo_offsets_and_entries (CoglJournalEntry *batch_start,
       /* Mapping a buffer for read is probably a really bad thing to
          do but this will only happen during debugging so it probably
          doesn't matter */
-      verts = ((uint8_t *)cogl_buffer_map (COGL_BUFFER (state->attribute_buffer),
-                                          COGL_BUFFER_ACCESS_READ, 0) +
+      verts = ((uint8_t *)_cogl_buffer_map (COGL_BUFFER (state->attribute_buffer),
+                                            COGL_BUFFER_ACCESS_READ, 0,
+                                            NULL) +
                state->array_offset);
 
       _cogl_journal_dump_quad_batch (verts,
@@ -646,7 +662,7 @@ _cogl_journal_flush_vbo_offsets_and_entries (CoglJournalEntry *batch_start,
 
   batch_and_call (batch_start,
                   batch_len,
-                  compare_entry_n_layers,
+                  compare_entry_layer_numbers,
                   _cogl_journal_flush_texcoord_vbo_offsets_and_entries,
                   data);
 
@@ -786,7 +802,7 @@ can_software_clip_entry (CoglJournalEntry *journal_entry,
       clip_rect = (CoglClipStackRect *) clip_entry;
 
       modelview_entry = journal_entry->modelview_entry;
-      if (!_cogl_matrix_entry_calculate_translation (clip_rect->matrix_entry,
+      if (!cogl_matrix_entry_calculate_translation (clip_rect->matrix_entry,
                                                      modelview_entry,
                                                      &tx, &ty, &tz))
         return FALSE;
@@ -922,12 +938,11 @@ maybe_software_clip_entries (CoglJournalEntry      *batch_start,
                              int                    batch_len,
                              CoglJournalFlushState *state)
 {
-  CoglJournal *journal = state->journal;
+  CoglContext *ctx;
+  CoglJournal *journal;
   CoglClipStack *clip_stack, *clip_entry;
   int entry_num;
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
   /* This tries to find cases where the entry is logged with a clip
      but it would be faster to modify the vertex and texture
      coordinates rather than flush the clip so that it can batch
@@ -949,6 +964,9 @@ maybe_software_clip_entries (CoglJournalEntry      *batch_start,
     if (clip_entry->type != COGL_CLIP_STACK_RECT)
       return;
 
+  ctx = state->ctx;
+  journal = state->journal;
+
   /* This scratch buffer is used to store the translation for each
      entry in the journal. We store it in a separate buffer because
      it's expensive to calculate but at this point we still don't know
@@ -1003,8 +1021,6 @@ _cogl_journal_maybe_software_clip_entries (CoglJournalEntry *batch_start,
                      "Time spent software clipping",
                      0 /* no application private data */);
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
   COGL_TIMER_START (_cogl_uprof_context,
                     time_check_software_clip);
 
@@ -1027,27 +1043,26 @@ create_attribute_buffer (CoglJournal *journal,
                          size_t n_bytes)
 {
   CoglAttributeBuffer *vbo;
-
-  _COGL_GET_CONTEXT (ctx, NULL);
+  CoglContext *ctx = journal->framebuffer->context;
 
   /* If CoglBuffers are being emulated with malloc then there's not
      really any point in using the pool so we'll just allocate the
      buffer directly */
   if (!(ctx->private_feature_flags & COGL_PRIVATE_FEATURE_VBOS))
-    return cogl_attribute_buffer_new (ctx, n_bytes, NULL);
+    return cogl_attribute_buffer_new_with_size (ctx, n_bytes);
 
   vbo = journal->vbo_pool[journal->next_vbo_in_pool];
 
   if (vbo == NULL)
     {
-      vbo = cogl_attribute_buffer_new (ctx, n_bytes, NULL);
+      vbo = cogl_attribute_buffer_new_with_size (ctx, n_bytes);
       journal->vbo_pool[journal->next_vbo_in_pool] = vbo;
     }
   else if (cogl_buffer_get_size (COGL_BUFFER (vbo)) < n_bytes)
     {
       /* If the buffer is too small then we'll just recreate it */
       cogl_object_unref (vbo);
-      vbo = cogl_attribute_buffer_new (ctx, n_bytes, NULL);
+      vbo = cogl_attribute_buffer_new_with_size (ctx, n_bytes);
       journal->vbo_pool[journal->next_vbo_in_pool] = vbo;
     }
 
@@ -1079,7 +1094,9 @@ upload_vertices (CoglJournal *journal,
   buffer = COGL_BUFFER (attribute_buffer);
   cogl_buffer_set_update_hint (buffer, COGL_BUFFER_UPDATE_HINT_STATIC);
 
-  vout = _cogl_buffer_map_for_fill_or_fallback (buffer);
+  vout = _cogl_buffer_map_range_for_fill_or_fallback (buffer,
+                                                      0, /* offset */
+                                                      needed_vbo_len * 4);
   vin = &g_array_index (vertices, float, 0);
 
   /* Expand the number of vertices from 2 to 4 while uploading */
@@ -1120,7 +1137,7 @@ upload_vertices (CoglJournal *journal,
           v[7] = vin[1];
 
           if (entry->modelview_entry != last_modelview_entry)
-            _cogl_matrix_entry_get (entry->modelview_entry, &modelview);
+            cogl_matrix_entry_get (entry->modelview_entry, &modelview);
           cogl_matrix_transform_points (&modelview,
                                         2, /* n_components */
                                         sizeof (float) * 2, /* stride_in */
@@ -1168,7 +1185,7 @@ _cogl_journal_discard (CoglJournal *journal)
       CoglJournalEntry *entry =
         &g_array_index (journal->entries, CoglJournalEntry, i);
       _cogl_pipeline_journal_unref (entry->pipeline);
-      _cogl_matrix_entry_unref (entry->modelview_entry);
+      cogl_matrix_entry_unref (entry->modelview_entry);
       _cogl_clip_stack_unref (entry->clip_stack);
     }
 
@@ -1299,6 +1316,7 @@ _cogl_journal_flush (CoglJournal *journal)
                                  ~(COGL_FRAMEBUFFER_STATE_MODELVIEW |
                                    COGL_FRAMEBUFFER_STATE_CLIP));
 
+  state.ctx = ctx;
   state.journal = journal;
 
   state.attributes = ctx->journal_flush_attributes_array;
@@ -1505,7 +1523,7 @@ _cogl_journal_log_quad (CoglJournal  *journal,
 
   modelview_stack =
     _cogl_framebuffer_get_modelview_stack (framebuffer);
-  entry->modelview_entry = _cogl_matrix_entry_ref (modelview_stack->last_entry);
+  entry->modelview_entry = cogl_matrix_entry_ref (modelview_stack->last_entry);
 
   _cogl_pipeline_foreach_layer_internal (pipeline,
                                          add_framebuffer_deps_cb,
@@ -1555,7 +1573,7 @@ entry_to_screen_polygon (CoglFramebuffer *framebuffer,
    * _cogl_transform_points utility...
    */
 
-  _cogl_matrix_entry_get (entry->modelview_entry, &modelview);
+  cogl_matrix_entry_get (entry->modelview_entry, &modelview);
   cogl_matrix_transform_points (&modelview,
                                 2, /* n_components */
                                 sizeof (float) * 4, /* stride_in */
@@ -1567,7 +1585,7 @@ entry_to_screen_polygon (CoglFramebuffer *framebuffer,
 
   projection_stack =
     _cogl_framebuffer_get_projection_stack (framebuffer);
-  _cogl_matrix_stack_get (projection_stack, &projection);
+  cogl_matrix_stack_get (projection_stack, &projection);
 
   cogl_matrix_project_points (&projection,
                               3, /* n_components */
@@ -1691,11 +1709,10 @@ _cogl_journal_try_read_pixel (CoglJournal *journal,
                               CoglBitmap *bitmap,
                               CoglBool *found_intersection)
 {
+  CoglContext *ctx;
   CoglPixelFormat format;
   int i;
 
-  _COGL_GET_CONTEXT (ctx, FALSE);
-
   /* XXX: this number has been plucked out of thin air, but the idea
    * is that if so many pixels are being read from the same un-changed
    * journal than we expect that it will be more efficient to fail
@@ -1713,6 +1730,8 @@ _cogl_journal_try_read_pixel (CoglJournal *journal,
       format != COGL_PIXEL_FORMAT_RGBA_8888)
     return FALSE;
 
+  ctx = _cogl_bitmap_get_context (bitmap);
+
   *found_intersection = FALSE;
 
   /* NB: The most recently added journal entry is the last entry, and
@@ -1732,6 +1751,7 @@ _cogl_journal_try_read_pixel (CoglJournal *journal,
       float poly[16];
       CoglFramebuffer *framebuffer = journal->framebuffer;
       uint8_t *pixel;
+      CoglError *ignore_error;
 
       entry_to_screen_polygon (framebuffer, entry, vertices, poly);
 
@@ -1772,9 +1792,13 @@ _cogl_journal_try_read_pixel (CoglJournal *journal,
 
       pixel = _cogl_bitmap_map (bitmap,
                                 COGL_BUFFER_ACCESS_WRITE,
-                                COGL_BUFFER_MAP_HINT_DISCARD);
+                                COGL_BUFFER_MAP_HINT_DISCARD,
+                                &ignore_error);
       if (pixel == NULL)
-        return FALSE;
+        {
+          cogl_error_free (ignore_error);
+          return FALSE;
+        }
 
       pixel[0] = color[0];
       pixel[1] = color[1];
