@@ -34,9 +34,8 @@
 
 #include <cogl/cogl-context.h>
 #include <cogl/cogl-framebuffer.h>
-#include <glib.h>
 
-G_BEGIN_DECLS
+COGL_BEGIN_DECLS
 
 typedef struct _CoglOnscreen CoglOnscreen;
 #define COGL_ONSCREEN(X) ((CoglOnscreen *)(X))
@@ -70,6 +69,7 @@ typedef void (*CoglOnscreenX11MaskCallback) (CoglOnscreen *onscreen,
  * @xid: The XID of an existing X window
  * @update: A callback that notifies of updates to what Cogl requires
  *          to be in the core X protocol event mask.
+ * @user_data: user data passed to @update
  *
  * Ideally we would recommend that you let Cogl be responsible for
  * creating any X window required to back an onscreen framebuffer but
@@ -224,11 +224,11 @@ cogl_wayland_onscreen_resize (CoglOnscreen *onscreen,
 
 /**
  * cogl_onscreen_set_swap_throttled:
- * @onscreen: A #CoglOncsreen framebuffer
+ * @onscreen: A #CoglOnscreen framebuffer
  * @throttled: Whether swap throttling is wanted or not.
  *
  * Requests that the given @onscreen framebuffer should have swap buffer
- * requests (made using cogl_framebuffer_swap_buffers()) throttled either by a
+ * requests (made using cogl_onscreen_swap_buffers()) throttled either by a
  * displays vblank period or perhaps some other mechanism in a composited
  * environment.
  *
@@ -319,7 +319,7 @@ cogl_onscreen_swap_buffers (CoglOnscreen *onscreen);
  * defined by 4 sequential (x, y, width, height) integers.
  *
  * This function also implicitly discards the contents of the color, depth and
- * stencil buffers as if cogl_onscreen_discard_buffers() were used. The
+ * stencil buffers as if cogl_framebuffer_discard_buffers() were used. The
  * significance of the discard is that you should not expect to be able to
  * start a new frame that incrementally builds on the contents of the previous
  * frame.
@@ -532,6 +532,6 @@ cogl_onscreen_remove_resize_handler (CoglOnscreen *onscreen,
 CoglBool
 cogl_is_onscreen (void *object);
 
-G_END_DECLS
+COGL_END_DECLS
 
 #endif /* __COGL_ONSCREEN_H */
