@@ -34,7 +34,7 @@
 #include <cogl/cogl-types.h>
 #include <cogl/cogl-vector.h>
 
-G_BEGIN_DECLS
+COGL_BEGIN_DECLS
 
 /**
  * SECTION:cogl-quaternion
@@ -45,9 +45,11 @@ G_BEGIN_DECLS
  * rotations and have some nice properties when compared with other
  * representation such as (roll,pitch,yaw) Euler angles. They can be
  * used to interpolate between different rotations and they don't
- * suffer from a problem called "Gimbal lock" where two of the axis of
- * rotation may become aligned and you loose a degree of freedom.
- * (<ulink url="http://en.wikipedia.org/wiki/Gimbal_lock"/>).
+ * suffer from a problem called
+ * <ulink url="http://en.wikipedia.org/wiki/Gimbal_lock">"Gimbal lock"</ulink>
+ * where two of the axis of rotation may become aligned and you loose a
+ * degree of freedom.
+ * .
  */
 #include <cogl/cogl-vector.h>
 #include <cogl/cogl-euler.h>
@@ -165,6 +167,7 @@ cogl_quaternion_init (CoglQuaternion *quaternion,
 /**
  * cogl_quaternion_init_from_angle_vector:
  * @quaternion: An uninitialized #CoglQuaternion
+ * @angle: The angle to rotate around @axis3f
  * @axis3f: your 3 component axis vector about which you want to rotate.
  *
  * Initializes a quaternion that rotates @angle degrees around the
@@ -343,7 +346,7 @@ cogl_quaternion_get_rotation_angle (const CoglQuaternion *quaternion);
 /**
  * cogl_quaternion_get_rotation_axis:
  * @quaternion: A #CoglQuaternion
- *
+ * @vector3: (out): an allocated 3-float array
  *
  * Since: 2.0
  */
@@ -407,6 +410,7 @@ cogl_quaternion_multiply (CoglQuaternion *result,
 /**
  * cogl_quaternion_pow:
  * @quaternion: A #CoglQuaternion
+ * @exponent: the exponent
  *
  *
  * Since: 2.0
@@ -416,6 +420,11 @@ cogl_quaternion_pow (CoglQuaternion *quaternion, float exponent);
 
 /**
  * cogl_quaternion_slerp:
+ * @result: The destination #CoglQuaternion
+ * @a: The first #CoglQuaternion
+ * @b: The second #CoglQuaternion
+ * @t: The factor in the range [0,1] used to interpolate between
+ * quaternion @a and @b.
  *
  * Performs a spherical linear interpolation between two quaternions.
  *
@@ -483,6 +492,13 @@ cogl_quaternion_nlerp (CoglQuaternion *result,
                        float t);
 /**
  * cogl_quaternion_squad:
+ * @result: The destination #CoglQuaternion
+ * @prev: A #CoglQuaternion used before @a
+ * @a: The first #CoglQuaternion
+ * @b: The second #CoglQuaternion
+ * @next: A #CoglQuaternion that will be used after @b
+ * @t: The factor in the range [0,1] used to interpolate between
+ * quaternion @a and @b.
  *
  *
  * Since: 2.0
@@ -523,7 +539,7 @@ cogl_get_static_identity_quaternion (void);
 const CoglQuaternion *
 cogl_get_static_zero_quaternion (void);
 
-G_END_DECLS
+COGL_END_DECLS
 
 #endif /* __COGL_QUATERNION_H__ */
 
