@@ -32,10 +32,10 @@
 #include <string.h>
 
 #include "cogl-util.h"
-#include "cogl-internal.h"
 #include "cogl-private.h"
 #include "cogl-object.h"
 #include "cogl-context-private.h"
+#include "cogl-util-gl-private.h"
 
 #include "cogl-renderer.h"
 #include "cogl-renderer-private.h"
@@ -43,6 +43,7 @@
 #include "cogl-winsys-private.h"
 #include "cogl-winsys-stub-private.h"
 #include "cogl-config-private.h"
+#include "cogl-error-private.h"
 
 #ifdef COGL_HAS_EGL_PLATFORM_XLIB_SUPPORT
 #include "cogl-winsys-egl-x11-private.h"
@@ -72,12 +73,23 @@
 #include "cogl-winsys-sdl-private.h"
 #endif
 
-#if COGL_HAS_XLIB_SUPPORT
+#ifdef COGL_HAS_XLIB_SUPPORT
 #include "cogl-xlib-renderer.h"
 #endif
 
 typedef const CoglWinsysVtable *(*CoglWinsysVtableGetter) (void);
 
+#ifdef HAVE_COGL_GL
+extern const CoglTextureDriver _cogl_texture_driver_gl;
+extern const CoglDriverVtable _cogl_driver_gl;
+#endif
+#if defined (HAVE_COGL_GLES) || defined (HAVE_COGL_GLES2)
+extern const CoglTextureDriver _cogl_texture_driver_gles;
+extern const CoglDriverVtable _cogl_driver_gles;
+#endif
+
+extern const CoglDriverVtable _cogl_driver_nop;
+
 static CoglWinsysVtableGetter _cogl_winsys_vtable_getters[] =
 {
 #ifdef COGL_HAS_GLX_SUPPORT
@@ -120,7 +132,7 @@ typedef struct _CoglNativeFilterClosure
   void *data;
 } CoglNativeFilterClosure;
 
-GQuark
+uint32_t
 cogl_renderer_error_quark (void)
 {
   return g_quark_from_static_string ("cogl-renderer-error-quark");
@@ -176,7 +188,7 @@ cogl_renderer_new (void)
   return _cogl_renderer_object_new (renderer);
 }
 
-#if COGL_HAS_XLIB_SUPPORT
+#ifdef COGL_HAS_XLIB_SUPPORT
 void
 cogl_xlib_renderer_set_foreign_display (CoglRenderer *renderer,
                                         Display *xdisplay)
@@ -216,7 +228,7 @@ cogl_xlib_renderer_set_event_retrieval_enabled (CoglRenderer *renderer,
 CoglBool
 cogl_renderer_check_onscreen_template (CoglRenderer *renderer,
                                        CoglOnscreenTemplate *onscreen_template,
-                                       GError **error)
+                                       CoglError **error)
 {
   CoglDisplay *display;
 
@@ -237,7 +249,7 @@ cogl_renderer_check_onscreen_template (CoglRenderer *renderer,
 
 static CoglBool
 _cogl_renderer_choose_driver (CoglRenderer *renderer,
-                              GError **error)
+                              CoglError **error)
 {
   const char *driver_name = g_getenv ("COGL_DRIVER");
   const char *libgl_name;
@@ -274,6 +286,15 @@ _cogl_renderer_choose_driver (CoglRenderer *renderer,
       libgl_name = COGL_GL_LIBNAME;
       goto found;
     }
+
+  if (renderer->driver_override == COGL_DRIVER_GL3 ||
+      (renderer->driver_override == COGL_DRIVER_ANY &&
+       (driver_name == NULL || !g_ascii_strcasecmp (driver_name, "gl3"))))
+    {
+      renderer->driver = COGL_DRIVER_GL3;
+      libgl_name = COGL_GL_LIBNAME;
+      goto found;
+    }
 #endif
 
 #ifdef HAVE_COGL_GLES2
@@ -298,7 +319,16 @@ _cogl_renderer_choose_driver (CoglRenderer *renderer,
     }
 #endif
 
-  g_set_error (error,
+  if (renderer->driver_override == COGL_DRIVER_NOP ||
+      (renderer->driver_override == COGL_DRIVER_ANY &&
+       (driver_name == NULL || !g_ascii_strcasecmp (driver_name, "nop"))))
+    {
+      renderer->driver = COGL_DRIVER_NOP;
+      libgl_name = NULL;
+      goto found;
+    }
+
+  _cogl_set_error (error,
                COGL_DRIVER_ERROR,
                COGL_DRIVER_ERROR_NO_SUITABLE_DRIVER_FOUND,
                "No suitable driver found");
@@ -309,7 +339,7 @@ found:
   if (support_gles2_constraint &&
       renderer->driver != COGL_DRIVER_GLES2)
     {
-      g_set_error (error,
+      _cogl_set_error (error,
                    COGL_RENDERER_ERROR,
                    COGL_RENDERER_ERROR_BAD_CONSTRAINT,
                    "No suitable driver found");
@@ -318,27 +348,57 @@ found:
 
 #ifndef HAVE_DIRECTLY_LINKED_GL_LIBRARY
 
-  renderer->libgl_module = g_module_open (libgl_name,
-                                          G_MODULE_BIND_LAZY);
-
-  if (renderer->libgl_module == NULL)
+  if (renderer->driver == COGL_DRIVER_GL ||
+      renderer->driver == COGL_DRIVER_GL3 ||
+      renderer->driver == COGL_DRIVER_GLES1 ||
+      renderer->driver == COGL_DRIVER_GLES2)
     {
-      g_set_error (error, COGL_DRIVER_ERROR,
-                   COGL_DRIVER_ERROR_FAILED_TO_LOAD_LIBRARY,
-                   "Failed to dynamically open the GL library \"%s\"",
-                   libgl_name);
-      return FALSE;
+      renderer->libgl_module = g_module_open (libgl_name,
+                                              G_MODULE_BIND_LAZY);
+
+      if (renderer->libgl_module == NULL)
+        {
+          _cogl_set_error (error, COGL_DRIVER_ERROR,
+                       COGL_DRIVER_ERROR_FAILED_TO_LOAD_LIBRARY,
+                       "Failed to dynamically open the GL library \"%s\"",
+                       libgl_name);
+          return FALSE;
+        }
     }
 
 #endif /* HAVE_DIRECTLY_LINKED_GL_LIBRARY */
 
+  switch (renderer->driver)
+    {
+#ifdef HAVE_COGL_GL
+    case COGL_DRIVER_GL:
+    case COGL_DRIVER_GL3:
+      renderer->driver_vtable = &_cogl_driver_gl;
+      renderer->texture_driver = &_cogl_texture_driver_gl;
+      break;
+#endif
+
+#if defined (HAVE_COGL_GLES) || defined (HAVE_COGL_GLES2)
+    case COGL_DRIVER_GLES1:
+    case COGL_DRIVER_GLES2:
+      renderer->driver_vtable = &_cogl_driver_gles;
+      renderer->texture_driver = &_cogl_texture_driver_gles;
+      break;
+#endif
+
+    case COGL_DRIVER_NOP:
+    default:
+      renderer->driver_vtable = &_cogl_driver_nop;
+      renderer->texture_driver = NULL;
+    }
+
   return TRUE;
 }
 
 /* Final connection API */
 
 CoglBool
-cogl_renderer_connect (CoglRenderer *renderer, GError **error)
+cogl_renderer_connect (CoglRenderer *renderer, CoglError **error)
 {
   int i;
   GString *error_message;
@@ -357,7 +417,7 @@ cogl_renderer_connect (CoglRenderer *renderer, GError **error)
   for (i = 0; i < G_N_ELEMENTS (_cogl_winsys_vtable_getters); i++)
     {
       const CoglWinsysVtable *winsys = _cogl_winsys_vtable_getters[i]();
-      GError *tmp_error = NULL;
+      CoglError *tmp_error = NULL;
       GList *l;
       CoglBool skip_due_to_constraints = FALSE;
 
@@ -400,7 +460,7 @@ cogl_renderer_connect (CoglRenderer *renderer, GError **error)
         {
           g_string_append_c (error_message, '\n');
           g_string_append (error_message, tmp_error->message);
-          g_error_free (tmp_error);
+          cogl_error_free (tmp_error);
         }
       else
         {
@@ -414,14 +474,14 @@ cogl_renderer_connect (CoglRenderer *renderer, GError **error)
     {
       if (constraints_failed)
         {
-          g_set_error (error, COGL_RENDERER_ERROR,
+          _cogl_set_error (error, COGL_RENDERER_ERROR,
                        COGL_RENDERER_ERROR_BAD_CONSTRAINT,
                        "Failed to connected to any renderer due to constraints");
           return FALSE;
         }
 
       renderer->winsys_vtable = NULL;
-      g_set_error (error, COGL_WINSYS_ERROR,
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_INIT,
                    "Failed to connected to any renderer: %s",
                    error_message->str);
@@ -530,7 +590,7 @@ cogl_renderer_get_n_fragment_texture_units (CoglRenderer *renderer)
   _COGL_GET_CONTEXT (ctx, 0);
 
 #if defined (HAVE_COGL_GL) || defined (HAVE_COGL_GLES2)
-  if (ctx->driver == COGL_DRIVER_GL || ctx->driver == COGL_DRIVER_GLES2)
+  if (cogl_has_feature (ctx, COGL_FEATURE_ID_GLSL))
     GE (ctx, glGetIntegerv (GL_MAX_TEXTURE_IMAGE_UNITS, &n));
 #endif
 
