@@ -26,10 +26,11 @@
 #endif
 
 #include "cogl-shader-private.h"
-#include "cogl-shader-boilerplate.h"
-#include "cogl-internal.h"
+#include "cogl-util-gl-private.h"
 #include "cogl-context-private.h"
 #include "cogl-object-private.h"
+#include "cogl-glsl-shader-private.h"
+#include "cogl-glsl-shader-boilerplate.h"
 
 #include <glib.h>
 
@@ -89,9 +90,7 @@ cogl_create_shader (CoglShaderType type)
   shader = g_slice_new (CoglShader);
   shader->language = COGL_SHADER_LANGUAGE_GLSL;
   shader->gl_handle = 0;
-#ifdef HAVE_COGL_GLES2
-  shader->n_tex_coord_attribs = 0;
-#endif
+  shader->compilation_pipeline = NULL;
   shader->type = type;
 
   return _cogl_shader_handle_new (shader);
@@ -116,6 +115,12 @@ delete_shader (CoglShader *shader)
     }
 
   shader->gl_handle = 0;
+
+  if (shader->compilation_pipeline)
+    {
+      cogl_object_unref (shader->compilation_pipeline);
+      shader->compilation_pipeline = NULL;
+    }
 }
 
 void
@@ -152,19 +157,22 @@ cogl_shader_source (CoglHandle   handle,
 void
 cogl_shader_compile (CoglHandle handle)
 {
-  CoglShader *shader = handle;
+  CoglShader *shader;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   if (!cogl_is_shader (handle))
     return;
 
-  if (ctx->driver == COGL_DRIVER_GL)
-    _cogl_shader_compile_real (shader, 0 /* ignored */);
+#ifdef HAVE_COGL_GL
+  shader = handle;
+  if (shader->language == COGL_SHADER_LANGUAGE_ARBFP)
+    _cogl_shader_compile_real (handle, NULL);
+#endif
 
-  /* XXX: For GLES2 we don't actually compile anything until the
-   * shader gets used so we have an opportunity to add some
-   * boilerplate to the shader.
+  /* XXX: For GLSL we don't actually compile anything until the shader
+   * gets used so we have an opportunity to add some boilerplate to
+   * the shader.
    *
    * At the end of the day this is obviously a badly designed API
    * given that we are having to lie to the user. It was a mistake to
@@ -172,126 +180,12 @@ cogl_shader_compile (CoglHandle handle)
    * replace it with a pipeline snippets API. */
 }
 
-void
-_cogl_shader_set_source_with_boilerplate (GLuint shader_gl_handle,
-                                          GLenum shader_gl_type,
-                                          int n_tex_coord_attribs,
-                                          GLsizei count_in,
-                                          const char **strings_in,
-                                          const GLint *lengths_in)
-{
-  const char *vertex_boilerplate;
-  const char *fragment_boilerplate;
-
-  const char **strings = g_alloca (sizeof (char *) * (count_in + 3));
-  GLint *lengths = g_alloca (sizeof (GLint) * (count_in + 3));
-  int count = 0;
-  char *tex_coord_declarations = NULL;
-
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  if (ctx->driver == COGL_DRIVER_GLES2)
-    {
-      vertex_boilerplate = _COGL_VERTEX_SHADER_BOILERPLATE_GLES2;
-      fragment_boilerplate = _COGL_FRAGMENT_SHADER_BOILERPLATE_GLES2;
-    }
-  else
-    {
-      vertex_boilerplate = _COGL_VERTEX_SHADER_BOILERPLATE_GL;
-      fragment_boilerplate = _COGL_FRAGMENT_SHADER_BOILERPLATE_GL;
-    }
-
-  if (ctx->driver == COGL_DRIVER_GLES2 &&
-      cogl_has_feature (ctx, COGL_FEATURE_ID_TEXTURE_3D))
-    {
-      static const char texture_3d_extension[] =
-        "#extension GL_OES_texture_3D : enable\n";
-      strings[count] = texture_3d_extension;
-      lengths[count++] = sizeof (texture_3d_extension) - 1;
-    }
-
-  if (shader_gl_type == GL_VERTEX_SHADER)
-    {
-      strings[count] = vertex_boilerplate;
-      lengths[count++] = strlen (vertex_boilerplate);
-    }
-  else if (shader_gl_type == GL_FRAGMENT_SHADER)
-    {
-      strings[count] = fragment_boilerplate;
-      lengths[count++] = strlen (fragment_boilerplate);
-    }
-
-  if (ctx->driver == COGL_DRIVER_GLES2 &&
-      n_tex_coord_attribs)
-    {
-      GString *declarations = g_string_new (NULL);
-
-      g_string_append_printf (declarations,
-                              "varying vec4 _cogl_tex_coord[%d];\n",
-                              n_tex_coord_attribs);
-
-      if (shader_gl_type == GL_VERTEX_SHADER)
-        {
-          int i;
-
-          g_string_append_printf (declarations,
-                                  "uniform mat4 cogl_texture_matrix[%d];\n",
-                                  n_tex_coord_attribs);
-
-          for (i = 0; i < n_tex_coord_attribs; i++)
-            g_string_append_printf (declarations,
-                                    "attribute vec4 cogl_tex_coord%d_in;\n",
-                                    i);
-        }
-
-      tex_coord_declarations = g_string_free (declarations, FALSE);
-      strings[count] = tex_coord_declarations;
-      lengths[count++] = -1; /* null terminated */
-    }
-
-  memcpy (strings + count, strings_in, sizeof (char *) * count_in);
-  if (lengths_in)
-    memcpy (lengths + count, lengths_in, sizeof (GLint) * count_in);
-  else
-    {
-      int i;
-
-      for (i = 0; i < count_in; i++)
-        lengths[count + i] = -1; /* null terminated */
-    }
-  count += count_in;
-
-  if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_SHOW_SOURCE)))
-    {
-      GString *buf = g_string_new (NULL);
-      int i;
-
-      g_string_append_printf (buf,
-                              "%s shader:\n",
-                              shader_gl_type == GL_VERTEX_SHADER ?
-                              "vertex" : "fragment");
-      for (i = 0; i < count; i++)
-        if (lengths[i] != -1)
-          g_string_append_len (buf, strings[i], lengths[i]);
-        else
-          g_string_append (buf, strings[i]);
-
-      g_message ("%s", buf->str);
-
-      g_string_free (buf, TRUE);
-    }
-
-  GE( ctx, glShaderSource (shader_gl_handle, count,
-                           (const char **) strings, lengths) );
-
-  g_free (tex_coord_declarations);
-}
-
 void
 _cogl_shader_compile_real (CoglHandle handle,
-                           int n_tex_coord_attribs)
+                           CoglPipeline *pipeline)
 {
   CoglShader *shader = handle;
+  const char *version;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -336,15 +230,20 @@ _cogl_shader_compile_real (CoglHandle handle,
 #endif
     {
       GLenum gl_type;
+      GLint status;
 
-      if (shader->gl_handle
-#ifdef HAVE_COGL_GLES2
-          &&
-          (ctx->driver != COGL_DRIVER_GLES2 ||
-           shader->n_tex_coord_attribs == n_tex_coord_attribs)
-#endif
-         )
-        return;
+      if (shader->gl_handle)
+        {
+          CoglPipeline *prev = shader->compilation_pipeline;
+
+          /* XXX: currently the only things that will affect the
+           * boilerplate for user shaders, apart from driver features,
+           * are the pipeline layer-indices and texture-unit-indices
+           */
+          if (pipeline == prev ||
+              _cogl_pipeline_layer_and_unit_numbers_equal (prev, pipeline))
+            return;
+        }
 
       if (shader->gl_handle)
         delete_shader (shader);
@@ -364,78 +263,73 @@ _cogl_shader_compile_real (CoglHandle handle,
 
       shader->gl_handle = ctx->glCreateShader (gl_type);
 
-      _cogl_shader_set_source_with_boilerplate (shader->gl_handle,
-                                                gl_type,
-                                                n_tex_coord_attribs,
-                                                1,
-                                                (const char **) &shader->source,
-                                                NULL);
+      if ((ctx->driver == COGL_DRIVER_GL ||
+           ctx->driver == COGL_DRIVER_GL3 ) &&
+          ctx->glsl_major == 1 &&
+          ctx->glsl_minor >= 2)
+        {
+          version = "#version 120\n";
+        }
+      else
+        version = NULL;
+
+      _cogl_glsl_shader_set_source_with_boilerplate (ctx,
+                                                     version,
+                                                     shader->gl_handle,
+                                                     gl_type,
+                                                     pipeline,
+                                                     1,
+                                                     (const char **)
+                                                      &shader->source,
+                                                     NULL);
 
       GE (ctx, glCompileShader (shader->gl_handle));
 
-#ifdef HAVE_COGL_GLES2
-      shader->n_tex_coord_attribs = n_tex_coord_attribs;
-#endif
+      shader->compilation_pipeline = cogl_object_ref (pipeline);
 
-#ifdef COGL_GL_DEBUG
-      if (!cogl_shader_is_compiled (handle))
+      GE (ctx, glGetShaderiv (shader->gl_handle, GL_COMPILE_STATUS, &status));
+      if (!status)
         {
-          char *log = cogl_shader_get_info_log (handle);
-          g_warning ("Failed to compile GLSL program:\nsrc:\n%s\nerror:\n%s\n",
+          char buffer[512];
+          int len = 0;
+
+          ctx->glGetShaderInfoLog (shader->gl_handle, 511, &len, buffer);
+          buffer[len] = '\0';
+
+          g_warning ("Failed to compile GLSL program:\n"
+                     "src:\n%s\n"
+                     "error:\n%s\n",
                      shader->source,
-                     log);
-          g_free (log);
+                     buffer);
         }
-#endif
     }
 }
 
 char *
 cogl_shader_get_info_log (CoglHandle handle)
 {
-  CoglShader *shader;
-
-  _COGL_GET_CONTEXT (ctx, NULL);
-
   if (!cogl_is_shader (handle))
     return NULL;
 
-  shader = handle;
+  /* XXX: This API doesn't really do anything!
+   *
+   * This API is purely for compatibility
+   *
+   * The reason we don't do anything is because a shader needs to
+   * be associated with a CoglPipeline for Cogl to be able to
+   * compile and link anything.
+   *
+   * The way this API was originally designed as a very thin wrapper
+   * over the GL api was a mistake and it's now very difficult to
+   * make the API work in a meaningful way given how the rest of Cogl
+   * has evolved.
+   *
+   * The CoglShader API is mostly deprecated by CoglSnippets and so
+   * these days we do the bare minimum to support the existing users
+   * of it until they are able to migrate to the snippets api.
+   */
 
-#ifdef HAVE_COGL_GL
-  if (shader->language == COGL_SHADER_LANGUAGE_ARBFP)
-    {
-      /* ARBfp exposes a program error string, but since cogl_program
-       * doesn't have any API to query an error log it is not currently
-       * exposed. */
-      return g_strdup ("");
-    }
-  else
-#endif
-    {
-      char buffer[512];
-      int len = 0;
-
-      /* We don't normally compile the shader when the user calls
-       * cogl_shader_compile() because we want to be able to add
-       * boilerplate code that depends on how it ends up finally being
-       * used.
-       *
-       * Here we force an early compile if the user is interested in
-       * log information to increase the chance that the log will be
-       * useful! We have to guess the number of texture coordinate
-       * attributes that may be used since that affects the
-       * boilerplate. We use four so that the shader will still
-       * compile if the user is using more than one
-       * layer. Unfortunately this is likely to end up causing it to
-       * be compiled again when we know the actual number of layers */
-      if (!shader->gl_handle)
-        _cogl_shader_compile_real (handle, 4);
-
-      ctx->glGetShaderInfoLog (shader->gl_handle, 511, &len, buffer);
-      buffer[len] = '\0';
-      return g_strdup (buffer);
-    }
+  return g_strdup ("");
 }
 
 CoglShaderType
@@ -459,46 +353,29 @@ CoglBool
 cogl_shader_is_compiled (CoglHandle handle)
 {
 #if defined (HAVE_COGL_GL) || defined (HAVE_COGL_GLES2)
-  GLint status;
-  CoglShader *shader;
-
-  _COGL_GET_CONTEXT (ctx, FALSE);
-
   if (!cogl_is_shader (handle))
     return FALSE;
 
-  shader = handle;
+  /* XXX: This API doesn't really do anything!
+   *
+   * This API is purely for compatibility and blatantly lies to the
+   * user about whether their shader has been compiled.
+   *
+   * I suppose we could say we're stretching the definition of
+   * "compile" and are deferring any related errors to be "linker"
+   * errors.
+   *
+   * The reason we don't do anything is because a shader needs to
+   * be associated with a CoglPipeline for Cogl to be able to
+   * compile and link anything.
+   *
+   * The CoglShader API is mostly deprecated by CoglSnippets and so
+   * these days we do the bare minimum to support the existing users
+   * of it until they are able to migrate to the snippets api.
+   */
 
-#ifdef HAVE_COGL_GL
-  if (shader->language == COGL_SHADER_LANGUAGE_ARBFP)
-    return TRUE;
-  else
-#endif
-    {
-      /* FIXME: We currently have an arbitrary limit of 4 texture
-       * coordinate attributes since our API means we have to add
-       * some boilerplate to the users GLSL program (for GLES2)
-       * before we actually know how many attributes are in use.
-       *
-       * 4 will probably be enough (or at least that limitation should
-       * be enough until we can replace this API with the pipeline
-       * snippets API) but if it isn't then the shader won't compile,
-       * through no fault of the user.
-       *
-       * To some extent this is just a symptom of bad API design; it
-       * was a mistake for Cogl to so thinly wrap the OpenGL shader
-       * API. Eventually we plan for this whole API will be deprecated
-       * by the pipeline snippets framework.
-       */
-      if (!shader->gl_handle)
-        _cogl_shader_compile_real (handle, 4);
+  return TRUE;
 
-      GE (ctx, glGetShaderiv (shader->gl_handle, GL_COMPILE_STATUS, &status));
-      if (status == GL_TRUE)
-        return TRUE;
-      else
-        return FALSE;
-    }
 #else
   return FALSE;
 #endif
