@@ -33,28 +33,29 @@
 
 struct _CoglTexture2DSliced
 {
-  CoglTexture       _parent;
-  GArray           *slice_x_spans;
-  GArray           *slice_y_spans;
-  GArray           *slice_textures;
-  int               max_waste;
-
-  int               width;
-  int               height;
+  CoglTexture _parent;
+  GArray *slice_x_spans;
+  GArray *slice_y_spans;
+  GArray *slice_textures;
+  int max_waste;
+  CoglPixelFormat internal_format;
 };
 
 CoglTexture2DSliced *
-_cogl_texture_2d_sliced_new_from_foreign (GLuint           gl_handle,
-                                          GLenum           gl_target,
-                                          GLuint           width,
-                                          GLuint           height,
-                                          GLuint           x_pot_waste,
-                                          GLuint           y_pot_waste,
-                                          CoglPixelFormat  format);
+_cogl_texture_2d_sliced_new_from_foreign (CoglContext *context,
+                                          unsigned int gl_handle,
+                                          unsigned int gl_target,
+                                          int width,
+                                          int height,
+                                          int x_pot_waste,
+                                          int y_pot_waste,
+                                          CoglPixelFormat  format,
+                                          CoglError **error);
 
 CoglTexture2DSliced *
-_cogl_texture_2d_sliced_new_from_bitmap (CoglBitmap      *bmp,
+_cogl_texture_2d_sliced_new_from_bitmap (CoglBitmap *bmp,
                                          CoglTextureFlags flags,
-                                         CoglPixelFormat  internal_format);
+                                         CoglPixelFormat internal_format,
+                                         CoglError **error);
 
 #endif /* __COGL_TEXTURE_2D_SLICED_PRIVATE_H */
