@@ -37,6 +37,8 @@
 #include "cogl-object-private.h"
 #include "cogl-journal-private.h"
 #include "cogl-pipeline-opengl-private.h"
+#include "cogl-error-private.h"
+#include "cogl-util-gl-private.h"
 
 #include <string.h>
 #include <math.h>
@@ -67,10 +69,10 @@ can_use_wrap_mode (GLenum wrap_mode)
 }
 
 static void
-_cogl_texture_rectangle_set_wrap_mode_parameters (CoglTexture *tex,
-                                                  GLenum wrap_mode_s,
-                                                  GLenum wrap_mode_t,
-                                                  GLenum wrap_mode_p)
+_cogl_texture_rectangle_gl_flush_legacy_texobj_wrap_modes (CoglTexture *tex,
+                                                           GLenum wrap_mode_s,
+                                                           GLenum wrap_mode_t,
+                                                           GLenum wrap_mode_p)
 {
   CoglTextureRectangle *tex_rect = COGL_TEXTURE_RECTANGLE (tex);
   CoglContext *ctx = tex->context;
@@ -78,8 +80,8 @@ _cogl_texture_rectangle_set_wrap_mode_parameters (CoglTexture *tex,
   /* Only set the wrap mode if it's different from the current value
      to avoid too many GL calls. Texture rectangle doesn't make use of
      the r coordinate so we can ignore its wrap mode */
-  if (tex_rect->wrap_mode_s != wrap_mode_s ||
-      tex_rect->wrap_mode_t != wrap_mode_t)
+  if (tex_rect->gl_legacy_texobj_wrap_mode_s != wrap_mode_s ||
+      tex_rect->gl_legacy_texobj_wrap_mode_t != wrap_mode_t)
     {
       g_assert (can_use_wrap_mode (wrap_mode_s));
       g_assert (can_use_wrap_mode (wrap_mode_t));
@@ -92,15 +94,15 @@ _cogl_texture_rectangle_set_wrap_mode_parameters (CoglTexture *tex,
       GE( ctx, glTexParameteri (GL_TEXTURE_RECTANGLE_ARB,
                                 GL_TEXTURE_WRAP_T, wrap_mode_t) );
 
-      tex_rect->wrap_mode_s = wrap_mode_s;
-      tex_rect->wrap_mode_t = wrap_mode_t;
+      tex_rect->gl_legacy_texobj_wrap_mode_s = wrap_mode_s;
+      tex_rect->gl_legacy_texobj_wrap_mode_t = wrap_mode_t;
     }
 }
 
 static void
 _cogl_texture_rectangle_free (CoglTextureRectangle *tex_rect)
 {
-  if (!tex_rect->is_foreign)
+  if (!tex_rect->is_foreign && tex_rect->gl_texture)
     _cogl_delete_gl_texture (tex_rect->gl_texture);
 
   /* Chain up */
@@ -112,7 +114,7 @@ _cogl_texture_rectangle_can_create (CoglContext *ctx,
                                     unsigned int width,
                                     unsigned int height,
                                     CoglPixelFormat internal_format,
-                                    GError **error)
+                                    CoglError **error)
 {
   GLenum gl_intformat;
   GLenum gl_format;
@@ -120,10 +122,10 @@ _cogl_texture_rectangle_can_create (CoglContext *ctx,
 
   if (!cogl_has_feature (ctx, COGL_FEATURE_ID_TEXTURE_RECTANGLE))
     {
-      g_set_error (error,
-                   COGL_TEXTURE_ERROR,
-                   COGL_TEXTURE_ERROR_TYPE,
-                   "The CoglTextureRectangle feature isn't available");
+      _cogl_set_error (error,
+                       COGL_TEXTURE_ERROR,
+                       COGL_TEXTURE_ERROR_TYPE,
+                       "The CoglTextureRectangle feature isn't available");
       return FALSE;
     }
 
@@ -142,10 +144,10 @@ _cogl_texture_rectangle_can_create (CoglContext *ctx,
                                             width,
                                             height))
     {
-      g_set_error (error,
-                   COGL_TEXTURE_ERROR,
-                   COGL_TEXTURE_ERROR_SIZE,
-                   "The requested texture size + format is unsupported");
+      _cogl_set_error (error,
+                       COGL_TEXTURE_ERROR,
+                       COGL_TEXTURE_ERROR_SIZE,
+                       "The requested texture size + format is unsupported");
       return FALSE;
     }
 
@@ -169,22 +171,21 @@ _cogl_texture_rectangle_create_base (CoglContext *ctx,
   CoglTextureRectangle *tex_rect = g_new (CoglTextureRectangle, 1);
   CoglTexture *tex = COGL_TEXTURE (tex_rect);
 
-  _cogl_texture_init (tex, ctx, &cogl_texture_rectangle_vtable);
+  _cogl_texture_init (tex, ctx, width, height, &cogl_texture_rectangle_vtable);
 
-  tex_rect->width = width;
-  tex_rect->height = height;
+  tex_rect->gl_texture = 0;
 
   /* We default to GL_LINEAR for both filters */
-  tex_rect->min_filter = GL_LINEAR;
-  tex_rect->mag_filter = GL_LINEAR;
+  tex_rect->gl_legacy_texobj_min_filter = GL_LINEAR;
+  tex_rect->gl_legacy_texobj_mag_filter = GL_LINEAR;
 
   /* Wrap mode not yet set */
-  tex_rect->wrap_mode_s = GL_FALSE;
-  tex_rect->wrap_mode_t = GL_FALSE;
+  tex_rect->gl_legacy_texobj_wrap_mode_s = GL_FALSE;
+  tex_rect->gl_legacy_texobj_wrap_mode_t = GL_FALSE;
 
-  tex_rect->format = internal_format;
+  tex_rect->internal_format = internal_format;
 
-  return tex_rect;
+  return _cogl_texture_rectangle_object_new (tex_rect);
 }
 
 CoglTextureRectangle *
@@ -192,49 +193,87 @@ cogl_texture_rectangle_new_with_size (CoglContext *ctx,
                                       int width,
                                       int height,
                                       CoglPixelFormat internal_format,
-                                      GError **error)
+                                      CoglError **error)
 {
   CoglTextureRectangle *tex_rect;
-  GLenum                gl_intformat;
-  GLenum                gl_format;
-  GLenum                gl_type;
 
   /* Since no data, we need some internal format */
   if (internal_format == COGL_PIXEL_FORMAT_ANY)
     internal_format = COGL_PIXEL_FORMAT_RGBA_8888_PRE;
 
+  tex_rect =_cogl_texture_rectangle_create_base (ctx,
+                                                 width, height,
+                                                 internal_format);
+
+  /* XXX: This api has been changed for Cogl 2.0 on the master branch
+   * to not take a CoglError to allow the storage to be allocated
+   * lazily but since Mutter uses this api we are currently
+   * maintaining the semantics of immediately allocating the storage
+   */
+  if (!cogl_texture_allocate (COGL_TEXTURE (tex_rect), error))
+    {
+      cogl_object_unref (tex_rect);
+      return NULL;
+    }
+  return tex_rect;
+}
+
+static CoglBool
+_cogl_texture_rectangle_allocate (CoglTexture *tex,
+                                  CoglError **error)
+{
+  CoglContext *ctx = tex->context;
+  CoglTextureRectangle *tex_rect = COGL_TEXTURE_RECTANGLE (tex);
+  GLenum gl_intformat;
+  GLenum gl_format;
+  GLenum gl_type;
+  GLenum gl_error;
+  GLenum gl_texture;
+
   if (!_cogl_texture_rectangle_can_create (ctx,
-                                           width, height,
-                                           internal_format, error))
-    return NULL;
+                                           tex->width,
+                                           tex->height,
+                                           tex_rect->internal_format,
+                                           error))
+    return FALSE;
 
-  internal_format = ctx->driver_vtable->pixel_format_to_gl (ctx,
-                                                            internal_format,
-                                                            &gl_intformat,
-                                                            &gl_format,
-                                                            &gl_type);
+  ctx->driver_vtable->pixel_format_to_gl (ctx,
+                                          tex_rect->internal_format,
+                                          &gl_intformat,
+                                          &gl_format,
+                                          &gl_type);
 
-  tex_rect = _cogl_texture_rectangle_create_base (ctx,
-                                                  width, height,
-                                                  internal_format);
-
-  ctx->texture_driver->gen (ctx,
-                            GL_TEXTURE_RECTANGLE_ARB,
-                            1, /* num textures */
-                            &tex_rect->gl_texture);
+  gl_texture =
+    ctx->texture_driver->gen (ctx,
+                              GL_TEXTURE_RECTANGLE_ARB,
+                              tex_rect->internal_format);
   _cogl_bind_gl_texture_transient (GL_TEXTURE_RECTANGLE_ARB,
-                                   tex_rect->gl_texture,
+                                   gl_texture,
                                    tex_rect->is_foreign);
-  GE( ctx, glTexImage2D (GL_TEXTURE_RECTANGLE_ARB, 0, gl_intformat,
-                         width, height, 0, gl_format, gl_type, NULL) );
 
-  return _cogl_texture_rectangle_object_new (tex_rect);
+  /* Clear any GL errors */
+  while ((gl_error = ctx->glGetError ()) != GL_NO_ERROR)
+    ;
+
+  ctx->glTexImage2D (GL_TEXTURE_RECTANGLE_ARB, 0, gl_intformat,
+                     tex->width, tex->height, 0, gl_format, gl_type, NULL);
+
+  if (_cogl_gl_util_catch_out_of_memory (ctx, error))
+    {
+      GE( ctx, glDeleteTextures (1, &gl_texture) );
+      return FALSE;
+    }
+
+  tex_rect->gl_texture = gl_texture;
+  tex_rect->gl_format = gl_intformat;
+
+  return TRUE;
 }
 
 CoglTextureRectangle *
 cogl_texture_rectangle_new_from_bitmap (CoglBitmap *bmp,
                                         CoglPixelFormat internal_format,
-                                        GError **error)
+                                        CoglError **error)
 {
   CoglTextureRectangle *tex_rect;
   CoglBitmap           *dst_bmp;
@@ -263,7 +302,8 @@ cogl_texture_rectangle_new_from_bitmap (CoglBitmap *bmp,
                                               &internal_format,
                                               &gl_intformat,
                                               &gl_format,
-                                              &gl_type);
+                                              &gl_type,
+                                              error);
 
   if (dst_bmp == NULL)
     return NULL;
@@ -273,50 +313,64 @@ cogl_texture_rectangle_new_from_bitmap (CoglBitmap *bmp,
                                                   cogl_bitmap_get_height (bmp),
                                                   internal_format);
 
-  ctx->texture_driver->gen (ctx,
-                            GL_TEXTURE_RECTANGLE_ARB,
-                            1, /* num textures */
-                            &tex_rect->gl_texture);
-  ctx->texture_driver->upload_to_gl (ctx,
-                                     GL_TEXTURE_RECTANGLE_ARB,
-                                     tex_rect->gl_texture,
-                                     FALSE,
-                                     dst_bmp,
-                                     gl_intformat,
-                                     gl_format,
-                                     gl_type);
+  tex_rect->gl_texture =
+    ctx->texture_driver->gen (ctx,
+                              GL_TEXTURE_RECTANGLE_ARB,
+                              internal_format);
+  if (!ctx->texture_driver->upload_to_gl (ctx,
+                                          GL_TEXTURE_RECTANGLE_ARB,
+                                          tex_rect->gl_texture,
+                                          FALSE,
+                                          dst_bmp,
+                                          gl_intformat,
+                                          gl_format,
+                                          gl_type,
+                                          error))
+    {
+      cogl_object_unref (dst_bmp);
+      cogl_object_unref (tex_rect);
+      return NULL;
+    }
 
   tex_rect->gl_format = gl_intformat;
 
   cogl_object_unref (dst_bmp);
 
-  return _cogl_texture_rectangle_object_new (tex_rect);
+  _cogl_texture_set_allocated (COGL_TEXTURE (tex_rect), TRUE);
+
+  return tex_rect;
 }
 
 CoglTextureRectangle *
-_cogl_texture_rectangle_new_from_foreign (GLuint gl_handle,
-                                          GLuint width,
-                                          GLuint height,
-                                          CoglPixelFormat format)
+cogl_texture_rectangle_new_from_foreign (CoglContext *ctx,
+                                         unsigned int gl_handle,
+                                         int width,
+                                         int height,
+                                         CoglPixelFormat format,
+                                         CoglError **error)
 {
   /* NOTE: width, height and internal format are not queriable
    * in GLES, hence such a function prototype.
    */
 
-  GLenum                gl_error      = 0;
-  GLint                 gl_compressed = GL_FALSE;
-  GLenum                gl_int_format = 0;
+  GLenum gl_error = 0;
+  GLint gl_compressed = GL_FALSE;
+  GLenum gl_int_format = 0;
   CoglTextureRectangle *tex_rect;
 
-  _COGL_GET_CONTEXT (ctx, NULL);
+  /* Assert that it is a valid GL texture object */
+  g_return_val_if_fail (ctx->glIsTexture (gl_handle), NULL);
 
   if (!ctx->texture_driver->allows_foreign_gl_target (ctx,
                                                       GL_TEXTURE_RECTANGLE_ARB))
-    return NULL;
-
-  /* Make sure it is a valid GL texture object */
-  if (!ctx->glIsTexture (gl_handle))
-    return NULL;
+    {
+      _cogl_set_error (error,
+                       COGL_SYSTEM_ERROR,
+                       COGL_SYSTEM_ERROR_UNSUPPORTED,
+                       "Foreign GL_TEXTURE_RECTANGLE textures are not "
+                       "supported by your system");
+      return NULL;
+    }
 
   /* Make sure binding succeeds */
   while ((gl_error = ctx->glGetError ()) != GL_NO_ERROR)
@@ -324,7 +378,13 @@ _cogl_texture_rectangle_new_from_foreign (GLuint gl_handle,
 
   _cogl_bind_gl_texture_transient (GL_TEXTURE_RECTANGLE_ARB, gl_handle, TRUE);
   if (ctx->glGetError () != GL_NO_ERROR)
-    return NULL;
+    {
+      _cogl_set_error (error,
+                       COGL_SYSTEM_ERROR,
+                       COGL_SYSTEM_ERROR_UNSUPPORTED,
+                       "Failed to bind foreign GL_TEXTURE_RECTANGLE texture");
+      return NULL;
+    }
 
   /* Obtain texture parameters */
 
@@ -348,7 +408,13 @@ _cogl_texture_rectangle_new_from_foreign (GLuint gl_handle,
       if (!ctx->driver_vtable->pixel_format_from_gl_internal (ctx,
                                                               gl_int_format,
                                                               &format))
-        return NULL;
+        {
+          _cogl_set_error (error,
+                           COGL_SYSTEM_ERROR,
+                           COGL_SYSTEM_ERROR_UNSUPPORTED,
+                           "Unsupported internal format for foreign texture");
+          return NULL;
+        }
     }
   else
 #endif
@@ -370,12 +436,17 @@ _cogl_texture_rectangle_new_from_foreign (GLuint gl_handle,
    */
 
   /* Validate width and height */
-  if (width <= 0 || height <= 0)
-    return NULL;
+  g_return_val_if_fail (width > 0 && height > 0, NULL);
 
   /* Compressed texture images not supported */
   if (gl_compressed == GL_TRUE)
-    return NULL;
+    {
+      _cogl_set_error (error,
+                       COGL_SYSTEM_ERROR,
+                       COGL_SYSTEM_ERROR_UNSUPPORTED,
+                       "Compressed foreign textures aren't currently supported");
+      return NULL;
+    }
 
   /* Create new texture */
   tex_rect = _cogl_texture_rectangle_create_base (ctx, width, height, format);
@@ -383,16 +454,18 @@ _cogl_texture_rectangle_new_from_foreign (GLuint gl_handle,
   /* Setup bitmap info */
   tex_rect->is_foreign = TRUE;
 
-  tex_rect->format = format;
+  tex_rect->internal_format = format;
 
   tex_rect->gl_texture = gl_handle;
   tex_rect->gl_format = gl_int_format;
 
   /* Unknown filter */
-  tex_rect->min_filter = GL_FALSE;
-  tex_rect->mag_filter = GL_FALSE;
+  tex_rect->gl_legacy_texobj_min_filter = GL_FALSE;
+  tex_rect->gl_legacy_texobj_mag_filter = GL_FALSE;
 
-  return _cogl_texture_rectangle_object_new (tex_rect);
+  _cogl_texture_set_allocated (COGL_TEXTURE (tex_rect), TRUE);
+
+  return tex_rect;
 }
 
 static int
@@ -418,17 +491,14 @@ _cogl_texture_rectangle_transform_coords_to_gl (CoglTexture *tex,
                                                 float *s,
                                                 float *t)
 {
-  CoglTextureRectangle *tex_rect = COGL_TEXTURE_RECTANGLE (tex);
-
-  *s *= tex_rect->width;
-  *t *= tex_rect->height;
+  *s *= tex->width;
+  *t *= tex->height;
 }
 
 static CoglTransformResult
 _cogl_texture_rectangle_transform_quad_coords_to_gl (CoglTexture *tex,
                                                      float *coords)
 {
-  CoglTextureRectangle *tex_rect = COGL_TEXTURE_RECTANGLE (tex);
   CoglBool need_repeat = FALSE;
   int i;
 
@@ -436,7 +506,7 @@ _cogl_texture_rectangle_transform_quad_coords_to_gl (CoglTexture *tex,
     {
       if (coords[i] < 0.0f || coords[i] > 1.0f)
         need_repeat = TRUE;
-      coords[i] *= (i & 1) ? tex_rect->height : tex_rect->width;
+      coords[i] *= (i & 1) ? tex->height : tex->width;
     }
 
   return (need_repeat ? COGL_TRANSFORM_SOFTWARE_REPEAT
@@ -460,23 +530,23 @@ _cogl_texture_rectangle_get_gl_texture (CoglTexture *tex,
 }
 
 static void
-_cogl_texture_rectangle_set_filters (CoglTexture *tex,
-                                     GLenum       min_filter,
-                                     GLenum       mag_filter)
+_cogl_texture_rectangle_gl_flush_legacy_texobj_filters (CoglTexture *tex,
+                                                        GLenum min_filter,
+                                                        GLenum mag_filter)
 {
   CoglTextureRectangle *tex_rect = COGL_TEXTURE_RECTANGLE (tex);
   CoglContext *ctx = tex->context;
 
-  if (min_filter == tex_rect->min_filter
-      && mag_filter == tex_rect->mag_filter)
+  if (min_filter == tex_rect->gl_legacy_texobj_min_filter
+      && mag_filter == tex_rect->gl_legacy_texobj_mag_filter)
     return;
 
   /* Rectangle textures don't support mipmapping */
   g_assert (min_filter == GL_LINEAR || min_filter == GL_NEAREST);
 
   /* Store new values */
-  tex_rect->min_filter = min_filter;
-  tex_rect->mag_filter = mag_filter;
+  tex_rect->gl_legacy_texobj_min_filter = min_filter;
+  tex_rect->gl_legacy_texobj_mag_filter = mag_filter;
 
   /* Apply new filters to the texture */
   _cogl_bind_gl_texture_transient (GL_TEXTURE_RECTANGLE_ARB,
@@ -503,48 +573,55 @@ _cogl_texture_rectangle_ensure_non_quad_rendering (CoglTexture *tex)
 }
 
 static CoglBool
-_cogl_texture_rectangle_set_region (CoglTexture    *tex,
-                                    int             src_x,
-                                    int             src_y,
-                                    int             dst_x,
-                                    int             dst_y,
-                                    unsigned int    dst_width,
-                                    unsigned int    dst_height,
-                                    CoglBitmap     *bmp)
+_cogl_texture_rectangle_set_region (CoglTexture *tex,
+                                    int src_x,
+                                    int src_y,
+                                    int dst_x,
+                                    int dst_y,
+                                    int dst_width,
+                                    int dst_height,
+                                    int level,
+                                    CoglBitmap *bmp,
+                                    CoglError **error)
 {
-  CoglTextureRectangle *tex_rect = COGL_TEXTURE_RECTANGLE (tex);
   GLenum gl_format;
   GLenum gl_type;
   CoglContext *ctx = tex->context;
+  CoglBool status;
 
   bmp = _cogl_texture_prepare_for_upload (bmp,
                                           cogl_texture_get_format (tex),
                                           NULL,
                                           NULL,
                                           &gl_format,
-                                          &gl_type);
+                                          &gl_type,
+                                          error);
+  if (!bmp)
+    return FALSE;
 
   /* Send data to GL */
-  ctx->texture_driver->upload_subregion_to_gl (ctx,
-                                               GL_TEXTURE_RECTANGLE_ARB,
-                                               tex_rect->gl_texture,
-                                               FALSE,
-                                               src_x, src_y,
-                                               dst_x, dst_y,
-                                               dst_width, dst_height,
-                                               bmp,
-                                               gl_format,
-                                               gl_type);
+  status =
+    ctx->texture_driver->upload_subregion_to_gl (ctx,
+                                                 tex,
+                                                 FALSE,
+                                                 src_x, src_y,
+                                                 dst_x, dst_y,
+                                                 dst_width, dst_height,
+                                                 level,
+                                                 bmp,
+                                                 gl_format,
+                                                 gl_type,
+                                                 error);
 
   cogl_object_unref (bmp);
 
-  return TRUE;
+  return status;
 }
 
 static CoglBool
 _cogl_texture_rectangle_get_data (CoglTexture *tex,
                                   CoglPixelFormat format,
-                                  unsigned int rowstride,
+                                  int rowstride,
                                   uint8_t *data)
 {
   CoglTextureRectangle *tex_rect = COGL_TEXTURE_RECTANGLE (tex);
@@ -563,7 +640,7 @@ _cogl_texture_rectangle_get_data (CoglTexture *tex,
 
   ctx->texture_driver->prep_gl_for_pixels_download (ctx,
                                                     rowstride,
-                                                    tex_rect->width,
+                                                    tex->width,
                                                     bpp);
 
   _cogl_bind_gl_texture_transient (GL_TEXTURE_RECTANGLE_ARB,
@@ -579,7 +656,7 @@ _cogl_texture_rectangle_get_data (CoglTexture *tex,
 static CoglPixelFormat
 _cogl_texture_rectangle_get_format (CoglTexture *tex)
 {
-  return COGL_TEXTURE_RECTANGLE (tex)->format;
+  return COGL_TEXTURE_RECTANGLE (tex)->internal_format;
 }
 
 static GLenum
@@ -588,18 +665,6 @@ _cogl_texture_rectangle_get_gl_format (CoglTexture *tex)
   return COGL_TEXTURE_RECTANGLE (tex)->gl_format;
 }
 
-static int
-_cogl_texture_rectangle_get_width (CoglTexture *tex)
-{
-  return COGL_TEXTURE_RECTANGLE (tex)->width;
-}
-
-static int
-_cogl_texture_rectangle_get_height (CoglTexture *tex)
-{
-  return COGL_TEXTURE_RECTANGLE (tex)->height;
-}
-
 static CoglBool
 _cogl_texture_rectangle_is_foreign (CoglTexture *tex)
 {
@@ -616,6 +681,7 @@ static const CoglTextureVtable
 cogl_texture_rectangle_vtable =
   {
     TRUE, /* primitive */
+    _cogl_texture_rectangle_allocate,
     _cogl_texture_rectangle_set_region,
     _cogl_texture_rectangle_get_data,
     NULL, /* foreach_sub_texture_in_region */
@@ -625,14 +691,12 @@ cogl_texture_rectangle_vtable =
     _cogl_texture_rectangle_transform_coords_to_gl,
     _cogl_texture_rectangle_transform_quad_coords_to_gl,
     _cogl_texture_rectangle_get_gl_texture,
-    _cogl_texture_rectangle_set_filters,
+    _cogl_texture_rectangle_gl_flush_legacy_texobj_filters,
     _cogl_texture_rectangle_pre_paint,
     _cogl_texture_rectangle_ensure_non_quad_rendering,
-    _cogl_texture_rectangle_set_wrap_mode_parameters,
+    _cogl_texture_rectangle_gl_flush_legacy_texobj_wrap_modes,
     _cogl_texture_rectangle_get_format,
     _cogl_texture_rectangle_get_gl_format,
-    _cogl_texture_rectangle_get_width,
-    _cogl_texture_rectangle_get_height,
     _cogl_texture_rectangle_get_type,
     _cogl_texture_rectangle_is_foreign,
     _cogl_texture_rectangle_set_auto_mipmap
