@@ -29,11 +29,31 @@
 #define __COGL_TYPES_H__
 
 #include <stdint.h>
-#include <glib-object.h>
+#include <stddef.h>
 
 #include <cogl/cogl-defines.h>
 
-G_BEGIN_DECLS
+#ifdef COGL_HAS_GTYPE_SUPPORT
+#include <glib-object.h>
+#endif /* COGL_HAS_GTYPE_SUPPORT */
+
+/* Guard C code in headers, while including them from C++ */
+#ifdef  __cplusplus
+#define COGL_BEGIN_DECLS  extern "C" {
+#define COGL_END_DECLS    }
+#else
+#define COGL_BEGIN_DECLS
+#define COGL_END_DECLS
+#endif
+
+COGL_BEGIN_DECLS
+
+/**
+ * SECTION:cogl-types
+ * @short_description: Types used throughout the library
+ *
+ * General types used by various Cogl functions.
+*/
 
 /**
  * CoglBool:
@@ -61,7 +81,7 @@ typedef int CoglBool;
 #endif
 
 /**
- * TRUE:
+ * FALSE:
  *
  * A constant to be used with #CoglBool types to indicate a boolean in
  * the "false" state.
@@ -73,6 +93,19 @@ typedef int CoglBool;
 #define FALSE 0
 #endif
 
+#if __GNUC__ >= 4
+#define COGL_GNUC_NULL_TERMINATED __attribute__((__sentinel__))
+#else
+#define COGL_GNUC_NULL_TERMINATED
+#endif
+
+#if __GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 1)
+#define COGL_GNUC_DEPRECATED                       \
+  __attribute__((__deprecated__))
+#else
+#define COGL_GNUC_DEPRECATED
+#endif /* __GNUC__ */
+
 /* Some structures are meant to be opaque but they have public
    definitions because we want the size to be public so they can be
    allocated on the stack. This macro is used to ensure that users
@@ -109,10 +142,14 @@ typedef void * CoglHandle;
  */
 #define COGL_INVALID_HANDLE NULL
 
+#ifdef COGL_HAS_GTYPE_SUPPORT
+
 #define COGL_TYPE_HANDLE        (cogl_handle_get_type ())
 GType
 cogl_handle_get_type (void) G_GNUC_CONST;
 
+#endif /* COGL_HAS_GTYPE_SUPPORT */
+
 /**
  * cogl_handle_ref:
  * @handle: a #CoglHandle
@@ -134,27 +171,6 @@ cogl_handle_ref (CoglHandle handle);
 void
 cogl_handle_unref (CoglHandle handle);
 
-/**
- * cogl_object_ref: (skip)
- * @object: a #CoglObject
- *
- * Increases the reference count of @object by 1
- *
- * Returns: the @object, with its reference count increased
- */
-void *
-cogl_object_ref (void *object);
-
-/**
- * cogl_object_unref: (skip)
- * @object: a #CoglObject
- *
- * Drecreases the reference count of @object by 1; if the reference
- * count reaches 0, the resources allocated by @object will be freed
- */
-void
-cogl_object_unref (void *object);
-
 /**
  * CoglFuncPtr:
  *
@@ -206,18 +222,22 @@ typedef struct _CoglTextureVertex       CoglTextureVertex;
 #define COGL_BGR_BIT            (1 << 5)
 #define COGL_AFIRST_BIT         (1 << 6)
 #define COGL_PREMULT_BIT        (1 << 7)
+#define COGL_DEPTH_BIT          (1 << 8)
+#define COGL_STENCIL_BIT        (1 << 9)
 
 /* XXX: Notes to those adding new formats here...
  *
  * First this diagram outlines how we allocate the 32bits of a
  * CoglPixelFormat currently...
  *
- *                             4 bits for flags
- *                             |--|
+ *                            6 bits for flags
+ *                          |-----|
  *  enum        unused             4 bits for the bytes-per-pixel
  *                                 and component alignment info
- *  |------| |---------------|     |--|
- *  00000000 xxxxxxxx xxxxxxxx PFBA0000
+ *  |------| |-------------|       |--|
+ *  00000000 xxxxxxxx xxxxxxSD PFBA0000
+ *                          ^ stencil
+ *                           ^ depth
  *                             ^ premult
  *                              ^ alpha first
  *                               ^ bgr order
@@ -239,7 +259,7 @@ typedef struct _CoglTextureVertex       CoglTextureVertex;
  * 4-6   = 2 bpp, not aligned (e.g. 565, 4444, 5551)
  * 7     = YUV: undefined bpp, undefined alignment
  * 9     = 2 bpp, aligned
- * 10    = undefined
+ * 10    = depth, aligned (8, 16, 24, 32, 32f)
  * 11    = undefined
  * 12    = 3 bpp, not aligned
  * 13    = 4 bpp, not aligned (e.g. 2101010)
@@ -357,7 +377,12 @@ typedef enum { /*< prefix=COGL_PIXEL_FORMAT >*/
   COGL_PIXEL_FORMAT_RGBA_1010102_PRE = (COGL_PIXEL_FORMAT_RGBA_1010102 | COGL_PREMULT_BIT),
   COGL_PIXEL_FORMAT_BGRA_1010102_PRE = (COGL_PIXEL_FORMAT_BGRA_1010102 | COGL_PREMULT_BIT),
   COGL_PIXEL_FORMAT_ARGB_2101010_PRE = (COGL_PIXEL_FORMAT_ARGB_2101010 | COGL_PREMULT_BIT),
-  COGL_PIXEL_FORMAT_ABGR_2101010_PRE = (COGL_PIXEL_FORMAT_ABGR_2101010 | COGL_PREMULT_BIT)
+  COGL_PIXEL_FORMAT_ABGR_2101010_PRE = (COGL_PIXEL_FORMAT_ABGR_2101010 | COGL_PREMULT_BIT),
+
+  COGL_PIXEL_FORMAT_DEPTH_16  = (9 | COGL_DEPTH_BIT),
+  COGL_PIXEL_FORMAT_DEPTH_32  = (3 | COGL_DEPTH_BIT),
+
+  COGL_PIXEL_FORMAT_DEPTH_24_STENCIL_8 = (3 | COGL_DEPTH_BIT | COGL_STENCIL_BIT)
 } CoglPixelFormat;
 
 /**
@@ -400,6 +425,8 @@ typedef enum { /*< prefix=COGL_PIXEL_FORMAT >*/
  *     supported with CoglBufferAccess including read support.
  * @COGL_FEATURE_MAP_BUFFER_FOR_WRITE: Whether cogl_buffer_map() is
  *     supported with CoglBufferAccess including write support.
+ * @COGL_FEATURE_DEPTH_TEXTURE: Whether #CoglFramebuffer support rendering the
+ *     depth buffer to a texture.
  *
  * Flags for the supported features.
  *
@@ -429,7 +456,8 @@ typedef enum
   COGL_FEATURE_SHADERS_ARBFP          = (1 << 20),
   COGL_FEATURE_MAP_BUFFER_FOR_READ    = (1 << 21),
   COGL_FEATURE_MAP_BUFFER_FOR_WRITE   = (1 << 22),
-  COGL_FEATURE_ONSCREEN_MULTIPLE      = (1 << 23)
+  COGL_FEATURE_ONSCREEN_MULTIPLE      = (1 << 23),
+  COGL_FEATURE_DEPTH_TEXTURE          = (1 << 24)
 } CoglFeatureFlags;
 
 /**
@@ -449,6 +477,10 @@ typedef enum
 
 /**
  * CoglColor:
+ * @red: amount of red
+ * @green: amount of green
+ * @blue: amount of green
+ * @alpha: alpha
  *
  * A structure for holding a color definition. The contents of
  * the CoglColor structure are private and should never by accessed
@@ -554,7 +586,7 @@ typedef enum {
 /**
  * COGL_BLEND_STRING_ERROR:
  *
- * #GError domain for blend string parser errors
+ * #CoglError domain for blend string parser errors
  *
  * Since: 1.0
  */
@@ -579,28 +611,28 @@ typedef enum { /*< prefix=COGL_BLEND_STRING_ERROR >*/
   COGL_BLEND_STRING_ERROR_GPU_UNSUPPORTED_ERROR
 } CoglBlendStringError;
 
-GQuark
+uint32_t
 cogl_blend_string_error_quark (void);
 
-#define COGL_ERROR (_cogl_error_quark ())
+#define COGL_SYSTEM_ERROR (_cogl_system_error_quark ())
 
 /**
- * CoglError:
- * @COGL_ERROR_UNSUPPORTED: You tried to use a feature or
+ * CoglSystemError:
+ * @COGL_SYSTEM_ERROR_UNSUPPORTED: You tried to use a feature or
  *    configuration not currently available.
- * @COGL_ERROR_NO_MEMORY: You tried to allocate a resource
+ * @COGL_SYSTEM_ERROR_NO_MEMORY: You tried to allocate a resource
  *    such as a texture and there wasn't enough memory.
  *
  * Error enumeration for Cogl
  *
- * The @COGL_ERROR_UNSUPPORTED error can be thrown for a variety of
- * reasons. For example:
+ * The @COGL_SYSTEM_ERROR_UNSUPPORTED error can be thrown for a
+ * variety of reasons. For example:
  *
  * <itemizedlist>
  *  <listitem><para>You've tried to use a feature that is not
- *   advertised by cogl_get_features(). This could happen if you create
+ *   advertised by cogl_has_feature(). This could happen if you create
  *   a 2d texture with a non-power-of-two size when
- *   %COGL_FEATURE_TEXTURE_NPOT is not advertised.</para></listitem>
+ *   %COGL_FEATURE_ID_TEXTURE_NPOT is not advertised.</para></listitem>
  *  <listitem><para>The GPU can not handle the configuration you have
  *   requested. An example might be if you try to use too many texture
  *   layers in a single #CoglPipeline</para></listitem>
@@ -615,12 +647,12 @@ cogl_blend_string_error_quark (void);
  * Stability: unstable
  */
 typedef enum { /*< prefix=COGL_ERROR >*/
-  COGL_ERROR_UNSUPPORTED,
-  COGL_ERROR_NO_MEMORY
-} CoglError;
+  COGL_SYSTEM_ERROR_UNSUPPORTED,
+  COGL_SYSTEM_ERROR_NO_MEMORY
+} CoglSystemError;
 
-GQuark
-_cogl_error_quark (void);
+uint32_t
+_cogl_system_error_quark (void);
 
 /**
  * CoglAttributeType:
@@ -655,7 +687,7 @@ typedef enum {
  * reduce the demand on memory bandwidth.
  *
  * Note that %COGL_INDICES_TYPE_UNSIGNED_INT is only supported if the
- * %COGL_FEATURE_UNSIGNED_INT_INDICES feature is available. This
+ * %COGL_FEATURE_ID_UNSIGNED_INT_INDICES feature is available. This
  * should always be available on OpenGL but on OpenGL ES it will only
  * be available if the GL_OES_element_index_uint extension is
  * advertized.
@@ -668,13 +700,18 @@ typedef enum {
 
 /**
  * CoglVerticesMode:
- * @COGL_VERTICES_MODE_POINTS: FIXME, equivalent to %GL_POINTS
- * @COGL_VERTICES_MODE_LINES: FIXME, equivalent to %GL_LINES
- * @COGL_VERTICES_MODE_LINE_LOOP: FIXME, equivalent to %GL_LINE_LOOP
- * @COGL_VERTICES_MODE_LINE_STRIP: FIXME, equivalent to %GL_LINE_STRIP
- * @COGL_VERTICES_MODE_TRIANGLES: FIXME, equivalent to %GL_TRIANGLES
- * @COGL_VERTICES_MODE_TRIANGLE_STRIP: FIXME, equivalent to %GL_TRIANGLE_STRIP
- * @COGL_VERTICES_MODE_TRIANGLE_FAN: FIXME, equivalent to %GL_TRIANGLE_FAN
+ * @COGL_VERTICES_MODE_POINTS: FIXME, equivalent to
+ * <constant>GL_POINTS</constant>
+ * @COGL_VERTICES_MODE_LINES: FIXME, equivalent to <constant>GL_LINES</constant>
+ * @COGL_VERTICES_MODE_LINE_LOOP: FIXME, equivalent to
+ * <constant>GL_LINE_LOOP</constant>
+ * @COGL_VERTICES_MODE_LINE_STRIP: FIXME, equivalent to
+ * <constant>GL_LINE_STRIP</constant>
+ * @COGL_VERTICES_MODE_TRIANGLES: FIXME, equivalent to
+ * <constant>GL_TRIANGLES</constant>
+ * @COGL_VERTICES_MODE_TRIANGLE_STRIP: FIXME, equivalent to
+ * <constant>GL_TRIANGLE_STRIP</constant>
+ * @COGL_VERTICES_MODE_TRIANGLE_FAN: FIXME, equivalent to <constant>GL_TRIANGLE_FAN</constant>
  *
  * Different ways of interpreting vertices when drawing.
  *
@@ -717,13 +754,12 @@ typedef enum {
  * When using depth testing one of these functions is used to compare
  * the depth of an incoming fragment against the depth value currently
  * stored in the depth buffer. The function is changed using
- * cogl_material_set_depth_test_function().
+ * cogl_depth_state_set_test_function().
  *
  * The test is only done when depth testing is explicitly enabled. (See
- * cogl_material_set_depth_test_enabled())
+ * cogl_depth_state_set_test_enabled())
  */
-typedef enum
-{
+typedef enum {
   COGL_DEPTH_TEST_FUNCTION_NEVER    = 0x0200,
   COGL_DEPTH_TEST_FUNCTION_LESS     = 0x0201,
   COGL_DEPTH_TEST_FUNCTION_EQUAL    = 0x0202,
@@ -736,18 +772,17 @@ typedef enum
 /* NB: The above definitions are taken from gl.h equivalents */
 
 typedef enum { /*< prefix=COGL_RENDERER_ERROR >*/
-  COGL_RENDERER_ERROR_NOT_FOUND,
   COGL_RENDERER_ERROR_XLIB_DISPLAY_OPEN,
   COGL_RENDERER_ERROR_BAD_CONSTRAINT
 } CoglRendererError;
 
-/*
+/**
  * CoglFilterReturn:
  * @COGL_FILTER_CONTINUE: The event was not handled, continues the
  *                        processing
  * @COGL_FILTER_REMOVE: Remove the event, stops the processing
  *
- * Return values for the #CoglFilterFunc function.
+ * Return values for the #CoglXlibFilterFunc and #CoglWin32FilterFunc functions.
  *
  * Stability: Unstable
  */
@@ -858,7 +893,7 @@ typedef enum {
  * CoglReadPixelsFlags:
  * @COGL_READ_PIXELS_COLOR_BUFFER: Read from the color buffer
  *
- * Flags for cogl_read_pixels()
+ * Flags for cogl_framebuffer_read_pixels_into_bitmap()
  *
  * Since: 1.0
  */
@@ -866,6 +901,6 @@ typedef enum { /*< prefix=COGL_READ_PIXELS >*/
   COGL_READ_PIXELS_COLOR_BUFFER = 1L << 0
 } CoglReadPixelsFlags;
 
-G_END_DECLS
+COGL_END_DECLS
 
 #endif /* __COGL_TYPES_H__ */
