@@ -33,7 +33,7 @@
 
 #include <wayland-client.h>
 
-G_BEGIN_DECLS
+COGL_BEGIN_DECLS
 
 /**
  * cogl_wayland_renderer_set_foreign_display:
@@ -63,7 +63,6 @@ cogl_wayland_renderer_set_foreign_display (CoglRenderer *renderer,
 /**
  * cogl_wayland_renderer_get_display:
  * @renderer: A #CoglRenderer
- * @display: A Wayland display
  *
  * Retrieves the Wayland display that Cogl is using. If a foreign
  * display has been specified using
@@ -159,6 +158,6 @@ cogl_wayland_renderer_set_foreign_shell (CoglRenderer *renderer,
 struct wl_shell *
 cogl_wayland_renderer_get_shell (CoglRenderer *renderer);
 
-G_END_DECLS
+COGL_END_DECLS
 
 #endif /* __COGL_WAYLAND_RENDERER_H__ */
