@@ -31,7 +31,6 @@
 #include <glib/gi18n-lib.h>
 
 #include "cogl-debug.h"
-#include "cogl-internal.h"
 #include "cogl-util.h"
 #include "cogl-context-private.h"
 #include "cogl-pipeline-private.h"
@@ -50,44 +49,8 @@
 #include "cogl-private.h"
 #include "cogl1-context.h"
 #include "cogl-offscreen.h"
-
-#ifdef COGL_GL_DEBUG
-/* GL error to string conversion */
-static const struct {
-  GLuint error_code;
-  const char *error_string;
-} gl_errors[] = {
-  { GL_NO_ERROR,          "No error" },
-  { GL_INVALID_ENUM,      "Invalid enumeration value" },
-  { GL_INVALID_VALUE,     "Invalid value" },
-  { GL_INVALID_OPERATION, "Invalid operation" },
-#ifdef HAVE_COGL_GL
-  { GL_STACK_OVERFLOW,    "Stack overflow" },
-  { GL_STACK_UNDERFLOW,   "Stack underflow" },
-#endif
-  { GL_OUT_OF_MEMORY,     "Out of memory" },
-
-#ifdef GL_INVALID_FRAMEBUFFER_OPERATION_EXT
-  { GL_INVALID_FRAMEBUFFER_OPERATION_EXT, "Invalid framebuffer operation" }
-#endif
-};
-
-static const unsigned int n_gl_errors = G_N_ELEMENTS (gl_errors);
-
-const char *
-_cogl_gl_error_to_string (GLenum error_code)
-{
-  int i;
-
-  for (i = 0; i < n_gl_errors; i++)
-    {
-      if (gl_errors[i].error_code == error_code)
-        return gl_errors[i].error_string;
-    }
-
-  return "Unknown GL error";
-}
-#endif /* COGL_GL_DEBUG */
+#include "cogl-attribute-gl-private.h"
+#include "cogl-clutter.h"
 
 CoglFuncPtr
 cogl_get_proc_address (const char* name)
@@ -98,26 +61,13 @@ cogl_get_proc_address (const char* name)
 }
 
 CoglBool
-_cogl_check_extension (const char *name, const gchar *ext)
+_cogl_check_extension (const char *name, char * const *ext)
 {
-  char *end;
-  int name_len, n;
-
-  if (name == NULL || ext == NULL)
-    return FALSE;
-
-  end = (char*)(ext + strlen(ext));
-
-  name_len = strlen(name);
-
-  while (ext < end)
-    {
-      n = strcspn(ext, " ");
-
-      if ((name_len == n) && (!strncmp(name, ext, n)))
-	return TRUE;
-      ext += (n + 1);
-    }
+  while (*ext)
+    if (!strcmp (name, *ext))
+      return TRUE;
+    else
+      ext++;
 
   return FALSE;
 }
@@ -126,7 +76,7 @@ _cogl_check_extension (const char *name, const gchar *ext)
 CoglBool
 cogl_check_extension (const char *name, const char *ext)
 {
-  return _cogl_check_extension (name, ext);
+  return cogl_clutter_check_extension (name, ext);
 }
 
 /* XXX: it's expected that we'll deprecated this with
@@ -442,11 +392,10 @@ cogl_begin_gl (void)
   pipeline = cogl_get_source ();
   _cogl_pipeline_flush_gl_state (pipeline,
                                  cogl_get_draw_framebuffer (),
-                                 FALSE,
-                                 cogl_pipeline_get_n_layers (pipeline));
+                                 FALSE);
 
   /* Disable any cached vertex arrays */
-  _cogl_attribute_disable_cached_arrays ();
+  _cogl_gl_disable_all_attributes (ctx);
 }
 
 void
@@ -559,7 +508,7 @@ cogl_set_projection_matrix (CoglMatrix *matrix)
   cogl_framebuffer_set_projection_matrix (cogl_get_draw_framebuffer (), matrix);
 }
 
-GQuark
+uint32_t
 _cogl_driver_error_quark (void)
 {
   return g_quark_from_static_string ("cogl-driver-error-quark");
@@ -784,27 +733,29 @@ _cogl_transform_point (const CoglMatrix *matrix_mv,
 #undef VIEWPORT_TRANSFORM_X
 #undef VIEWPORT_TRANSFORM_Y
 
-GQuark
-_cogl_error_quark (void)
+uint32_t
+_cogl_system_error_quark (void)
 {
-  return g_quark_from_static_string ("cogl-error-quark");
+  return g_quark_from_static_string ("cogl-system-error-quark");
 }
 
 void
 _cogl_init (void)
 {
-  static size_t init_status = 0;
+  static CoglBool initialized = FALSE;
 
-  if (g_once_init_enter (&init_status))
+  if (initialized == FALSE)
     {
       bindtextdomain (GETTEXT_PACKAGE, COGL_LOCALEDIR);
       bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
+#ifdef COGL_HAS_GTYPE_SUPPORT
       g_type_init ();
+#endif
 
       _cogl_config_read ();
       _cogl_debug_check_environment ();
-      g_once_init_leave (&init_status, 1);
+      initialized = TRUE;
     }
 }
 
