@@ -31,7 +31,6 @@
 #endif
 
 #include "cogl-debug.h"
-#include "cogl-internal.h"
 #include "cogl-util.h"
 #include "cogl-texture-pixmap-x11.h"
 #include "cogl-texture-pixmap-x11-private.h"
@@ -47,6 +46,9 @@
 #include "cogl-winsys-private.h"
 #include "cogl-pipeline-opengl-private.h"
 #include "cogl-xlib.h"
+#include "cogl-error-private.h"
+#include "cogl-texture-gl-private.h"
+#include "cogl-private.h"
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -64,7 +66,7 @@ COGL_TEXTURE_DEFINE (TexturePixmapX11, texture_pixmap_x11);
 
 static const CoglTextureVtable cogl_texture_pixmap_x11_vtable;
 
-GQuark
+uint32_t
 cogl_texture_pixmap_x11_error_quark (void)
 {
   return g_quark_from_static_string ("cogl-texture-pixmap-error-quark");
@@ -123,13 +125,14 @@ static void
 process_damage_event (CoglTexturePixmapX11 *tex_pixmap,
                       XDamageNotifyEvent *damage_event)
 {
+  CoglTexture *tex = COGL_TEXTURE (tex_pixmap);
   Display *display;
   enum { DO_NOTHING, NEEDS_SUBTRACT, NEED_BOUNDING_BOX } handle_mode;
   const CoglWinsysVtable *winsys;
 
   _COGL_GET_CONTEXT (ctxt, NO_RETVAL);
 
-  display = cogl_xlib_get_display ();
+  display = cogl_xlib_renderer_get_display (ctxt->display->renderer);
 
   COGL_NOTE (TEXTURE_PIXMAP, "Damage event received for %p", tex_pixmap);
 
@@ -165,8 +168,8 @@ process_damage_event (CoglTexturePixmapX11 *tex_pixmap,
      need to request the bounding box of the region because we're
      going to update the whole texture anyway. */
   if (cogl_damage_rectangle_is_whole (&tex_pixmap->damage_rect,
-                                      tex_pixmap->width,
-                                      tex_pixmap->height))
+                                      tex->width,
+                                      tex->height))
     {
       if (handle_mode != DO_NOTHING)
         XDamageSubtract (display, tex_pixmap->damage, None, None);
@@ -247,6 +250,8 @@ set_damage_object_internal (CoglContext *ctx,
                             Damage damage,
                             CoglTexturePixmapX11ReportLevel report_level)
 {
+  Display *display = cogl_xlib_renderer_get_display (ctx->display->renderer);
+
   if (tex_pixmap->damage)
     {
       cogl_xlib_renderer_remove_filter (ctx->display->renderer,
@@ -255,7 +260,7 @@ set_damage_object_internal (CoglContext *ctx,
 
       if (tex_pixmap->damage_owned)
         {
-          XDamageDestroy (cogl_xlib_get_display (), tex_pixmap->damage);
+          XDamageDestroy (display, tex_pixmap->damage);
           tex_pixmap->damage_owned = FALSE;
         }
     }
@@ -273,19 +278,34 @@ CoglTexturePixmapX11 *
 cogl_texture_pixmap_x11_new (CoglContext *ctxt,
                              uint32_t pixmap,
                              CoglBool automatic_updates,
-                             GError **error)
+                             CoglError **error)
 {
   CoglTexturePixmapX11 *tex_pixmap = g_new (CoglTexturePixmapX11, 1);
-  Display *display = cogl_xlib_get_display ();
+  Display *display = cogl_xlib_renderer_get_display (ctxt->display->renderer);
   Window pixmap_root_window;
   int pixmap_x, pixmap_y;
+  unsigned int pixmap_width, pixmap_height;
   unsigned int pixmap_border_width;
   CoglTexture *tex = COGL_TEXTURE (tex_pixmap);
   XWindowAttributes window_attributes;
   int damage_base;
   const CoglWinsysVtable *winsys;
 
-  _cogl_texture_init (tex, ctxt, &cogl_texture_pixmap_x11_vtable);
+  if (!XGetGeometry (display, pixmap, &pixmap_root_window,
+                     &pixmap_x, &pixmap_y,
+                     &pixmap_width, &pixmap_height,
+                     &pixmap_border_width, &tex_pixmap->depth))
+    {
+      g_free (tex_pixmap);
+      _cogl_set_error (error,
+                   COGL_TEXTURE_PIXMAP_X11_ERROR,
+                   COGL_TEXTURE_PIXMAP_X11_ERROR_X11,
+                   "Unable to query pixmap size");
+      return NULL;
+    }
+
+  _cogl_texture_init (tex, ctxt, pixmap_width, pixmap_height,
+                      &cogl_texture_pixmap_x11_vtable);
 
   tex_pixmap->pixmap = pixmap;
   tex_pixmap->image = NULL;
@@ -294,25 +314,12 @@ cogl_texture_pixmap_x11_new (CoglContext *ctxt,
   tex_pixmap->damage_owned = FALSE;
   tex_pixmap->damage = 0;
 
-  if (!XGetGeometry (display, pixmap, &pixmap_root_window,
-                     &pixmap_x, &pixmap_y,
-                     &tex_pixmap->width, &tex_pixmap->height,
-                     &pixmap_border_width, &tex_pixmap->depth))
-    {
-      g_free (tex_pixmap);
-      g_set_error (error,
-                   COGL_TEXTURE_PIXMAP_X11_ERROR,
-                   COGL_TEXTURE_PIXMAP_X11_ERROR_X11,
-                   "Unable to query pixmap size");
-      return NULL;
-    }
-
   /* We need a visual to use for shared memory images so we'll query
      it from the pixmap's root window */
   if (!XGetWindowAttributes (display, pixmap_root_window, &window_attributes))
     {
       g_free (tex_pixmap);
-      g_set_error (error,
+      _cogl_set_error (error,
                    COGL_TEXTURE_PIXMAP_X11_ERROR,
                    COGL_TEXTURE_PIXMAP_X11_ERROR_X11,
                    "Unable to query root window attributes");
@@ -338,9 +345,9 @@ cogl_texture_pixmap_x11_new (CoglContext *ctxt,
 
   /* Assume the entire pixmap is damaged to begin with */
   tex_pixmap->damage_rect.x1 = 0;
-  tex_pixmap->damage_rect.x2 = tex_pixmap->width;
+  tex_pixmap->damage_rect.x2 = tex->width;
   tex_pixmap->damage_rect.y1 = 0;
-  tex_pixmap->damage_rect.y2 = tex_pixmap->height;
+  tex_pixmap->damage_rect.y2 = tex->height;
 
   winsys = _cogl_texture_pixmap_x11_get_winsys (tex_pixmap);
   if (winsys->texture_pixmap_x11_create)
@@ -354,18 +361,30 @@ cogl_texture_pixmap_x11_new (CoglContext *ctxt,
   if (!tex_pixmap->use_winsys_texture)
     tex_pixmap->winsys = NULL;
 
+  _cogl_texture_set_allocated (tex, TRUE);
+
   return _cogl_texture_pixmap_x11_object_new (tex_pixmap);
 }
 
+static CoglBool
+_cogl_texture_pixmap_x11_allocate (CoglTexture *tex,
+                                   CoglError **error)
+{
+  return TRUE;
+}
+
 /* Tries to allocate enough shared mem to handle a full size
  * update size of the X Pixmap. */
 static void
 try_alloc_shm (CoglTexturePixmapX11 *tex_pixmap)
 {
+  CoglTexture *tex = COGL_TEXTURE (tex_pixmap);
   XImage *dummy_image;
   Display *display;
 
-  display = cogl_xlib_get_display ();
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  display = cogl_xlib_renderer_get_display (ctx->display->renderer);
 
   if (!XShmQueryExtension (display))
     return;
@@ -386,8 +405,8 @@ try_alloc_shm (CoglTexturePixmapX11 *tex_pixmap)
                      ZPixmap,
                      NULL,
                      NULL, /* shminfo, */
-                     tex_pixmap->width,
-                     tex_pixmap->height);
+                     tex->width,
+                     tex->height);
   if (!dummy_image)
     goto failed_image_create;
 
@@ -473,14 +492,20 @@ cogl_texture_pixmap_x11_set_damage_object (CoglTexturePixmapX11 *tex_pixmap,
 static void
 _cogl_texture_pixmap_x11_update_image_texture (CoglTexturePixmapX11 *tex_pixmap)
 {
+  CoglTexture *tex = COGL_TEXTURE (tex_pixmap);
   Display *display;
   Visual *visual;
   CoglPixelFormat image_format;
   XImage *image;
   int src_x, src_y;
   int x, y, width, height;
+  int bpp;
+  int offset;
+  CoglError *ignore = NULL;
 
-  display = cogl_xlib_get_display ();
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  display = cogl_xlib_renderer_get_display (ctx->display->renderer);
   visual = tex_pixmap->visual;
 
   /* If the damage region is empty then there's nothing to do */
@@ -503,8 +528,8 @@ _cogl_texture_pixmap_x11_update_image_texture (CoglTexturePixmapX11 *tex_pixmap)
                         ? COGL_PIXEL_FORMAT_RGBA_8888_PRE
                         : COGL_PIXEL_FORMAT_RGB_888);
 
-      tex_pixmap->tex = cogl_texture_new_with_size (tex_pixmap->width,
-                                                    tex_pixmap->height,
+      tex_pixmap->tex = cogl_texture_new_with_size (tex->width,
+                                                    tex->height,
                                                     COGL_TEXTURE_NONE,
                                                     texture_format);
     }
@@ -529,7 +554,7 @@ _cogl_texture_pixmap_x11_update_image_texture (CoglTexturePixmapX11 *tex_pixmap)
           tex_pixmap->image = XGetImage (display,
                                          tex_pixmap->pixmap,
                                          0, 0,
-                                         tex_pixmap->width, tex_pixmap->height,
+                                         tex->width, tex->height,
                                          AllPlanes, ZPixmap);
           image = tex_pixmap->image;
           src_x = x;
@@ -583,14 +608,18 @@ _cogl_texture_pixmap_x11_update_image_texture (CoglTexturePixmapX11 *tex_pixmap)
                                         image->bits_per_pixel,
                                         image->byte_order == LSBFirst);
 
-  cogl_texture_set_region (tex_pixmap->tex,
-                           src_x, src_y,
-                           x, y, width, height,
-                           image->width,
-                           image->height,
-                           image_format,
-                           image->bytes_per_line,
-                           (const uint8_t *) image->data);
+  bpp = _cogl_pixel_format_get_bytes_per_pixel (image_format);
+  offset = image->bytes_per_line * src_y + bpp * src_x;
+
+  _cogl_texture_set_region (tex_pixmap->tex,
+                            width,
+                            height,
+                            image_format,
+                            image->bytes_per_line,
+                            ((const uint8_t *) image->data) + offset,
+                            x, y,
+                            0, /* level */
+                            &ignore);
 
   /* If we have a shared memory segment then the XImage would be a
      temporary one with no data allocated so we can just XFree it */
@@ -676,24 +705,30 @@ _cogl_texture_pixmap_x11_get_texture (CoglTexturePixmapX11 *tex_pixmap)
 }
 
 static CoglBool
-_cogl_texture_pixmap_x11_set_region (CoglTexture     *tex,
-                                     int              src_x,
-                                     int              src_y,
-                                     int              dst_x,
-                                     int              dst_y,
-                                     unsigned int     dst_width,
-                                     unsigned int     dst_height,
-                                     CoglBitmap      *bmp)
+_cogl_texture_pixmap_x11_set_region (CoglTexture *tex,
+                                     int src_x,
+                                     int src_y,
+                                     int dst_x,
+                                     int dst_y,
+                                     int dst_width,
+                                     int dst_height,
+                                     int level,
+                                     CoglBitmap *bmp,
+                                     CoglError **error)
 {
   /* This doesn't make much sense for texture from pixmap so it's not
      supported */
+  _cogl_set_error (error,
+                   COGL_SYSTEM_ERROR,
+                   COGL_SYSTEM_ERROR_UNSUPPORTED,
+                   "Explicitly setting a region of a TFP texture unsupported");
   return FALSE;
 }
 
 static CoglBool
 _cogl_texture_pixmap_x11_get_data (CoglTexture *tex,
                                    CoglPixelFormat format,
-                                   unsigned int rowstride,
+                                   int rowstride,
                                    uint8_t *data)
 {
   CoglTexturePixmapX11 *tex_pixmap = COGL_TEXTURE_PIXMAP_X11 (tex);
@@ -752,8 +787,8 @@ _cogl_texture_pixmap_x11_foreach_sub_texture_in_region
   if (cogl_is_texture_rectangle (child_tex))
     {
       NormalizeCoordsWrapperData data;
-      int width = tex_pixmap->width;
-      int height = tex_pixmap->height;
+      int width = tex->width;
+      int height = tex->height;
 
       virtual_tx_1 *= width;
       virtual_ty_1 *= height;
@@ -852,15 +887,16 @@ _cogl_texture_pixmap_x11_get_gl_texture (CoglTexture *tex,
 }
 
 static void
-_cogl_texture_pixmap_x11_set_filters (CoglTexture *tex,
-                                      GLenum       min_filter,
-                                      GLenum       mag_filter)
+_cogl_texture_pixmap_x11_gl_flush_legacy_texobj_filters (CoglTexture *tex,
+                                                         GLenum min_filter,
+                                                         GLenum mag_filter)
 {
   CoglTexturePixmapX11 *tex_pixmap = COGL_TEXTURE_PIXMAP_X11 (tex);
   CoglTexture *child_tex = _cogl_texture_pixmap_x11_get_texture (tex_pixmap);
 
   /* Forward on to the child texture */
-  _cogl_texture_set_filters (child_tex, min_filter, mag_filter);
+  _cogl_texture_gl_flush_legacy_texobj_filters (child_tex,
+                                                min_filter, mag_filter);
 }
 
 static void
@@ -889,19 +925,19 @@ _cogl_texture_pixmap_x11_ensure_non_quad_rendering (CoglTexture *tex)
 }
 
 static void
-_cogl_texture_pixmap_x11_set_wrap_mode_parameters (CoglTexture *tex,
-                                                   GLenum       wrap_mode_s,
-                                                   GLenum       wrap_mode_t,
-                                                   GLenum       wrap_mode_p)
+_cogl_texture_pixmap_x11_gl_flush_legacy_texobj_wrap_modes (CoglTexture *tex,
+                                                            GLenum wrap_mode_s,
+                                                            GLenum wrap_mode_t,
+                                                            GLenum wrap_mode_p)
 {
   CoglTexturePixmapX11 *tex_pixmap = COGL_TEXTURE_PIXMAP_X11 (tex);
   CoglTexture *child_tex = _cogl_texture_pixmap_x11_get_texture (tex_pixmap);
 
   /* Forward on to the child texture */
-  _cogl_texture_set_wrap_mode_parameters (child_tex,
-                                          wrap_mode_s,
-                                          wrap_mode_t,
-                                          wrap_mode_p);
+  _cogl_texture_gl_flush_legacy_texobj_wrap_modes (child_tex,
+                                                   wrap_mode_s,
+                                                   wrap_mode_t,
+                                                   wrap_mode_p);
 }
 
 static CoglPixelFormat
@@ -920,23 +956,7 @@ _cogl_texture_pixmap_x11_get_gl_format (CoglTexture *tex)
   CoglTexturePixmapX11 *tex_pixmap = COGL_TEXTURE_PIXMAP_X11 (tex);
   CoglTexture *child_tex = _cogl_texture_pixmap_x11_get_texture (tex_pixmap);
 
-  return _cogl_texture_get_gl_format (child_tex);
-}
-
-static int
-_cogl_texture_pixmap_x11_get_width (CoglTexture *tex)
-{
-  CoglTexturePixmapX11 *tex_pixmap = COGL_TEXTURE_PIXMAP_X11 (tex);
-
-  return tex_pixmap->width;
-}
-
-static int
-_cogl_texture_pixmap_x11_get_height (CoglTexture *tex)
-{
-  CoglTexturePixmapX11 *tex_pixmap = COGL_TEXTURE_PIXMAP_X11 (tex);
-
-  return tex_pixmap->height;
+  return _cogl_texture_gl_get_format (child_tex);
 }
 
 static CoglTextureType
@@ -954,8 +974,12 @@ _cogl_texture_pixmap_x11_get_type (CoglTexture *tex)
 static void
 _cogl_texture_pixmap_x11_free (CoglTexturePixmapX11 *tex_pixmap)
 {
+  Display *display;
+
   _COGL_GET_CONTEXT (ctxt, NO_RETVAL);
 
+  display = cogl_xlib_renderer_get_display (ctxt->display->renderer);
+
   set_damage_object_internal (ctxt, tex_pixmap, 0, 0);
 
   if (tex_pixmap->image)
@@ -963,7 +987,7 @@ _cogl_texture_pixmap_x11_free (CoglTexturePixmapX11 *tex_pixmap)
 
   if (tex_pixmap->shm_info.shmid != -1)
     {
-      XShmDetach (cogl_xlib_get_display (), &tex_pixmap->shm_info);
+      XShmDetach (display, &tex_pixmap->shm_info);
       shmdt (tex_pixmap->shm_info.shmaddr);
       shmctl (tex_pixmap->shm_info.shmid, IPC_RMID, 0);
     }
@@ -986,6 +1010,7 @@ static const CoglTextureVtable
 cogl_texture_pixmap_x11_vtable =
   {
     FALSE, /* not primitive */
+    _cogl_texture_pixmap_x11_allocate,
     _cogl_texture_pixmap_x11_set_region,
     _cogl_texture_pixmap_x11_get_data,
     _cogl_texture_pixmap_x11_foreach_sub_texture_in_region,
@@ -995,14 +1020,12 @@ cogl_texture_pixmap_x11_vtable =
     _cogl_texture_pixmap_x11_transform_coords_to_gl,
     _cogl_texture_pixmap_x11_transform_quad_coords_to_gl,
     _cogl_texture_pixmap_x11_get_gl_texture,
-    _cogl_texture_pixmap_x11_set_filters,
+    _cogl_texture_pixmap_x11_gl_flush_legacy_texobj_filters,
     _cogl_texture_pixmap_x11_pre_paint,
     _cogl_texture_pixmap_x11_ensure_non_quad_rendering,
-    _cogl_texture_pixmap_x11_set_wrap_mode_parameters,
+    _cogl_texture_pixmap_x11_gl_flush_legacy_texobj_wrap_modes,
     _cogl_texture_pixmap_x11_get_format,
     _cogl_texture_pixmap_x11_get_gl_format,
-    _cogl_texture_pixmap_x11_get_width,
-    _cogl_texture_pixmap_x11_get_height,
     _cogl_texture_pixmap_x11_get_type,
     NULL, /* is_foreign */
     NULL /* set_auto_mipmap */
