@@ -39,6 +39,7 @@
 #include "cogl-renderer-private.h"
 #include "cogl-onscreen-template-private.h"
 #include "cogl-gles2-context-private.h"
+#include "cogl-error-private.h"
 
 #include "cogl-private.h"
 
@@ -147,9 +148,11 @@ check_egl_extensions (CoglRenderer *renderer)
 {
   CoglRendererEGL *egl_renderer = renderer->winsys;
   const char *egl_extensions;
+  char **split_extensions;
   int i;
 
   egl_extensions = eglQueryString (egl_renderer->edpy, EGL_EXTENSIONS);
+  split_extensions = g_strsplit (egl_extensions, " ", 0 /* max_tokens */);
 
   COGL_NOTE (WINSYS, "  EGL Extensions: %s", egl_extensions);
 
@@ -158,17 +161,19 @@ check_egl_extensions (CoglRenderer *renderer)
     if (_cogl_feature_check (renderer,
                              "EGL", winsys_feature_data + i, 0, 0,
                              COGL_DRIVER_GL, /* the driver isn't used */
-                             egl_extensions,
+                             split_extensions,
                              egl_renderer))
       {
         egl_renderer->private_features |=
           winsys_feature_data[i].feature_flags_private;
       }
+
+  g_strfreev (split_extensions);
 }
 
 CoglBool
 _cogl_winsys_egl_renderer_connect_common (CoglRenderer *renderer,
-                                          GError **error)
+                                          CoglError **error)
 {
   CoglRendererEGL *egl_renderer = renderer->winsys;
 
@@ -176,7 +181,7 @@ _cogl_winsys_egl_renderer_connect_common (CoglRenderer *renderer,
                       &egl_renderer->egl_version_major,
                       &egl_renderer->egl_version_minor))
     {
-      g_set_error (error, COGL_WINSYS_ERROR,
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_INIT,
                    "Couldn't initialize EGL");
       return FALSE;
@@ -189,7 +194,7 @@ _cogl_winsys_egl_renderer_connect_common (CoglRenderer *renderer,
 
 static CoglBool
 _cogl_winsys_renderer_connect (CoglRenderer *renderer,
-                               GError **error)
+                               CoglError **error)
 {
   /* This function must be overridden by a platform winsys */
   g_assert_not_reached ();
@@ -198,7 +203,6 @@ _cogl_winsys_renderer_connect (CoglRenderer *renderer,
 static void
 egl_attributes_from_framebuffer_config (CoglDisplay *display,
                                         CoglFramebufferConfig *config,
-                                        CoglBool needs_stencil_override,
                                         EGLint *attributes)
 {
   CoglRenderer *renderer = display->renderer;
@@ -211,8 +215,11 @@ egl_attributes_from_framebuffer_config (CoglDisplay *display,
                                                               config,
                                                               attributes);
 
-  attributes[i++] = EGL_STENCIL_SIZE;
-  attributes[i++] = needs_stencil_override ? 2 : 0;
+  if (config->need_stencil)
+    {
+      attributes[i++] = EGL_STENCIL_SIZE;
+      attributes[i++] = 2;
+    }
 
   attributes[i++] = EGL_RED_SIZE;
   attributes[i++] = 1;
@@ -231,11 +238,12 @@ egl_attributes_from_framebuffer_config (CoglDisplay *display,
   attributes[i++] = EGL_DONT_CARE;
 
   attributes[i++] = EGL_RENDERABLE_TYPE;
-  attributes[i++] = (renderer->driver == COGL_DRIVER_GL ?
-                      EGL_OPENGL_BIT :
-                      renderer->driver == COGL_DRIVER_GLES1 ?
-                      EGL_OPENGL_ES_BIT :
-                      EGL_OPENGL_ES2_BIT);
+  attributes[i++] = ((renderer->driver == COGL_DRIVER_GL ||
+                      renderer->driver == COGL_DRIVER_GL3) ?
+                     EGL_OPENGL_BIT :
+                     renderer->driver == COGL_DRIVER_GLES1 ?
+                     EGL_OPENGL_ES_BIT :
+                     EGL_OPENGL_ES2_BIT);
 
   attributes[i++] = EGL_SURFACE_TYPE;
   attributes[i++] = EGL_WINDOW_BIT;
@@ -253,78 +261,6 @@ egl_attributes_from_framebuffer_config (CoglDisplay *display,
   g_assert (i < MAX_EGL_CONFIG_ATTRIBS);
 }
 
-static CoglBool
-try_create_context (CoglDisplay *display,
-                    CoglBool with_stencil_buffer,
-                    GError **error)
-{
-  CoglRenderer *renderer = display->renderer;
-  CoglDisplayEGL *egl_display = display->winsys;
-  CoglRendererEGL *egl_renderer = renderer->winsys;
-  EGLDisplay edpy;
-  EGLConfig config;
-  EGLint config_count = 0;
-  EGLBoolean status;
-  EGLint attribs[3];
-  EGLint cfg_attribs[MAX_EGL_CONFIG_ATTRIBS];
-  const char *error_message;
-
-  _COGL_RETURN_VAL_IF_FAIL (egl_display->egl_context == NULL, TRUE);
-
-  if (renderer->driver == COGL_DRIVER_GL)
-    eglBindAPI (EGL_OPENGL_API);
-
-  if (display->renderer->driver == COGL_DRIVER_GLES2)
-    {
-      attribs[0] = EGL_CONTEXT_CLIENT_VERSION;
-      attribs[1] = 2;
-      attribs[2] = EGL_NONE;
-    }
-  else
-    attribs[0] = EGL_NONE;
-
-  egl_attributes_from_framebuffer_config (display,
-                                          &display->onscreen_template->config,
-                                          with_stencil_buffer,
-                                          cfg_attribs);
-
-  edpy = egl_renderer->edpy;
-
-  status = eglChooseConfig (edpy,
-                            cfg_attribs,
-                            &config, 1,
-                            &config_count);
-  if (status != EGL_TRUE || config_count == 0)
-    {
-      error_message = "Unable to find a usable EGL configuration";
-      goto fail;
-    }
-
-  egl_display->egl_config = config;
-
-  egl_display->egl_context = eglCreateContext (edpy,
-                                               config,
-                                               EGL_NO_CONTEXT,
-                                               attribs);
-  if (egl_display->egl_context == EGL_NO_CONTEXT)
-    {
-      error_message = "Unable to create a suitable EGL context";
-      goto fail;
-    }
-
-  if (egl_renderer->platform_vtable->context_created &&
-      !egl_renderer->platform_vtable->context_created (display, error))
-    return FALSE;
-
-  return TRUE;
-
-fail:
-  g_set_error (error, COGL_WINSYS_ERROR,
-               COGL_WINSYS_ERROR_CREATE_CONTEXT,
-               "%s", error_message);
-  return FALSE;
-}
-
 EGLBoolean
 _cogl_winsys_egl_make_current (CoglDisplay *display,
                                EGLSurface draw,
@@ -373,28 +309,98 @@ cleanup_context (CoglDisplay *display)
 }
 
 static CoglBool
-create_context (CoglDisplay *display, GError **error)
+try_create_context (CoglDisplay *display,
+                    CoglError **error)
 {
+  CoglRenderer *renderer = display->renderer;
   CoglDisplayEGL *egl_display = display->winsys;
+  CoglRendererEGL *egl_renderer = renderer->winsys;
+  EGLDisplay edpy;
+  EGLConfig config;
+  EGLint config_count = 0;
+  EGLBoolean status;
+  EGLint attribs[9];
+  EGLint cfg_attribs[MAX_EGL_CONFIG_ATTRIBS];
+  const char *error_message;
 
-  /* Note: we don't just rely on eglChooseConfig to correctly
-   * report that the driver doesn't support a stencil buffer
-   * because we've seen PVR drivers that claim stencil buffer
-   * support according to the EGLConfig but then later fail
-   * when trying to create a context with such a config.
-   */
-  if (try_create_context (display, TRUE, error))
+  _COGL_RETURN_VAL_IF_FAIL (egl_display->egl_context == NULL, TRUE);
+
+  if (renderer->driver == COGL_DRIVER_GL ||
+      renderer->driver == COGL_DRIVER_GL3)
+    eglBindAPI (EGL_OPENGL_API);
+
+  egl_attributes_from_framebuffer_config (display,
+                                          &display->onscreen_template->config,
+                                          cfg_attribs);
+
+  edpy = egl_renderer->edpy;
+
+  status = eglChooseConfig (edpy,
+                            cfg_attribs,
+                            &config, 1,
+                            &config_count);
+  if (status != EGL_TRUE || config_count == 0)
     {
-      egl_display->stencil_disabled = FALSE;
-      return TRUE;
+      error_message = "Unable to find a usable EGL configuration";
+      goto fail;
+    }
+
+  egl_display->egl_config = config;
+
+  if (display->renderer->driver == COGL_DRIVER_GL3)
+    {
+      if (!(egl_renderer->private_features &
+            COGL_EGL_WINSYS_FEATURE_CREATE_CONTEXT))
+        {
+          error_message = "Driver does not support GL 3 contexts";
+          goto fail;
+        }
+
+      /* Try to get a core profile 3.1 context with no deprecated features */
+      attribs[0] = EGL_CONTEXT_MAJOR_VERSION_KHR;
+      attribs[1] = 3;
+      attribs[2] = EGL_CONTEXT_MINOR_VERSION_KHR;
+      attribs[3] = 1;
+      attribs[4] = EGL_CONTEXT_FLAGS_KHR;
+      attribs[5] = EGL_CONTEXT_OPENGL_FORWARD_COMPATIBLE_BIT_KHR;
+      attribs[6] = EGL_CONTEXT_OPENGL_PROFILE_MASK_KHR;
+      attribs[7] = EGL_CONTEXT_OPENGL_CORE_PROFILE_BIT_KHR;
+      attribs[8] = EGL_NONE;
+    }
+  else if (display->renderer->driver == COGL_DRIVER_GLES2)
+    {
+      attribs[0] = EGL_CONTEXT_CLIENT_VERSION;
+      attribs[1] = 2;
+      attribs[2] = EGL_NONE;
     }
   else
+    attribs[0] = EGL_NONE;
+
+  egl_display->egl_context = eglCreateContext (edpy,
+                                               config,
+                                               EGL_NO_CONTEXT,
+                                               attribs);
+
+  if (egl_display->egl_context == EGL_NO_CONTEXT)
     {
-      g_clear_error (error);
-      cleanup_context (display);
-      egl_display->stencil_disabled = TRUE;
-      return try_create_context (display, FALSE, error);
+      error_message = "Unable to create a suitable EGL context";
+      goto fail;
     }
+
+  if (egl_renderer->platform_vtable->context_created &&
+      !egl_renderer->platform_vtable->context_created (display, error))
+    return FALSE;
+
+  return TRUE;
+
+fail:
+  _cogl_set_error (error, COGL_WINSYS_ERROR,
+               COGL_WINSYS_ERROR_CREATE_CONTEXT,
+               "%s", error_message);
+
+  cleanup_context (display);
+
+  return FALSE;
 }
 
 static void
@@ -416,7 +422,7 @@ _cogl_winsys_display_destroy (CoglDisplay *display)
 
 static CoglBool
 _cogl_winsys_display_setup (CoglDisplay *display,
-                            GError **error)
+                            CoglError **error)
 {
   CoglDisplayEGL *egl_display;
   CoglRenderer *renderer = display->renderer;
@@ -442,7 +448,7 @@ _cogl_winsys_display_setup (CoglDisplay *display,
       !egl_renderer->platform_vtable->display_setup (display, error))
     goto error;
 
-  if (!create_context (display, error))
+  if (!try_create_context (display, error))
     goto error;
 
   egl_display->found_egl_config = TRUE;
@@ -455,7 +461,7 @@ error:
 }
 
 static CoglBool
-_cogl_winsys_context_init (CoglContext *context, GError **error)
+_cogl_winsys_context_init (CoglContext *context, CoglError **error)
 {
   CoglRenderer *renderer = context->display->renderer;
   CoglDisplayEGL *egl_display = context->display->winsys;
@@ -514,7 +520,7 @@ typedef struct _CoglGLES2ContextEGL
 } CoglGLES2ContextEGL;
 
 static void *
-_cogl_winsys_context_create_gles2_context (CoglContext *ctx, GError **error)
+_cogl_winsys_context_create_gles2_context (CoglContext *ctx, CoglError **error)
 {
   CoglRendererEGL *egl_renderer = ctx->display->renderer->winsys;
   CoglDisplayEGL *egl_display = ctx->display->winsys;
@@ -531,7 +537,7 @@ _cogl_winsys_context_create_gles2_context (CoglContext *ctx, GError **error)
                                   attribs);
   if (egl_context == EGL_NO_CONTEXT)
     {
-      g_set_error (error, COGL_WINSYS_ERROR,
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_CREATE_GLES2_CONTEXT,
                    "%s", get_error_string ());
       return NULL;
@@ -557,7 +563,7 @@ _cogl_winsys_destroy_gles2_context (CoglGLES2Context *gles2_ctx)
 
 static CoglBool
 _cogl_winsys_onscreen_init (CoglOnscreen *onscreen,
-                            GError **error)
+                            CoglError **error)
 {
   CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
   CoglContext *context = framebuffer->context;
@@ -569,14 +575,11 @@ _cogl_winsys_onscreen_init (CoglOnscreen *onscreen,
   EGLConfig egl_config;
   EGLint config_count = 0;
   EGLBoolean status;
-  CoglBool need_stencil =
-    egl_display->stencil_disabled ? FALSE : framebuffer->config.need_stencil;
 
   _COGL_RETURN_VAL_IF_FAIL (egl_display->egl_context, FALSE);
 
   egl_attributes_from_framebuffer_config (display,
                                           &framebuffer->config,
-                                          need_stencil,
                                           attributes);
 
   status = eglChooseConfig (egl_renderer->edpy,
@@ -585,7 +588,7 @@ _cogl_winsys_onscreen_init (CoglOnscreen *onscreen,
                             &config_count);
   if (status != EGL_TRUE || config_count == 0)
     {
-      g_set_error (error, COGL_WINSYS_ERROR,
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_CREATE_ONSCREEN,
                    "Failed to find a suitable EGL configuration");
       return FALSE;
@@ -779,7 +782,7 @@ _cogl_winsys_save_context (CoglContext *ctx)
 }
 
 static CoglBool
-_cogl_winsys_set_gles2_context (CoglGLES2Context *gles2_ctx, GError **error)
+_cogl_winsys_set_gles2_context (CoglGLES2Context *gles2_ctx, CoglError **error)
 {
   CoglContext *ctx = gles2_ctx->context;
   CoglDisplayEGL *egl_display = ctx->display->winsys;
@@ -798,7 +801,7 @@ _cogl_winsys_set_gles2_context (CoglGLES2Context *gles2_ctx, GError **error)
 
   if (!status)
     {
-      g_set_error (error,
+      _cogl_set_error (error,
                    COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_MAKE_CURRENT,
                    "Failed to make gles2 context current");
