@@ -48,6 +48,7 @@
 #include "cogl-xlib-renderer.h"
 #include "cogl-util.h"
 #include "cogl-winsys-glx-private.h"
+#include "cogl-error-private.h"
 
 #include <stdlib.h>
 #include <sys/types.h>
@@ -56,7 +57,6 @@
 
 #include <glib/gi18n-lib.h>
 
-#include <dlfcn.h>
 #include <GL/glx.h>
 #include <X11/Xlib.h>
 
@@ -97,13 +97,14 @@ typedef struct _CoglTexturePixmapGLX
 
 /* Define a set of arrays containing the functions required from GL
    for each winsys feature */
-#define COGL_WINSYS_FEATURE_BEGIN(name, namespaces, extension_names,    \
-                                  feature_flags, feature_flags_private, \
+#define COGL_WINSYS_FEATURE_BEGIN(major_version, minor_version,         \
+                                  name, namespaces, extension_names,    \
+                                  feature_flags,                        \
                                   winsys_feature)                       \
   static const CoglFeatureFunction                                      \
   cogl_glx_feature_ ## name ## _funcs[] = {
 #define COGL_WINSYS_FEATURE_FUNCTION(ret, name, args)                   \
-  { G_STRINGIFY (name), G_STRUCT_OFFSET (CoglGLXRenderer, pf_ ## name) },
+  { G_STRINGIFY (name), G_STRUCT_OFFSET (CoglGLXRenderer, name) },
 #define COGL_WINSYS_FEATURE_END()               \
   { NULL, 0 },                                  \
     };
@@ -111,11 +112,14 @@ typedef struct _CoglTexturePixmapGLX
 
 /* Define an array of features */
 #undef COGL_WINSYS_FEATURE_BEGIN
-#define COGL_WINSYS_FEATURE_BEGIN(name, namespaces, extension_names,    \
-                                  feature_flags, feature_flags_private, \
+#define COGL_WINSYS_FEATURE_BEGIN(major_version, minor_version,         \
+                                  name, namespaces, extension_names,    \
+                                  feature_flags,                        \
                                   winsys_feature)                       \
-  { 255, 255, 0, namespaces, extension_names,                           \
-      feature_flags, feature_flags_private,                             \
+  { major_version, minor_version,                                       \
+      0, namespaces, extension_names,                                   \
+      feature_flags,                                                    \
+      0,                                                                \
       winsys_feature, \
       cogl_glx_feature_ ## name ## _funcs },
 #undef COGL_WINSYS_FEATURE_FUNCTION
@@ -261,44 +265,16 @@ _cogl_winsys_renderer_disconnect (CoglRenderer *renderer)
 
 static CoglBool
 resolve_core_glx_functions (CoglRenderer *renderer,
-                            GError **error)
+                            CoglError **error)
 {
   CoglGLXRenderer *glx_renderer;
 
   glx_renderer = renderer->winsys;
 
-  if (!g_module_symbol (glx_renderer->libgl_module, "glXCreatePixmap",
-                        (void **) &glx_renderer->glXCreatePixmap) ||
-      !g_module_symbol (glx_renderer->libgl_module, "glXDestroyPixmap",
-                        (void **) &glx_renderer->glXDestroyPixmap) ||
-      !g_module_symbol (glx_renderer->libgl_module, "glXChooseFBConfig",
-                        (void **) &glx_renderer->glXChooseFBConfig) ||
-      !g_module_symbol (glx_renderer->libgl_module, "glXCreateNewContext",
-                        (void **) &glx_renderer->glXCreateNewContext) ||
-      !g_module_symbol (glx_renderer->libgl_module, "glXGetFBConfigAttrib",
-                        (void **) &glx_renderer->glXGetFBConfigAttrib) ||
+  if (!g_module_symbol (glx_renderer->libgl_module, "glXQueryExtension",
+                        (void **) &glx_renderer->glXQueryExtension) ||
       !g_module_symbol (glx_renderer->libgl_module, "glXQueryVersion",
                         (void **) &glx_renderer->glXQueryVersion) ||
-      !g_module_symbol (glx_renderer->libgl_module, "glXDestroyContext",
-                        (void **) &glx_renderer->glXDestroyContext) ||
-      !g_module_symbol (glx_renderer->libgl_module, "glXMakeContextCurrent",
-                        (void **) &glx_renderer->glXMakeContextCurrent) ||
-      !g_module_symbol (glx_renderer->libgl_module, "glXSwapBuffers",
-                        (void **) &glx_renderer->glXSwapBuffers) ||
-      !g_module_symbol (glx_renderer->libgl_module, "glXQueryExtension",
-                        (void **) &glx_renderer->glXQueryExtension) ||
-      !g_module_symbol (glx_renderer->libgl_module, "glXIsDirect",
-                        (void **) &glx_renderer->glXIsDirect) ||
-      !g_module_symbol (glx_renderer->libgl_module, "glXGetVisualFromFBConfig",
-                        (void **) &glx_renderer->glXGetVisualFromFBConfig) ||
-      !g_module_symbol (glx_renderer->libgl_module, "glXSelectEvent",
-                        (void **) &glx_renderer->glXSelectEvent) ||
-      !g_module_symbol (glx_renderer->libgl_module, "glXCreateWindow",
-                        (void **) &glx_renderer->glXCreateWindow) ||
-      !g_module_symbol (glx_renderer->libgl_module, "glXGetFBConfigs",
-                        (void **) &glx_renderer->glXGetFBConfigs) ||
-      !g_module_symbol (glx_renderer->libgl_module, "glXDestroyWindow",
-                        (void **) &glx_renderer->glXDestroyWindow) ||
       !g_module_symbol (glx_renderer->libgl_module, "glXQueryExtensionsString",
                         (void **) &glx_renderer->glXQueryExtensionsString) ||
       (!g_module_symbol (glx_renderer->libgl_module, "glXGetProcAddress",
@@ -306,7 +282,7 @@ resolve_core_glx_functions (CoglRenderer *renderer,
        !g_module_symbol (glx_renderer->libgl_module, "glXGetProcAddressARB",
                          (void **) &glx_renderer->glXGetProcAddress)))
     {
-      g_set_error (error, COGL_WINSYS_ERROR,
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_INIT,
                    "Failed to resolve required GLX symbol");
       return FALSE;
@@ -315,9 +291,69 @@ resolve_core_glx_functions (CoglRenderer *renderer,
   return TRUE;
 }
 
+static void
+update_base_winsys_features (CoglRenderer *renderer)
+{
+  CoglGLXRenderer *glx_renderer = renderer->winsys;
+  CoglXlibRenderer *xlib_renderer =
+    _cogl_xlib_renderer_get_data (renderer);
+  const char *glx_extensions;
+  int default_screen;
+  char **split_extensions;
+  int i;
+
+  default_screen = DefaultScreen (xlib_renderer->xdpy);
+  glx_extensions =
+    glx_renderer->glXQueryExtensionsString (xlib_renderer->xdpy,
+                                            default_screen);
+
+  COGL_NOTE (WINSYS, "  GLX Extensions: %s", glx_extensions);
+
+  split_extensions = g_strsplit (glx_extensions, " ", 0 /* max_tokens */);
+
+  for (i = 0; i < G_N_ELEMENTS (winsys_feature_data); i++)
+    if (_cogl_feature_check (renderer,
+                             "GLX", winsys_feature_data + i,
+                             glx_renderer->glx_major,
+                             glx_renderer->glx_minor,
+                             COGL_DRIVER_GL, /* the driver isn't used */
+                             split_extensions,
+                             glx_renderer))
+      {
+        glx_renderer->legacy_feature_flags |=
+          winsys_feature_data[i].feature_flags;
+        if (winsys_feature_data[i].winsys_feature)
+          COGL_FLAGS_SET (glx_renderer->base_winsys_features,
+                          winsys_feature_data[i].winsys_feature,
+                          TRUE);
+      }
+
+  g_strfreev (split_extensions);
+
+  /* Note: the GLX_SGI_video_sync spec explicitly states this extension
+   * only works for direct contexts. */
+  if (!glx_renderer->is_direct)
+    {
+      glx_renderer->glXGetVideoSync = NULL;
+      glx_renderer->glXWaitVideoSync = NULL;
+      COGL_FLAGS_SET (glx_renderer->base_winsys_features,
+                      COGL_WINSYS_FEATURE_VBLANK_COUNTER,
+                      FALSE);
+    }
+
+  COGL_FLAGS_SET (glx_renderer->base_winsys_features,
+                  COGL_WINSYS_FEATURE_MULTIPLE_ONSCREEN,
+                  TRUE);
+
+  if (glx_renderer->glXWaitVideoSync)
+    COGL_FLAGS_SET (glx_renderer->base_winsys_features,
+                    COGL_WINSYS_FEATURE_VBLANK_WAIT,
+                    TRUE);
+}
+
 static CoglBool
 _cogl_winsys_renderer_connect (CoglRenderer *renderer,
-                               GError **error)
+                               CoglError **error)
 {
   CoglGLXRenderer *glx_renderer;
   CoglXlibRenderer *xlib_renderer;
@@ -330,9 +366,10 @@ _cogl_winsys_renderer_connect (CoglRenderer *renderer,
   if (!_cogl_xlib_renderer_connect (renderer, error))
     goto error;
 
-  if (renderer->driver != COGL_DRIVER_GL)
+  if (renderer->driver != COGL_DRIVER_GL &&
+      renderer->driver != COGL_DRIVER_GL3)
     {
-      g_set_error (error, COGL_WINSYS_ERROR,
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_INIT,
                    "GLX Backend can only be used in conjunction with OpenGL");
       goto error;
@@ -343,7 +380,7 @@ _cogl_winsys_renderer_connect (CoglRenderer *renderer,
 
   if (glx_renderer->libgl_module == NULL)
     {
-      g_set_error (error, COGL_WINSYS_ERROR,
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_INIT,
                    "Failed to dynamically open the OpenGL library");
       goto error;
@@ -356,7 +393,7 @@ _cogl_winsys_renderer_connect (CoglRenderer *renderer,
                                         &glx_renderer->glx_error_base,
                                         &glx_renderer->glx_event_base))
     {
-      g_set_error (error, COGL_WINSYS_ERROR,
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_INIT,
                    "XServer appears to lack required GLX support");
       goto error;
@@ -370,12 +407,14 @@ _cogl_winsys_renderer_connect (CoglRenderer *renderer,
                                       &glx_renderer->glx_minor)
       || !(glx_renderer->glx_major == 1 && glx_renderer->glx_minor >= 2))
     {
-      g_set_error (error, COGL_WINSYS_ERROR,
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_INIT,
                    "XServer appears to lack required GLX 1.2 support");
       goto error;
     }
 
+  update_base_winsys_features (renderer);
+
   glx_renderer->dri_fd = -1;
 
   return TRUE;
@@ -386,68 +425,27 @@ error:
 }
 
 static CoglBool
-update_winsys_features (CoglContext *context, GError **error)
+update_winsys_features (CoglContext *context, CoglError **error)
 {
   CoglGLXDisplay *glx_display = context->display->winsys;
-  CoglXlibRenderer *xlib_renderer =
-    _cogl_xlib_renderer_get_data (context->display->renderer);
   CoglGLXRenderer *glx_renderer = context->display->renderer->winsys;
-  const char *glx_extensions;
-  int default_screen;
-  int i;
 
   _COGL_RETURN_VAL_IF_FAIL (glx_display->glx_context, FALSE);
 
   if (!_cogl_context_update_features (context, error))
     return FALSE;
 
-  memset (context->winsys_features, 0, sizeof (context->winsys_features));
+  memcpy (context->winsys_features,
+          glx_renderer->base_winsys_features,
+          sizeof (context->winsys_features));
 
-  default_screen = DefaultScreen (xlib_renderer->xdpy);
-  glx_extensions =
-    glx_renderer->glXQueryExtensionsString (xlib_renderer->xdpy,
-                                            default_screen);
-
-  COGL_NOTE (WINSYS, "  GLX Extensions: %s", glx_extensions);
+  context->feature_flags |= glx_renderer->legacy_feature_flags;
 
   context->feature_flags |= COGL_FEATURE_ONSCREEN_MULTIPLE;
   COGL_FLAGS_SET (context->features,
                   COGL_FEATURE_ID_ONSCREEN_MULTIPLE, TRUE);
-  COGL_FLAGS_SET (context->winsys_features,
-                  COGL_WINSYS_FEATURE_MULTIPLE_ONSCREEN,
-                  TRUE);
 
-  for (i = 0; i < G_N_ELEMENTS (winsys_feature_data); i++)
-    if (_cogl_feature_check (context->display->renderer,
-                             "GLX", winsys_feature_data + i, 0, 0,
-                             COGL_DRIVER_GL, /* the driver isn't used */
-                             glx_extensions,
-                             glx_renderer))
-      {
-        context->feature_flags |= winsys_feature_data[i].feature_flags;
-        if (winsys_feature_data[i].winsys_feature)
-          COGL_FLAGS_SET (context->winsys_features,
-                          winsys_feature_data[i].winsys_feature,
-                          TRUE);
-      }
-
-  /* Note: the GLX_SGI_video_sync spec explicitly states this extension
-   * only works for direct contexts. */
-  if (!glx_renderer->is_direct)
-    {
-      glx_renderer->pf_glXGetVideoSync = NULL;
-      glx_renderer->pf_glXWaitVideoSync = NULL;
-      COGL_FLAGS_SET (context->winsys_features,
-                      COGL_WINSYS_FEATURE_VBLANK_COUNTER,
-                      FALSE);
-    }
-
-  if (glx_renderer->pf_glXWaitVideoSync)
-    COGL_FLAGS_SET (context->winsys_features,
-                    COGL_WINSYS_FEATURE_VBLANK_WAIT,
-                    TRUE);
-
-  if (glx_renderer->pf_glXCopySubBuffer || context->glBlitFramebuffer)
+  if (glx_renderer->glXCopySubBuffer || context->glBlitFramebuffer)
     {
       CoglGpuInfoArchitecture arch;
 
@@ -540,7 +538,7 @@ static CoglBool
 find_fbconfig (CoglDisplay *display,
                CoglFramebufferConfig *config,
                GLXFBConfig *config_ret,
-               GError **error)
+               CoglError **error)
 {
   CoglXlibRenderer *xlib_renderer =
     _cogl_xlib_renderer_get_data (display->renderer);
@@ -559,7 +557,7 @@ find_fbconfig (CoglDisplay *display,
                                              &n_configs);
   if (!configs || n_configs == 0)
     {
-      g_set_error (error, COGL_WINSYS_ERROR,
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_CREATE_CONTEXT,
                    "Failed to find any compatible fbconfigs");
       ret = FALSE;
@@ -589,7 +587,7 @@ find_fbconfig (CoglDisplay *display,
             }
         }
 
-      g_set_error (error, COGL_WINSYS_ERROR,
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_CREATE_CONTEXT,
                    "Unable to find fbconfig with rgba visual");
       ret = FALSE;
@@ -606,8 +604,38 @@ done:
   return ret;
 }
 
+static GLXContext
+create_gl3_context (CoglDisplay *display,
+                    GLXFBConfig fb_config)
+{
+  CoglXlibRenderer *xlib_renderer =
+    _cogl_xlib_renderer_get_data (display->renderer);
+  CoglGLXRenderer *glx_renderer = display->renderer->winsys;
+
+  /* We want a core profile 3.1 context with no deprecated features */
+  static const int attrib_list[] =
+    {
+      GLX_CONTEXT_MAJOR_VERSION_ARB, 3,
+      GLX_CONTEXT_MINOR_VERSION_ARB, 1,
+      GLX_CONTEXT_PROFILE_MASK_ARB,  GLX_CONTEXT_CORE_PROFILE_BIT_ARB,
+      GLX_CONTEXT_FLAGS_ARB,         GLX_CONTEXT_FORWARD_COMPATIBLE_BIT_ARB,
+      None
+    };
+
+  /* Make sure that the display supports the GLX_ARB_create_context
+     extension */
+  if (glx_renderer->glXCreateContextAttribs == NULL)
+    return NULL;
+
+  return glx_renderer->glXCreateContextAttribs (xlib_renderer->xdpy,
+                                                fb_config,
+                                                NULL /* share_context */,
+                                                True, /* direct */
+                                                attrib_list);
+}
+
 static CoglBool
-create_context (CoglDisplay *display, GError **error)
+create_context (CoglDisplay *display, CoglError **error)
 {
   CoglGLXDisplay *glx_display = display->winsys;
   CoglXlibRenderer *xlib_renderer =
@@ -616,7 +644,7 @@ create_context (CoglDisplay *display, GError **error)
   CoglBool support_transparent_windows =
     display->onscreen_template->config.swap_chain->has_alpha;
   GLXFBConfig config;
-  GError *fbconfig_error = NULL;
+  CoglError *fbconfig_error = NULL;
   XSetWindowAttributes attrs;
   XVisualInfo *xvisinfo;
   GLXDrawable dummy_drawable;
@@ -629,11 +657,11 @@ create_context (CoglDisplay *display, GError **error)
                    &fbconfig_error);
   if (!glx_display->found_fbconfig)
     {
-      g_set_error (error, COGL_WINSYS_ERROR,
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_CREATE_CONTEXT,
                    "Unable to find suitable fbconfig for the GLX context: %s",
                    fbconfig_error->message);
-      g_error_free (fbconfig_error);
+      cogl_error_free (fbconfig_error);
       return FALSE;
     }
 
@@ -643,15 +671,19 @@ create_context (CoglDisplay *display, GError **error)
   COGL_NOTE (WINSYS, "Creating GLX Context (display: %p)",
              xlib_renderer->xdpy);
 
-  glx_display->glx_context =
-    glx_renderer->glXCreateNewContext (xlib_renderer->xdpy,
-                                       config,
-                                       GLX_RGBA_TYPE,
-                                       NULL,
-                                       True);
+  if (display->renderer->driver == COGL_DRIVER_GL3)
+    glx_display->glx_context = create_gl3_context (display, config);
+  else
+    glx_display->glx_context =
+      glx_renderer->glXCreateNewContext (xlib_renderer->xdpy,
+                                         config,
+                                         GLX_RGBA_TYPE,
+                                         NULL,
+                                         True);
+
   if (glx_display->glx_context == NULL)
     {
-      g_set_error (error, COGL_WINSYS_ERROR,
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_CREATE_CONTEXT,
                    "Unable to create suitable GL context");
       return FALSE;
@@ -672,7 +704,7 @@ create_context (CoglDisplay *display, GError **error)
                                                      config);
   if (xvisinfo == NULL)
     {
-      g_set_error (error, COGL_WINSYS_ERROR,
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_CREATE_CONTEXT,
                    "Unable to retrieve the X11 visual");
       return FALSE;
@@ -727,7 +759,7 @@ create_context (CoglDisplay *display, GError **error)
 
   if (_cogl_xlib_renderer_untrap_errors (display->renderer, &old_state))
     {
-      g_set_error (error, COGL_WINSYS_ERROR,
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_CREATE_CONTEXT,
                    "Unable to select the newly created GLX context");
       return FALSE;
@@ -774,7 +806,7 @@ _cogl_winsys_display_destroy (CoglDisplay *display)
 
 static CoglBool
 _cogl_winsys_display_setup (CoglDisplay *display,
-                            GError **error)
+                            CoglError **error)
 {
   CoglGLXDisplay *glx_display;
   int i;
@@ -798,7 +830,7 @@ error:
 }
 
 static CoglBool
-_cogl_winsys_context_init (CoglContext *context, GError **error)
+_cogl_winsys_context_init (CoglContext *context, CoglError **error)
 {
   context->winsys = g_new0 (CoglContextGLX, 1);
 
@@ -819,7 +851,7 @@ _cogl_winsys_context_deinit (CoglContext *context)
 
 static CoglBool
 _cogl_winsys_onscreen_init (CoglOnscreen *onscreen,
-                            GError **error)
+                            CoglError **error)
 {
   CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
   CoglContext *context = framebuffer->context;
@@ -832,7 +864,7 @@ _cogl_winsys_onscreen_init (CoglOnscreen *onscreen,
   CoglOnscreenXlib *xlib_onscreen;
   CoglOnscreenGLX *glx_onscreen;
   GLXFBConfig fbconfig;
-  GError *fbconfig_error = NULL;
+  CoglError *fbconfig_error = NULL;
 
   _COGL_RETURN_VAL_IF_FAIL (glx_display->glx_context, FALSE);
 
@@ -840,11 +872,11 @@ _cogl_winsys_onscreen_init (CoglOnscreen *onscreen,
                       &fbconfig,
                       &fbconfig_error))
     {
-      g_set_error (error, COGL_WINSYS_ERROR,
+      _cogl_set_error (error, COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_CREATE_CONTEXT,
                    "Unable to find suitable fbconfig for the GLX context: %s",
                    fbconfig_error->message);
-      g_error_free (fbconfig_error);
+      cogl_error_free (fbconfig_error);
       return FALSE;
     }
 
@@ -888,7 +920,7 @@ _cogl_winsys_onscreen_init (CoglOnscreen *onscreen,
         {
           char message[1000];
           XGetErrorText (xlib_renderer->xdpy, xerror, message, sizeof(message));
-          g_set_error (error, COGL_WINSYS_ERROR,
+          _cogl_set_error (error, COGL_WINSYS_ERROR,
                        COGL_WINSYS_ERROR_CREATE_ONSCREEN,
                        "Unable to query geometry of foreign xid 0x%08lX: %s",
                        xwin, message);
@@ -922,7 +954,7 @@ _cogl_winsys_onscreen_init (CoglOnscreen *onscreen,
                                                          fbconfig);
       if (xvisinfo == NULL)
         {
-          g_set_error (error, COGL_WINSYS_ERROR,
+          _cogl_set_error (error, COGL_WINSYS_ERROR,
                        COGL_WINSYS_ERROR_CREATE_ONSCREEN,
                        "Unable to retrieve the X11 visual of context's "
                        "fbconfig");
@@ -961,7 +993,7 @@ _cogl_winsys_onscreen_init (CoglOnscreen *onscreen,
           char message[1000];
           XGetErrorText (xlib_renderer->xdpy, xerror,
                          message, sizeof (message));
-          g_set_error (error, COGL_WINSYS_ERROR,
+          _cogl_set_error (error, COGL_WINSYS_ERROR,
                        COGL_WINSYS_ERROR_CREATE_ONSCREEN,
                        "X error while creating Window for CoglOnscreen: %s",
                        message);
@@ -1102,18 +1134,18 @@ _cogl_winsys_onscreen_bind (CoglOnscreen *onscreen)
    * MESA and SGI extensions which should technically be mutually
    * exclusive.
    */
-  if (glx_renderer->pf_glXSwapInterval)
+  if (glx_renderer->glXSwapInterval)
     {
       CoglFramebuffer *fb = COGL_FRAMEBUFFER (onscreen);
       if (fb->config.swap_throttled)
-        glx_renderer->pf_glXSwapInterval (1);
+        glx_renderer->glXSwapInterval (1);
       else
-        glx_renderer->pf_glXSwapInterval (0);
+        glx_renderer->glXSwapInterval (0);
     }
 
   XSync (xlib_renderer->xdpy, False);
 
-  /* FIXME: We should be reporting a GError here
+  /* FIXME: We should be reporting a CoglError here
    */
   if (_cogl_xlib_renderer_untrap_errors (context->display->renderer,
                                          &old_state))
@@ -1127,36 +1159,32 @@ _cogl_winsys_onscreen_bind (CoglOnscreen *onscreen)
 }
 
 static void
-_cogl_winsys_wait_for_vblank (void)
+_cogl_winsys_wait_for_vblank (CoglContext *ctx)
 {
   CoglGLXRenderer *glx_renderer;
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
   glx_renderer = ctx->display->renderer->winsys;
 
-  if (glx_renderer->pf_glXGetVideoSync)
+  if (glx_renderer->glXGetVideoSync)
     {
       uint32_t current_count;
 
-      glx_renderer->pf_glXGetVideoSync (&current_count);
-      glx_renderer->pf_glXWaitVideoSync (2,
-                                         (current_count + 1) % 2,
-                                         &current_count);
+      glx_renderer->glXGetVideoSync (&current_count);
+      glx_renderer->glXWaitVideoSync (2,
+                                      (current_count + 1) % 2,
+                                      &current_count);
     }
 }
 
 static uint32_t
-_cogl_winsys_get_vsync_counter (void)
+_cogl_winsys_get_vsync_counter (CoglContext *ctx)
 {
   uint32_t video_sync_count;
   CoglGLXRenderer *glx_renderer;
 
-  _COGL_GET_CONTEXT (ctx, 0);
-
   glx_renderer = ctx->display->renderer->winsys;
 
-  glx_renderer->pf_glXGetVideoSync (&video_sync_count);
+  glx_renderer->glXGetVideoSync (&video_sync_count);
 
   return video_sync_count;
 }
@@ -1257,27 +1285,27 @@ _cogl_winsys_onscreen_swap_region (CoglOnscreen *onscreen,
 
   if (blit_sub_buffer_is_synchronized && have_counter && can_wait)
     {
-      end_frame_vsync_counter = _cogl_winsys_get_vsync_counter ();
+      end_frame_vsync_counter = _cogl_winsys_get_vsync_counter (context);
 
       /* If we have the GLX_SGI_video_sync extension then we can
        * be a bit smarter about how we throttle blits by avoiding
        * any waits if we can see that the video sync count has
        * already progressed. */
       if (glx_onscreen->last_swap_vsync_counter == end_frame_vsync_counter)
-        _cogl_winsys_wait_for_vblank ();
+        _cogl_winsys_wait_for_vblank (context);
     }
   else if (can_wait)
-    _cogl_winsys_wait_for_vblank ();
+    _cogl_winsys_wait_for_vblank (context);
 
-  if (glx_renderer->pf_glXCopySubBuffer)
+  if (glx_renderer->glXCopySubBuffer)
     {
       Display *xdpy = xlib_renderer->xdpy;
       int i;
       for (i = 0; i < n_rectangles; i++)
         {
           int *rect = &rectangles[4 * i];
-          glx_renderer->pf_glXCopySubBuffer (xdpy, drawable,
-                                             rect[0], rect[1], rect[2], rect[3]);
+          glx_renderer->glXCopySubBuffer (xdpy, drawable,
+                                          rect[0], rect[1], rect[2], rect[3]);
         }
     }
   else if (context->glBlitFramebuffer)
@@ -1285,6 +1313,15 @@ _cogl_winsys_onscreen_swap_region (CoglOnscreen *onscreen,
       int i;
       /* XXX: checkout how this state interacts with the code to use
        * glBlitFramebuffer in Neil's texture atlasing branch */
+
+      /* glBlitFramebuffer is affected by the scissor so we need to
+       * ensure we have flushed an empty clip stack to get rid of it.
+       * We also mark that the clip state is dirty so that it will be
+       * flushed to the correct state the next time something is
+       * drawn */
+      _cogl_clip_stack_flush (NULL, framebuffer);
+      context->current_draw_buffer_changes |= COGL_FRAMEBUFFER_STATE_CLIP;
+
       context->glDrawBuffer (GL_FRONT);
       for (i = 0; i < n_rectangles; i++)
         {
@@ -1352,9 +1389,9 @@ _cogl_winsys_onscreen_swap_buffers (CoglOnscreen *onscreen)
        * the vsync counter for each swap request so we can manually
        * throttle swap_region requests. */
       if (have_counter)
-        end_frame_vsync_counter = _cogl_winsys_get_vsync_counter ();
+        end_frame_vsync_counter = _cogl_winsys_get_vsync_counter (context);
 
-      if (!glx_renderer->pf_glXSwapInterval)
+      if (!glx_renderer->glXSwapInterval)
         {
           CoglBool can_wait =
             _cogl_winsys_has_feature (COGL_WINSYS_FEATURE_VBLANK_WAIT);
@@ -1381,10 +1418,10 @@ _cogl_winsys_onscreen_swap_buffers (CoglOnscreen *onscreen)
             {
               if (glx_onscreen->last_swap_vsync_counter ==
                   end_frame_vsync_counter)
-                _cogl_winsys_wait_for_vblank ();
+                _cogl_winsys_wait_for_vblank (context);
             }
           else if (can_wait)
-            _cogl_winsys_wait_for_vblank ();
+            _cogl_winsys_wait_for_vblank (context);
         }
     }
   else
@@ -1393,7 +1430,8 @@ _cogl_winsys_onscreen_swap_buffers (CoglOnscreen *onscreen)
   glx_renderer->glXSwapBuffers (xlib_renderer->xdpy, drawable);
 
   if (have_counter)
-    glx_onscreen->last_swap_vsync_counter = _cogl_winsys_get_vsync_counter ();
+    glx_onscreen->last_swap_vsync_counter =
+      _cogl_winsys_get_vsync_counter (context);
 }
 
 static uint32_t
@@ -1786,10 +1824,7 @@ static CoglBool
 _cogl_winsys_texture_pixmap_x11_create (CoglTexturePixmapX11 *tex_pixmap)
 {
   CoglTexturePixmapGLX *glx_tex_pixmap;
-
-  /* FIXME: It should be possible to get to a CoglContext from any
-   * CoglTexture pointer. */
-  _COGL_GET_CONTEXT (ctx, FALSE);
+  CoglContext *ctx = COGL_TEXTURE (tex_pixmap)->context;
 
   if (!_cogl_winsys_has_feature (COGL_WINSYS_FEATURE_TEXTURE_FROM_PIXMAP))
     {
@@ -1834,9 +1869,9 @@ free_glx_pixmap (CoglContext *context,
   glx_renderer = renderer->winsys;
 
   if (glx_tex_pixmap->pixmap_bound)
-    glx_renderer->pf_glXReleaseTexImage (xlib_renderer->xdpy,
-                                         glx_tex_pixmap->glx_pixmap,
-                                         GLX_FRONT_LEFT_EXT);
+    glx_renderer->glXReleaseTexImage (xlib_renderer->xdpy,
+                                      glx_tex_pixmap->glx_pixmap,
+                                      GLX_FRONT_LEFT_EXT);
 
   /* FIXME - we need to trap errors and synchronize here because
    * of ordering issues between the XPixmap destruction and the
@@ -1869,16 +1904,12 @@ _cogl_winsys_texture_pixmap_x11_free (CoglTexturePixmapX11 *tex_pixmap)
 {
   CoglTexturePixmapGLX *glx_tex_pixmap;
 
-  /* FIXME: It should be possible to get to a CoglContext from any
-   * CoglTexture pointer. */
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
   if (!tex_pixmap->winsys)
     return;
 
   glx_tex_pixmap = tex_pixmap->winsys;
 
-  free_glx_pixmap (ctx, glx_tex_pixmap);
+  free_glx_pixmap (COGL_TEXTURE (tex_pixmap)->context, glx_tex_pixmap);
 
   if (glx_tex_pixmap->glx_tex)
     cogl_object_unref (glx_tex_pixmap->glx_tex);
@@ -1891,13 +1922,11 @@ static CoglBool
 _cogl_winsys_texture_pixmap_x11_update (CoglTexturePixmapX11 *tex_pixmap,
                                         CoglBool needs_mipmap)
 {
+  CoglTexture *tex = COGL_TEXTURE (tex_pixmap);
+  CoglContext *ctx = COGL_TEXTURE (tex_pixmap)->context;
   CoglTexturePixmapGLX *glx_tex_pixmap = tex_pixmap->winsys;
   CoglGLXRenderer *glx_renderer;
 
-  /* FIXME: It should be possible to get to a CoglContext from any CoglTexture
-   * pointer. */
-  _COGL_GET_CONTEXT (ctx, FALSE);
-
   /* If we don't have a GLX pixmap then fallback */
   if (glx_tex_pixmap->glx_pixmap == None)
     return FALSE;
@@ -1908,7 +1937,7 @@ _cogl_winsys_texture_pixmap_x11_update (CoglTexturePixmapX11 *tex_pixmap,
   if (glx_tex_pixmap->glx_tex == NULL)
     {
       CoglPixelFormat texture_format;
-      GError *error = NULL;
+      CoglError *error = NULL;
 
       texture_format = (tex_pixmap->depth >= 32 ?
                         COGL_PIXEL_FORMAT_RGBA_8888_PRE :
@@ -1918,8 +1947,8 @@ _cogl_winsys_texture_pixmap_x11_update (CoglTexturePixmapX11 *tex_pixmap,
         {
           glx_tex_pixmap->glx_tex = COGL_TEXTURE (
             cogl_texture_rectangle_new_with_size (ctx,
-                                                  tex_pixmap->width,
-                                                  tex_pixmap->height,
+                                                  tex->width,
+                                                  tex->height,
                                                   texture_format,
                                                   &error));
 
@@ -1931,7 +1960,7 @@ _cogl_winsys_texture_pixmap_x11_update (CoglTexturePixmapX11 *tex_pixmap,
               COGL_NOTE (TEXTURE_PIXMAP, "Falling back for %p because a "
                          "texture rectangle could not be created: %s",
                          tex_pixmap, error->message);
-              g_error_free (error);
+              cogl_error_free (error);
               free_glx_pixmap (ctx, glx_tex_pixmap);
               return FALSE;
             }
@@ -1940,19 +1969,19 @@ _cogl_winsys_texture_pixmap_x11_update (CoglTexturePixmapX11 *tex_pixmap,
         {
           glx_tex_pixmap->glx_tex = COGL_TEXTURE (
             cogl_texture_2d_new_with_size (ctx,
-                                           tex_pixmap->width,
-                                           tex_pixmap->height,
-                                           texture_format,
-                                           NULL));
+                                           tex->width,
+                                           tex->height,
+                                           texture_format));
 
-          if (glx_tex_pixmap->glx_tex)
+          if (cogl_texture_allocate (glx_tex_pixmap->glx_tex, &error))
             COGL_NOTE (TEXTURE_PIXMAP, "Created a texture 2d for %p",
                        tex_pixmap);
           else
             {
               COGL_NOTE (TEXTURE_PIXMAP, "Falling back for %p because a "
-                         "texture 2d could not be created",
-                         tex_pixmap);
+                         "texture 2d could not be created: %s",
+                         tex_pixmap, error->message);
+              cogl_error_free (error);
               free_glx_pixmap (ctx, glx_tex_pixmap);
               return FALSE;
             }
@@ -2005,14 +2034,14 @@ _cogl_winsys_texture_pixmap_x11_update (CoglTexturePixmapX11 *tex_pixmap,
       _cogl_bind_gl_texture_transient (gl_target, gl_handle, FALSE);
 
       if (glx_tex_pixmap->pixmap_bound)
-        glx_renderer->pf_glXReleaseTexImage (xlib_renderer->xdpy,
-                                             glx_tex_pixmap->glx_pixmap,
-                                             GLX_FRONT_LEFT_EXT);
+        glx_renderer->glXReleaseTexImage (xlib_renderer->xdpy,
+                                          glx_tex_pixmap->glx_pixmap,
+                                          GLX_FRONT_LEFT_EXT);
 
-      glx_renderer->pf_glXBindTexImage (xlib_renderer->xdpy,
-                                        glx_tex_pixmap->glx_pixmap,
-                                        GLX_FRONT_LEFT_EXT,
-                                        NULL);
+      glx_renderer->glXBindTexImage (xlib_renderer->xdpy,
+                                     glx_tex_pixmap->glx_pixmap,
+                                     GLX_FRONT_LEFT_EXT,
+                                     NULL);
 
       /* According to the recommended usage in the spec for
        * GLX_EXT_texture_pixmap we should release the texture after
