@@ -1,5 +1,4 @@
 #include <cogl/cogl.h>
-#include <glib.h>
 #include <stdio.h>
 #include <SDL.h>
 
@@ -77,7 +76,7 @@ main (int argc, char **argv)
 {
   CoglContext *ctx;
   CoglOnscreen *onscreen;
-  GError *error = NULL;
+  CoglError *error = NULL;
   CoglVertexP2C4 triangle_vertices[] = {
     {0, 0.7, 0xff, 0x00, 0x00, 0x80},
     {-0.7, -0.7, 0x00, 0xff, 0x00, 0xff},
@@ -100,6 +99,10 @@ main (int argc, char **argv)
   data.center_y = 0.0f;
   data.quit = FALSE;
 
+  /* In SDL2, setting resizable only works before allocating the
+   * onscreen */
+  cogl_onscreen_set_resizable (onscreen, TRUE);
+
   cogl_onscreen_show (onscreen);
 
   data.triangle = cogl_primitive_new_p2c4 (ctx, COGL_VERTICES_MODE_TRIANGLES,
