@@ -21,20 +21,19 @@ test_push_pop_single_context (void)
   CoglPipeline *pipeline;
   CoglGLES2Context *gles2_ctx;
   const CoglGLES2Vtable *gles2;
-  GError *error = NULL;
+  CoglError *error = NULL;
 
   offscreen_texture = COGL_TEXTURE (
-    cogl_texture_2d_new_with_size (ctx,
-                                   cogl_framebuffer_get_width (fb),
-                                   cogl_framebuffer_get_height (fb),
-                                   COGL_PIXEL_FORMAT_ANY,
-                                   NULL));
+    cogl_texture_2d_new_with_size (test_ctx,
+                                   cogl_framebuffer_get_width (test_fb),
+                                   cogl_framebuffer_get_height (test_fb),
+                                   COGL_PIXEL_FORMAT_ANY));
   offscreen = cogl_offscreen_new_to_texture (offscreen_texture);
 
-  pipeline = cogl_pipeline_new (ctx);
+  pipeline = cogl_pipeline_new (test_ctx);
   cogl_pipeline_set_layer_texture (pipeline, 0, offscreen_texture);
 
-  gles2_ctx = cogl_gles2_context_new (ctx, &error);
+  gles2_ctx = cogl_gles2_context_new (test_ctx, &error);
   if (!gles2_ctx)
     g_error ("Failed to create GLES2 context: %s\n", error->message);
 
@@ -42,10 +41,10 @@ test_push_pop_single_context (void)
 
   /* Clear onscreen to 0xffff00 using GLES2 */
 
-  if (!cogl_push_gles2_context (ctx,
+  if (!cogl_push_gles2_context (test_ctx,
                                 gles2_ctx,
-                                fb,
-                                fb,
+                                test_fb,
+                                test_fb,
                                 &error))
     {
       g_error ("Failed to push gles2 context: %s\n", error->message);
@@ -54,9 +53,9 @@ test_push_pop_single_context (void)
   gles2->glClearColor (1, 1, 0, 1);
   gles2->glClear (GL_COLOR_BUFFER_BIT);
 
-  cogl_pop_gles2_context (ctx);
+  cogl_pop_gles2_context (test_ctx);
 
-  test_utils_check_pixel (fb, 0, 0, 0xffff00ff);
+  test_utils_check_pixel (test_fb, 0, 0, 0xffff00ff);
 
   /* Clear offscreen to 0xff0000 using GLES2 and then copy the result
    * onscreen.
@@ -66,7 +65,7 @@ test_push_pop_single_context (void)
    * instead.
    */
 
-  if (!cogl_push_gles2_context (ctx,
+  if (!cogl_push_gles2_context (test_ctx,
                                 gles2_ctx,
                                 COGL_FRAMEBUFFER (offscreen),
                                 COGL_FRAMEBUFFER (offscreen),
@@ -78,9 +77,9 @@ test_push_pop_single_context (void)
   gles2->glClearColor (1, 0, 0, 1);
   gles2->glClear (GL_COLOR_BUFFER_BIT);
 
-  cogl_pop_gles2_context (ctx);
+  cogl_pop_gles2_context (test_ctx);
 
-  cogl_framebuffer_draw_rectangle (fb,
+  cogl_framebuffer_draw_rectangle (test_fb,
                                    pipeline,
                                    -1, 1, 1, -1);
   /* NB: Cogl doesn't automatically support mid-scene modifications
@@ -93,7 +92,7 @@ test_push_pop_single_context (void)
    * reading back from the onscreen framebuffer in case we mistakenly
    * read from the offscreen framebuffer and get a false positive
    */
-  if (!cogl_push_gles2_context (ctx,
+  if (!cogl_push_gles2_context (test_ctx,
                                 gles2_ctx,
                                 COGL_FRAMEBUFFER (offscreen),
                                 COGL_FRAMEBUFFER (offscreen),
@@ -105,22 +104,22 @@ test_push_pop_single_context (void)
   gles2->glClearColor (0, 0, 1, 1);
   gles2->glClear (GL_COLOR_BUFFER_BIT);
 
-  cogl_pop_gles2_context (ctx);
+  cogl_pop_gles2_context (test_ctx);
 
-  test_utils_check_pixel (fb, 0, 0, 0xff0000ff);
+  test_utils_check_pixel (test_fb, 0, 0, 0xff0000ff);
 
   /* Now copy the offscreen blue clear to the onscreen framebufer and
    * check that too */
-  cogl_framebuffer_draw_rectangle (fb,
+  cogl_framebuffer_draw_rectangle (test_fb,
                                    pipeline,
                                    -1, 1, 1, -1);
 
-  test_utils_check_pixel (fb, 0, 0, 0x0000ffff);
+  test_utils_check_pixel (test_fb, 0, 0, 0x0000ffff);
 
-  if (!cogl_push_gles2_context (ctx,
+  if (!cogl_push_gles2_context (test_ctx,
                                 gles2_ctx,
-                                fb,
-                                fb,
+                                test_fb,
+                                test_fb,
                                 &error))
     {
       g_error ("Failed to push gles2 context: %s\n", error->message);
@@ -129,9 +128,9 @@ test_push_pop_single_context (void)
   gles2->glClearColor (1, 0, 1, 1);
   gles2->glClear (GL_COLOR_BUFFER_BIT);
 
-  cogl_pop_gles2_context (ctx);
+  cogl_pop_gles2_context (test_ctx);
 
-  test_utils_check_pixel (fb, 0, 0, 0xff00ffff);
+  test_utils_check_pixel (test_fb, 0, 0, 0xff00ffff);
 
 
   cogl_object_unref (gles2_ctx);
@@ -146,20 +145,19 @@ create_gles2_context (CoglTexture **offscreen_texture,
                       CoglGLES2Context **gles2_ctx,
                       const CoglGLES2Vtable **gles2)
 {
-  GError *error = NULL;
+  CoglError *error = NULL;
 
   *offscreen_texture = COGL_TEXTURE (
-    cogl_texture_2d_new_with_size (ctx,
-                                   cogl_framebuffer_get_width (fb),
-                                   cogl_framebuffer_get_height (fb),
-                                   COGL_PIXEL_FORMAT_ANY,
-                                   NULL));
+    cogl_texture_2d_new_with_size (test_ctx,
+                                   cogl_framebuffer_get_width (test_fb),
+                                   cogl_framebuffer_get_height (test_fb),
+                                   COGL_PIXEL_FORMAT_ANY));
   *offscreen = cogl_offscreen_new_to_texture (*offscreen_texture);
 
-  *pipeline = cogl_pipeline_new (ctx);
+  *pipeline = cogl_pipeline_new (test_ctx);
   cogl_pipeline_set_layer_texture (*pipeline, 0, *offscreen_texture);
 
-  *gles2_ctx = cogl_gles2_context_new (ctx, &error);
+  *gles2_ctx = cogl_gles2_context_new (test_ctx, &error);
   if (!*gles2_ctx)
     g_error ("Failed to create GLES2 context: %s\n", error->message);
 
@@ -179,7 +177,7 @@ test_push_pop_multi_context (void)
   CoglPipeline *pipeline1;
   CoglGLES2Context *gles2_ctx1;
   const CoglGLES2Vtable *gles21;
-  GError *error = NULL;
+  CoglError *error = NULL;
 
   create_gles2_context (&offscreen_texture0,
                         &offscreen0,
@@ -193,9 +191,9 @@ test_push_pop_multi_context (void)
                         &gles2_ctx1,
                         &gles21);
 
-  cogl_framebuffer_clear4f (fb, COGL_BUFFER_BIT_COLOR, 1, 1, 1, 1);
+  cogl_framebuffer_clear4f (test_fb, COGL_BUFFER_BIT_COLOR, 1, 1, 1, 1);
 
-  if (!cogl_push_gles2_context (ctx,
+  if (!cogl_push_gles2_context (test_ctx,
                                 gles2_ctx0,
                                 COGL_FRAMEBUFFER (offscreen0),
                                 COGL_FRAMEBUFFER (offscreen0),
@@ -207,7 +205,7 @@ test_push_pop_multi_context (void)
   gles20->glClearColor (1, 0, 0, 1);
   gles20->glClear (GL_COLOR_BUFFER_BIT);
 
-  if (!cogl_push_gles2_context (ctx,
+  if (!cogl_push_gles2_context (test_ctx,
                                 gles2_ctx1,
                                 COGL_FRAMEBUFFER (offscreen1),
                                 COGL_FRAMEBUFFER (offscreen1),
@@ -219,22 +217,22 @@ test_push_pop_multi_context (void)
   gles21->glClearColor (0, 1, 0, 1);
   gles21->glClear (GL_COLOR_BUFFER_BIT);
 
-  cogl_pop_gles2_context (ctx);
-  cogl_pop_gles2_context (ctx);
+  cogl_pop_gles2_context (test_ctx);
+  cogl_pop_gles2_context (test_ctx);
 
-  test_utils_check_pixel (fb, 0, 0, 0xffffffff);
+  test_utils_check_pixel (test_fb, 0, 0, 0xffffffff);
 
-  cogl_framebuffer_draw_rectangle (fb,
+  cogl_framebuffer_draw_rectangle (test_fb,
                                    pipeline0,
                                    -1, 1, 1, -1);
 
-  test_utils_check_pixel (fb, 0, 0, 0xff0000ff);
+  test_utils_check_pixel (test_fb, 0, 0, 0xff0000ff);
 
-  cogl_framebuffer_draw_rectangle (fb,
+  cogl_framebuffer_draw_rectangle (test_fb,
                                    pipeline1,
                                    -1, 1, 1, -1);
 
-  test_utils_check_pixel (fb, 0, 0, 0x00ff00ff);
+  test_utils_check_pixel (test_fb, 0, 0, 0x00ff00ff);
 }
 
 static GLuint
@@ -278,7 +276,7 @@ test_gles2_read_pixels (void)
   CoglPipeline *pipeline;
   CoglGLES2Context *gles2_ctx;
   const CoglGLES2Vtable *gles2;
-  GError *error = NULL;
+  CoglError *error = NULL;
   GLubyte pixel[3];
   GLuint fbo_handle;
 
@@ -288,9 +286,9 @@ test_gles2_read_pixels (void)
                         &gles2_ctx,
                         &gles2);
 
-  cogl_framebuffer_clear4f (fb, COGL_BUFFER_BIT_COLOR, 1, 1, 1, 1);
+  cogl_framebuffer_clear4f (test_fb, COGL_BUFFER_BIT_COLOR, 1, 1, 1, 1);
 
-  if (!cogl_push_gles2_context (ctx,
+  if (!cogl_push_gles2_context (test_ctx,
                                 gles2_ctx,
                                 COGL_FRAMEBUFFER (offscreen),
                                 COGL_FRAMEBUFFER (offscreen),
@@ -329,15 +327,15 @@ test_gles2_read_pixels (void)
   g_assert (pixel[1] == 0xff);
   g_assert (pixel[2] == 0xff);
 
-  cogl_pop_gles2_context (ctx);
+  cogl_pop_gles2_context (test_ctx);
 
-  test_utils_check_pixel (fb, 0, 0, 0xffffffff);
+  test_utils_check_pixel (test_fb, 0, 0, 0xffffffff);
 
   /* Bind different read and write buffers */
-  if (!cogl_push_gles2_context (ctx,
+  if (!cogl_push_gles2_context (test_ctx,
                                 gles2_ctx,
                                 COGL_FRAMEBUFFER (offscreen),
-                                fb,
+                                test_fb,
                                 &error))
     {
       g_error ("Failed to push gles2 context: %s\n", error->message);
@@ -349,16 +347,16 @@ test_gles2_read_pixels (void)
   g_assert (pixel[1] == 0xff);
   g_assert (pixel[2] == 0xff);
 
-  cogl_pop_gles2_context (ctx);
+  cogl_pop_gles2_context (test_ctx);
 
-  test_utils_check_pixel (fb, 0, 0, 0xffffffff);
+  test_utils_check_pixel (test_fb, 0, 0, 0xffffffff);
 
   /* Bind different read and write buffers (the other way around from
    * before so when we test with COGL_TEST_ONSCREEN=1 we will read
    * from an onscreen framebuffer) */
-  if (!cogl_push_gles2_context (ctx,
+  if (!cogl_push_gles2_context (test_ctx,
                                 gles2_ctx,
-                                fb,
+                                test_fb,
                                 COGL_FRAMEBUFFER (offscreen),
                                 &error))
     {
@@ -371,7 +369,7 @@ test_gles2_read_pixels (void)
   g_assert (pixel[1] == 0xff);
   g_assert (pixel[2] == 0xff);
 
-  cogl_pop_gles2_context (ctx);
+  cogl_pop_gles2_context (test_ctx);
 }
 
 void
@@ -674,8 +672,8 @@ test_gles2_context_fbo (void)
   int i;
   PaintData data;
 
-  data.fb_width = cogl_framebuffer_get_width (fb);
-  data.fb_height = cogl_framebuffer_get_height (fb);
+  data.fb_width = cogl_framebuffer_get_width (test_fb);
+  data.fb_height = cogl_framebuffer_get_height (test_fb);
 
   for (i = 0; i < G_N_ELEMENTS (paint_methods); i++)
     {
@@ -684,7 +682,7 @@ test_gles2_context_fbo (void)
       CoglPipeline *pipeline;
       CoglGLES2Context *gles2_ctx;
       GLuint program;
-      GError *error = NULL;
+      CoglError *error = NULL;
 
       create_gles2_context (&offscreen_texture,
                             &offscreen,
@@ -692,7 +690,7 @@ test_gles2_context_fbo (void)
                             &gles2_ctx,
                             &data.gles2);
 
-      if (!cogl_push_gles2_context (ctx,
+      if (!cogl_push_gles2_context (test_ctx,
                                     gles2_ctx,
                                     COGL_FRAMEBUFFER (offscreen),
                                     COGL_FRAMEBUFFER (offscreen),
@@ -720,12 +718,12 @@ test_gles2_context_fbo (void)
 
       verify_read_pixels (&data);
 
-      cogl_pop_gles2_context (ctx);
+      cogl_pop_gles2_context (test_ctx);
 
       cogl_object_unref (offscreen);
       cogl_object_unref (gles2_ctx);
 
-      cogl_framebuffer_draw_rectangle (fb,
+      cogl_framebuffer_draw_rectangle (test_fb,
                                        pipeline,
                                        -1.0f, 1.0f,
                                        1.0f, -1.0f);
@@ -734,11 +732,11 @@ test_gles2_context_fbo (void)
       cogl_object_unref (offscreen_texture);
 
       /* Top half of the framebuffer should be red */
-      test_utils_check_pixel (fb,
+      test_utils_check_pixel (test_fb,
                               data.fb_width / 2, data.fb_height / 4,
                               0xff0000ff);
       /* Bottom half should be blue */
-      test_utils_check_pixel (fb,
+      test_utils_check_pixel (test_fb,
                               data.fb_width / 2, data.fb_height * 3 / 4,
                               0x0000ffff);
     }
@@ -807,14 +805,14 @@ test_gles2_context_copy_tex_image (void)
       -1.0f, 1.0f, 0.0f, 1.0f,
       1.0f, 1.0f, 1.0f, 1.0f
     };
-  int fb_width = cogl_framebuffer_get_width (fb);
-  int fb_height = cogl_framebuffer_get_height (fb);
+  int fb_width = cogl_framebuffer_get_width (test_fb);
+  int fb_height = cogl_framebuffer_get_height (test_fb);
   CoglTexture *offscreen_texture;
   CoglOffscreen *offscreen;
   CoglPipeline *pipeline;
   CoglGLES2Context *gles2_ctx;
   const CoglGLES2Vtable *gles2;
-  GError *error = NULL;
+  CoglError *error = NULL;
   GLuint tex;
   GLint tex_uniform_location;
   GLint pos_location;
@@ -827,7 +825,7 @@ test_gles2_context_copy_tex_image (void)
                         &gles2_ctx,
                         &gles2);
 
-  if (!cogl_push_gles2_context (ctx,
+  if (!cogl_push_gles2_context (test_ctx,
                                 gles2_ctx,
                                 COGL_FRAMEBUFFER (offscreen),
                                 COGL_FRAMEBUFFER (offscreen),
@@ -967,7 +965,7 @@ test_gles2_context_copy_tex_image (void)
                  RECTANGLE_HEIGHT / 2,
                  0x0000ffff);
 
-  cogl_pop_gles2_context (ctx);
+  cogl_pop_gles2_context (test_ctx);
 
   cogl_object_unref (offscreen);
   cogl_object_unref (gles2_ctx);
